#pragma once

#include <string>
#include <sstream>
#include <stdio.h>
#include <utility>
#include <stdlib.h>


#include "SList.h"
#include "Vector.h"
#include "HashMap.h"

#include "RTTI.h"
#include "Scope.h"
#include "Attributed.h"

#include "IXmlParseHelper.h"

#include "expat.h"

#include "Factory.h"
#include "WorldState.h"


class Action : public Attributed
{
	public:
		RTTI_DECLARATIONS(Action, Attributed)

		//! Constructor
		/*!
			a basic constructor
		*/
		Action();

		
		//! Deconstructor
		/*!
			a basic virtual deconstructor
		*/
		virtual ~Action();
		
		
		//! Process
		/*!
			Processes this action
			\param - a pointer to the current world state
		*/
		virtual void Process(WorldState * currentWorldState) = 0;


		//! Clone
		/*!
			clones this action and returns a pointer to the new action
			\param - sets the sector that owns this action
			\return - a pointer to the created cloned action
		*/
		Action * Clone(std::string typeName);


		//! Creates a new action
		/*!
			\param - the owner of the new action
			\param - the class name of the new action
			\param - the instance name of the new action
			\param - the index of the new action in the owner
		*/
		static Action * NewAction(Scope * scopeAddress, std::string actionClassName, std::string instanceName, unsigned int index, SList< pair<std::string, std::string> > * attributes = NULL);

	protected:
		//! LinkActionParameters
		/*!
			Links an action's parameters names to the actual paramaters.  This is done so that an action's parameters can be declared below it in xml scripting
			\return - if the link was successful or not
		*/
		virtual bool LinkActionParameters();

		//! SetupAction
		/*!

		\param - the slist of attributes to parse through
		*/
		virtual bool SetupAction(SList< pair<std::string, std::string> > * attributes);


	private:

	

};

//Macro
#define ActionFactory(ConcreteProductT) ConcreteFactory(Action, ConcreteProductT)