#pragma once

#include <string>
#include <sstream>
#include <stdio.h>
#include <utility>

#include "Scope.h"
#include "Vector.h"


class Attributed : public Scope 
{
	public:
		RTTI_DECLARATIONS(Attributed, Scope)

	protected:
		class Signature
		{
			public:
				std::string mName;
				Datum::TypeE mType;
				void* mInit;
				unsigned int mNumElements;
				Datum::DataType mStorage;

				//! Constructor
				/*!
					shall create an attribute 
				*/
				Signature() : mName(), mInit(NULL), mType(Datum::TYPE_UNKNOWN), mNumElements(0) 
				{	
					mStorage.TYPE_UNKNOWN = NULL;

				}

				//! Deconstructor
				/*!
					shall deconstruct the signature 
				*/
				~Signature()
				{
				}

				//Copy Constructor
				Signature(const Signature& source) : mName(), mInit(NULL), mType(Datum::TYPE_UNKNOWN), mNumElements(0) 
				{	
					this->operator=(source);
				}
		};

	public:

		//! Constructor
		/*!
			shall create an attribute 
		*/
		Attributed();

		//! Deconstructor
		/*!
			a virtual deconstructor
		*/
		virtual ~Attributed();

		//! Copy Constructor
		/*!
			shall create copy an attribute
			\param - the attribute to copy
		*/
		Attributed(const Attributed& source);

		//! Operator =
		/*!
			shall create copy the passed attriubute
			\param - the attribute to copy
		*/
		Attributed& operator=(const Attributed& source);

		//! IsPrescribedAttribute
		/*!
			shall check attribute associated with the passed name is a prescribed attribute
			\param - the name of the attribute to check
			\return - a bool if perscribed attribute 
		*/
		bool IsPrescribedAttribute(const std::string & checkedAttribute);


		//! IsAuxiliaryAttribute
		/*!
			shall check attribute associated with the passed name is a auxiliary attribute
			\param - the name of the attribute to check
			\return - a bool if it is an auxiliary attribute 
		*/
		bool IsAuxiliaryAttribute(const std::string & checkedAttribute);

		//! IsAttribute
		/*!
			shall check attribute associated with the passed name is a attribute
			\param - the name of the attribute to check
			\return - a bool if the attribute 
		*/
		bool IsAttribute(const std::string & checkedAttribute);


		//! AppendAuxiliaryAttribute
		/*!
			shall append  an auxiliary attribute
			\param - the name of the attribute to append
			\return - a pointer at the new datum
		*/
		Datum& AppendAuxiliaryAttribute(const std::string & checkedAttribute);


		//! AuxBegin
		/*!
			shall get where the aux attributes begin in the attribute table
			\return - an unsigned int of the start of the aux attributes
		*/
		const unsigned int AuxBegin() const;

		//!  Populate
		/*!
			shall go through the mSignatures table and create the data and place it in the scope table correctly.
		*/
		void Populate();

		Vector<Signature> mSignatures;	

	protected:
		//virtual int GetSignature(const unsigned int index, Attributed::Signature* mySignature) = 0;
	private:
		unsigned int mAuxBegin;

		
};
