#pragma once


#include <string>
#include <sstream>
#include <stdio.h>
#include <utility>
#include <stdlib.h>

#include "RTTI.h"

#include "SList.h"

#include "HashMap.h"

#include "Scope.h"
#include "Vector.h"
#include "Attributed.h"
#include "IXmlParseHelper.h"
#include "Factory.h"
#include "ActionList.h"

#include "expat.h"

#include "WorldState.h"




using namespace std;

//Forward Declarations
class Sector;

class Entity : public Attributed
{
	public:
		RTTI_DECLARATIONS(Entity, Attributed)
		
		//! Constructor
		/*!
			a basic constructor
		*/
		Entity();

		//! Deconstructor
		/*!
			a basic virtual deconstructor
		*/
		virtual ~Entity();
		
		//! GetSector
		/*!
			gets the sector that owns this entity
			\return - the sector that owns this entity
		*/
		Sector * GetSector();

		//! SetSector
		/*!
			sets the sector that owns this entity
			\param - sets the sector that owns this entity
		*/
		void SetSector(Sector * newSector);

		//! Clone
		/*!
			clones this entity and returns a pointer to the new entity
			\param - sets the sector that owns this entity
			\return - a pointer to the created cloned entity
		*/
		Entity * Clone(std::string typeName);
	
		int GetSignature();
		
		//! Process
		/*!
			Processes this entity
			\param - a pointer to the current world state
		*/
		virtual void Process(WorldState * currentWorldState);

	protected:

	private:
		Sector * mSector;
};

//Macro
#define EntityFactory(ConcreteProductT) ConcreteFactory(Entity, ConcreteProductT)
