#pragma once

#include <string>
#include <sstream>
#include <stdio.h>
#include <utility>
#include <stdlib.h>

//Foundation
#include "SList.h"

//Architecture
#include "RTTI.h"
#include "Datum.h"
#include "Scope.h"

//Event
#include "EventPublisher.h"
#include "IEventSubscriber.h"


template<typename TemplatedClass>
class Event : public EventPublisher
{
	public:
		RTTI_DECLARATIONS(Event, EventPublisher)

		//! Constructor
		/*!
			a constructor that takes an argument for the message type and a bool that determines 
			if this message will be destroyed when it is sent or if the user will handle that.
			\param - the message that the event ill contain
			\param - if the event will be deleted when delivered
		*/
		Event(TemplatedClass & message, bool deleteEvent);

		//! Deconstructor
		/*!
			a virtual deconstructor
		*/
		virtual ~Event();

		//! Subscribe
		/*!
			a static function that subscribes an event subscriber to an event
			\param - the event subscriber to add to the event's listener list
		*/
		static void Subscribe(IEventSubscriber * newEventSubscriber);

		//! Unsubscribe
		/*!
			a static function that unsubscribes an event subscriber from an event
			\param - the event subscriber to remove from the event's listener list
		*/
		static void Unsubscribe(IEventSubscriber * eventUnsubscriber);

		//! UnsubscribeAll
		/*!
			a static function that unsubscribes all event subscribers
		*/
		static void UnsubscribeAll();

		//! GetMessage
		/*!
			Gets this event's message
			\return - this event's message
		*/
		TemplatedClass GetMessage() const;

	protected:
		
		static SList<IEventSubscriber*> sSubscribers;

	private:
		TemplatedClass mMessage;

};

template <typename TemplatedClass>
RTTI_DEFINITIONS(Event<TemplatedClass>)

//Static SList of subscribers
template<typename TemplatedClass>
SList<IEventSubscriber*> Event<typename TemplatedClass>::sSubscribers;



template<typename TemplatedClass>
Event<typename TemplatedClass>::Event(TemplatedClass & message, bool deleteEvent) : EventPublisher(sSubscribers, deleteEvent)
{
	mMessage = message;
}

template<typename TemplatedClass>
Event<typename TemplatedClass>::~Event()
{
}

template<typename TemplatedClass>
void Event<typename TemplatedClass>::Subscribe(IEventSubscriber * newEventSubscriber)
{
	sSubscribers.PushBack(newEventSubscriber);
}

template<typename TemplatedClass>
void Event<typename TemplatedClass>::Unsubscribe(IEventSubscriber * eventUnsubscriber)
{
	sSubscribers.Remove(eventUnsubscriber);
}

template<typename TemplatedClass>
void Event<typename TemplatedClass>::UnsubscribeAll()
{
	sSubscribers.Clear();
}

template<typename TemplatedClass>
TemplatedClass Event<typename TemplatedClass>::GetMessage() const
{
	return(mMessage);
}

