#pragma once


#include <string>
#include <sstream>
#include <stdio.h>
#include <utility>
#include <stdlib.h>

//Architecture
#include "RTTI.h"
#include "Datum.h"
#include "Scope.h"

//Foundation
#include "SList.h"

//Event
#include "IEventSubscriber.h"


class EventPublisher : public RTTI
{
	public:
		RTTI_DECLARATIONS(EventPublisher, RTTI)

		//! Constructor
		/*!
			a constructor that takes a reference to the subscriber list, a bool if the event will delete it's self, 
			an optional enqueued time and an optional delay
	
			\param - reference to the subscriber list
			\param - bool indicating if the event will deletion will be handled automatically or not
			\param - optional enqueued time
			\param - optional delay
		*/
		EventPublisher(SList<IEventSubscriber*> & newSubscribersList, const bool deleteEvent, const double enqueuedTime = 0.0f, const double delayTime = 0.0f);
		
		//! Deconstructor
		/*!
			a virtual deconstructor
		*/
		virtual ~EventPublisher();

		//! Copy Constructor
		/*!
			A basic copy constructor that performs a deep copy
			\param source - event publisherto copy from
		*/
		EventPublisher(const EventPublisher & source);
	
		//! = operator overload
		/*!
			Overloads the = operator
			\param - event publisher to set equal to
			\return - reference to the source event publisher
		*/
		EventPublisher &operator=(const EventPublisher & source);

		//! SetTime
		/*!
			shall set the enqueued time and the delay time
			\param - the new enqueued time
			\param - the new delay time
		*/
		void SetTime(const double newEnqueuedTime, const double newDelayTime = 0.0f);

		//! GetTimeEnqueued
		/*!
			shall get the enqueued time
			\return - the enqueued time
		*/
		double GetTimeEnqueued() const;

		//! GetDelay
		/*!
			shall get the delay
			\return - the delay
		*/
		double GetDelay() const;

		//! IsExpired
		/*!
			shall return if the event publisher has expired or not, an event publisher has expired if the current time is
			greater than the enqueued time plus the delay
			\param - the current time
			\return - if the event has expired or not
		*/
		bool IsExpired(const double currentTime);

		//!  Deliver
		/*!
			shall deliver the message
		*/
		void Deliver();

		//! DeleteAfterPublishing
		/*!
			shall return a bool that determines if this event's deletion is automatically handled or not
			\return - bool that determines if this event's deletion is automatically handled or not
		*/
		bool DeleteAfterPublishing() const;

	protected:
		
	private:
		SList<IEventSubscriber*> & mSubscribers;

		bool mDeleteEvent;

		//Time
		double mEnqueuedTime;
		double mDelay;



};