#pragma once

#include "Vector.h"
#include "SList.h"
#include <utility>

// generic template Hash method 
template < typename KeyT >
class Hash{
	public:
		unsigned int operator()(const KeyT & myKey) const {
			const char* bits = reinterpret_cast<const char*>(&myKey);
			unsigned int hashValue = 0;
			for(unsigned int i = 0; i < sizeof(myKey); i++){
				hashValue += bits[i];
			}
			return(hashValue);
		}
};

// char* hash func
class HashCharString{
	public:
		unsigned int operator()(const char* myKey) const {
			unsigned int returnValue = (int)(myKey[0]);
			return (returnValue);
		}
};

// int hash func
class HashInt{
	public:
		unsigned int operator()(const int myKey) const {
			unsigned int returnValue = myKey;
			return (returnValue);
		}
};

// string hash func
class HashString{
	public:
		unsigned int operator()(const string myKey) const {
			unsigned int returnValue = 0;
			for(unsigned int i = 0; i < myKey.size(); i++){
				returnValue += (int)myKey[i];
			}
			return (returnValue);
		}
};

template < typename KeyT, typename DataT, typename H = Hash<KeyT> >
class HashMap
{	
	public:
		typedef KeyT KeyType;
		typedef DataT DataType;
		typedef pair<KeyT,DataT> ValueType;
	

		class HashMapIterator
		{
			friend class HashMap;
		
			public:
				HashMapIterator();
				~HashMapIterator();
				bool operator==(const HashMapIterator & otherIterator) const;
				bool operator!=(const HashMapIterator & otherIterator) const;
				ValueType* operator->();
				HashMapIterator operator++(int);
				HashMapIterator & operator++();
				ValueType & operator*();

				HashMapIterator & operator=(const HashMapIterator& source);


			private:
				//Memeber variables
				HashMap* mParent;
				unsigned int mBucketNumber;
				//Internal
				typename SList<ValueType>::SListIterator mySListIterator;
				
		};

		HashMap(unsigned int newNumberOfBuckets = 20);
		~HashMap();
		HashMapIterator Find(const KeyType& myKeyReference);
		HashMapIterator Insert(const ValueType & myReference);
		HashMapIterator Begin();
		HashMapIterator End();
		DataType& operator[](const KeyType& myKeyReference);
		void Remove(const KeyType & myKeyReference);
		void Clear();
		unsigned int Size() const;
		void SetNumberOfBuckets(unsigned int newNumberOfBuckets);
		void Rehash(unsigned int newSize);

	private:

		// The hash table
		Vector< SList< ValueType >* > mHashTable;

		unsigned int mSize;
};



//**********************************************
// Hashmap Iterator
//**********************************************

//! Constructor
/*!
	A constructor that creates a new iterator.
*/
template <typename KeyT, typename DataT, typename H  >
HashMap< KeyT, DataT, H >::HashMapIterator::HashMapIterator()
{
}

//! Deconstructor
/*!
	A deconstructor that creates a new iterator.
*/
template <typename KeyT, typename DataT, typename H  >
HashMap< KeyT, DataT, H >::HashMapIterator::~HashMapIterator()
{
} 

//! == operator overload
/*!
	Overloads the == operator
	\param - the iterator to compare to
	\return - a bool stating if the iterators are equal or not
*/
template <typename KeyT, typename DataT, typename H  >
bool HashMap< KeyT, DataT, H >::HashMapIterator::operator==(const HashMapIterator & otherIterator) const
{
	if(mParent == otherIterator.mParent){
		if(mySListIterator == otherIterator.mySListIterator){
			return(true);
		}
	}
	return(false);
}

//! != operator overload
/*!
	Overloads the != operator
	\param - the iterator to compare to
	\return - a bool stating if the iterators are equal or not
*/
template <typename KeyT, typename DataT, typename H >
bool HashMap< KeyT, DataT, H >::HashMapIterator::operator!=(const HashMapIterator & otherIterator) const
{
	if(mParent == otherIterator.mParent){
		if(mySListIterator == otherIterator.mySListIterator){
			return(false);
		}
	}
	return(true);
}

//! = operator overload
/*!
	Overloads the = operator
	\param - the source iterator
	\return - a deep copy of the source iterator
*/
template <typename KeyT, typename DataT, typename H >
typename HashMap< KeyT, DataT, H >::HashMapIterator& HashMap< KeyT, DataT, H >::HashMapIterator::operator=(const HashMapIterator& source)
{
	mParent = source.mParent;
	mBucketNumber = source.mBucketNumber;
	mySListIterator = source.mySListIterator;
	return(*this);
}

//! -> operator overload
/*!
	Overloads the -> operator
	\return - a pointer to the entry
*/
template <typename KeyT, typename DataT, typename H >
typename HashMap< KeyT, DataT, H >::ValueType* HashMap< KeyT, DataT, H >::HashMapIterator::operator->()
{
	return(&(*mySListIterator));
}

//! Postfix operator
/*!
	Overloads the ++ postfix incrementaor operator
	\param - a dummy  variable
	\return - a iterator reference that is pointing at the current spot then increases the iterator
	\asserts that the iterator is not trying to go past end
*/
template <typename KeyT, typename DataT, typename H >
typename HashMap< KeyT, DataT, H >::HashMapIterator HashMap< KeyT, DataT, H >::HashMapIterator::operator++(int)
{
	//assert(myNode);
	HashMapIterator tempCopy = (*this);
	// Call prefix
	operator++();
	return(tempCopy);
}

//! Prefix operator
/*!
	Overloads the ++ prefix incrementaor operator
	\return - a iterator reference that is pointing at the next spot
	\asserts that the iterator is not trying to go past end
*/
template <typename KeyT, typename DataT, typename H  >
typename HashMap< KeyT, DataT, H >::HashMapIterator& HashMap< KeyT, DataT, H >::HashMapIterator::operator++()
{
	//assert(myNode);
	//myNode = myNode->nextNode; // Move the next node
	
	mySListIterator++;
	if(mySListIterator == mParent->mHashTable[mBucketNumber]->End()){

		//if(myNode == 0){ // Move to the next list if we need to
			if((mBucketNumber + 1) < mParent->mHashTable.Capacity()) {
				mBucketNumber++; // Move to the next bucket
				while( (mBucketNumber < (mParent->mHashTable.Capacity() - 1)) && (mParent->mHashTable[mBucketNumber]->Size() == 0)) // And if this bucket is empty, skip it to the next bucket
					mBucketNumber++;
				
				//Make a copy of the old info so we can restore it later
				//int newBucketNumber = mBucketNumber;
				//HashMap* newParent = mParent;

				//SList<ValueType>::SListIterator 
				mySListIterator = mParent->mHashTable[mBucketNumber]->Begin();
				//HashMapIterator* myHashMapIterator = static_cast<HashMapIterator*>(&mySListIterator);
				
				//myHashMapIterator->mySListIterator = mParent->mHashTable[mBucketNumber]->Begin();


				//(*this) = (*myHashMapIterator);
				
				//Restore the correct information
				//mBucketNumber = newBucketNumber;
				//mParent = newParent;
			}
		//}
	}
	return(*this);
}

//! Dereference operator (operator*)
/*!
	Overloads the * operator
	\return - a reference to the entry in the hashmap
*/
template <typename KeyT, typename DataT, typename H  >
typename HashMap< KeyT, DataT, H >::ValueType& HashMap< KeyT, DataT, H >::HashMapIterator::operator*()
{
	return(*mySListIterator);
}

//**********************************************
// Hash Map
//**********************************************

//! Constructor
/*!
	A constructor that creates a new hash map of the passed in size.
	\param - a size of the hash map
*/
template <typename KeyT, typename DataT, typename H  >
HashMap< KeyT, DataT, H >::HashMap(unsigned int newNumberOfBuckets) : mSize(0)
{
	assert(newNumberOfBuckets > 0);

	mHashTable.Reserve(newNumberOfBuckets);
	for(unsigned int i = 0; i < newNumberOfBuckets; i++)
	{
		SList<ValueType>* newSList = new SList<ValueType>();
		mHashTable.PushBack(newSList);
	}
}

//! Deconstructor
/*!
	A deconstructor that creates a new iterator.
*/
template <typename KeyT, typename DataT, typename H >
HashMap< KeyT, DataT, H >::~HashMap()
{
	Clear();
}

//! Find function
/*!
	Finds the entry associated with the passed in key
	\param - the key to find
	\return - an iterator pointing at the found entry in the hashmap
*/
template <typename KeyT, typename DataT, typename H >
typename HashMap< KeyT, DataT, H >::HashMapIterator HashMap<KeyT, DataT, H >::Find(const KeyT & myKeyReference)
{
	if(mHashTable.Size() == 0)
		return(End());

	// Get the hash value from hash function pointer
	unsigned int hashValue = H()(myKeyReference);

	// Mod the hash value to get the index
	unsigned int bucketIndex = (hashValue%mHashTable.Capacity());

	HashMapIterator returnHashMapIterator;
	returnHashMapIterator.mySListIterator = mHashTable[bucketIndex]->Begin();		

	while(returnHashMapIterator.mySListIterator != mHashTable[bucketIndex]->End()){
		if((*returnHashMapIterator.mySListIterator).first == myKeyReference){
			returnHashMapIterator.mParent = (this);
			returnHashMapIterator.mBucketNumber = bucketIndex;

			return(returnHashMapIterator);
		}
		else
			returnHashMapIterator.mySListIterator++;
	}

	// Item was not found so return End
	if(returnHashMapIterator.mySListIterator == mHashTable[bucketIndex]->End())
		returnHashMapIterator = End();

	return(returnHashMapIterator);
}

//! Insert function
/*!
	//Shall insert the passed entry into the hash map
	\param - the entry to insert into the hash map
*/
template <typename KeyT, typename DataT, typename H >
typename HashMap< KeyT, DataT, H >::HashMapIterator HashMap< KeyT,  DataT, H >::Insert(const ValueType & myReference)
{
	// Get the hash value
	unsigned int hashValue = H()(myReference.first);

	// Mod the hash value to get the index
	unsigned int bucketIndex = (hashValue%mHashTable.Capacity());

	//Check if the valuetype is already in the hash table
	HashMapIterator returnHashMapIterator = Find(myReference.first);

	if(returnHashMapIterator == End()){ // inserting a new element
		mHashTable[bucketIndex]->PushFront(myReference);
		mSize++;
	}
	else{ // element already exists
		return returnHashMapIterator;
	}

	// Restore the correct information
	returnHashMapIterator.mySListIterator = mHashTable[bucketIndex]->Begin();
	returnHashMapIterator.mParent = (this);
	returnHashMapIterator.mBucketNumber = bucketIndex;

	return(returnHashMapIterator);
}

//! Clear function
/*!
	Clears all the data from the SList
*/
template <typename KeyT, typename DataT, typename H >
void HashMap< KeyT, DataT, H >::Clear()
{
	for(unsigned int i = 0; i < mHashTable.Size(); i++){
		mHashTable[i]->Clear();
		delete mHashTable[i];
	}
	mHashTable.Clear();
	mSize = 0;
	mHashTable.Resize(0);
}

//! Size function
/*!
	Gets the size of the HashMap
	\return - the size of the hashmap
*/
template <typename KeyT, typename DataT, typename H >
unsigned int HashMap< KeyT, DataT, H >::Size() const
{
	return(mSize);
}

//! Begin function
/*!
	shall return an Iterator (see below) pointing to the first element in the container.
	\return - a iterator that points at the front of the hashmap (first bucket)
*/
template <typename KeyT, typename DataT, typename H >
typename HashMap< KeyT, DataT, H >::HashMapIterator HashMap< KeyT,  DataT, H >::Begin()
{
	// Skip the empty beginning buckets
	if(Size() == 0)
		return(End());

	unsigned int begin = 0;
	for(unsigned int i = 0; i < mHashTable.Capacity(); i++){
		begin = i;
		if(mHashTable[i]->Size() > 0){
			i = mHashTable.Capacity() + 1;
		}
	}

	// Getting the iterator to return
	HashMapIterator returnHashMapIterator;

	//SList<ValueType>::SListIterator mySListIterator = mHashTable[begin]->Begin();
	//HashMapIterator* myHashMapIterator = ((HashMapIterator*)(&mySListIterator));
	//returnHashMapIterator = (*myHashMapIterator);
	//myHashMapIterator = 0; // Setting equal to null

	// Restore the correct information
	returnHashMapIterator.mParent = (this);
	returnHashMapIterator.mySListIterator = mHashTable[begin]->Begin();
	returnHashMapIterator.mBucketNumber = begin;
	return(returnHashMapIterator);
}

//! End function
/*!
	shall return an Iterator (see below) pointing to the last element in the container.
	\return - a iterator that points past the end of the hashMap
*/
template <typename KeyT, typename DataT, typename H >
typename HashMap< KeyT, DataT, H >::HashMapIterator HashMap< KeyT,  DataT, H >::End()
{
	// Getting the iterator to return
	HashMapIterator returnHashMapIterator;

	unsigned int endLocation = mHashTable.Size();
	if(endLocation > 0)
		endLocation--;

	// Restore the correct information
	returnHashMapIterator.mySListIterator.myNode = 0;
	returnHashMapIterator.mParent = (this);
	returnHashMapIterator.mBucketNumber = endLocation;
	return(returnHashMapIterator);
}

//! Remove function
/*!
	shall remove the entry associated with the passed key from the hash map
	\param - the key of the entry to remove from the hash map
*/
template <typename KeyT, typename DataT, typename H >
void HashMap< KeyT,  DataT, H >::Remove(const KeyType & myKeyReference)
{
	// Get the hash value
	unsigned int hashValue = H()(myKeyReference);

	// Mod the hash value to get the index
	unsigned int bucketIndex = (hashValue%mHashTable.Capacity());

	SList<ValueType>::SListIterator mySListIterator =  mHashTable[bucketIndex]->Begin();	

	while(mySListIterator != mHashTable[bucketIndex]->End()){
		if((*mySListIterator).first == myKeyReference){ // Remove the entry
			
			mHashTable[bucketIndex]->Remove((mySListIterator));
				
			// Exit
			mySListIterator = mHashTable[bucketIndex]->End();
			mSize--;
		}
		else
			mySListIterator++;
	}
}

//! [] overload
/*!
	shall return the data with the passed key in the hashmap.  Will assert if the key is not found
	\param - the key of the data to return
	\return - the data of the key
*/
template <typename KeyT, typename DataT, typename H >
typename HashMap< KeyT,  DataT, H >::DataType& HashMap< KeyT,  DataT, H >::operator[](const KeyType& myKeyReference)
{
	HashMapIterator myHashMapIterator = Find(myKeyReference);

	assert(myHashMapIterator != End());	

	return((*myHashMapIterator.mySListIterator).second);
}

//! SetNumberOfBuckets
/*!
	shall set the number of buckest to the passed in value
	\param - new number of buckets
*/
template <typename KeyT, typename DataT, typename H >
void HashMap< KeyT,  DataT, H >::SetNumberOfBuckets(unsigned int newNumberOfBuckets)
{
	Clear();
	mHashTable.Reserve(newNumberOfBuckets);
	for(unsigned int i = 0; i < newNumberOfBuckets; i++)
	{
		SList<ValueType>* newSList = new SList<ValueType>();
		mHashTable.PushBack(newSList);
	}
}

//! Rehash
/*!
	shall rehash the hash map
*/
template <typename KeyT, typename DataT, typename H >
void HashMap< KeyT,  DataT, H >::Rehash(unsigned int newSize)
{
	SList< ValueType > storedData;
	for(HashMapIterator myH = Begin(); myH != End(); myH++){
		storedData.PushBack(*myH);
	}

	mHashTable.Clear();
	for(unsigned int i = 0; i < mHashTable.Size(); i++){
		delete mHashTable[i];
		mHashTable[i] = NULL;
	}
	mHashTable.Resize(newSize);

	for(unsigned int i = 0; i < newSize; i++){
		SList<ValueType>* newSList = new SList<ValueType>();
		mHashTable.PushBack(newSList);
	}

	for(SList<ValueType>::SListIterator mySL = storedData.Begin(); mySL != storedData.End(); mySL++){
		Insert(*mySL);
	}
	storedData.Clear();
}