#pragma once

#include <string>
#include <sstream>
#include <stdio.h>
#include <utility>

#include "Scope.h"
#include "Attributed.h"
#include "SList.h"


//Forward Declarations
class XmlParseMaster;

class IXmlParseHelper
{
	public:
		class SharedData : public RTTI {
			public:
				RTTI_DECLARATIONS(SharedData, RTTI)

				//! Constructor
				/*!
					shall create shared data
				*/
				SharedData();

				//! Create
				/*!
					shall create copy of a this shared data and return it
				*/
				virtual SharedData* Create();
				
				//! SetXmlParseMaster
				/*!
					shall set the xml parse master of this shared data
					\param - new xml parse master
				*/
				void SetXmlParseMaster(XmlParseMaster* newXmlParseMaster);

				//! GetXmlParseMaster
				/*!
					shall return the xml parse object associated with that parse master
					\return - returns the xml parse master
				*/
				XmlParseMaster* GetXmlParseMaster();

				//! IncDepth
				/*!
					shall increases the depth of the shared data objects
				*/
				void IncDepth();

				//! DecDepth
				/*!
					shall decrease the depth of the shared data objects
				*/
				void DecDepth();

				//! GetDepth
				/*!
					shall decrease the depth of the shared data objects
					\return - shall return the current depth of the shared object
				*/
				unsigned int GetDepth();

			private:
				XmlParseMaster* mXmlParseMaster;
				unsigned int mNestingDepth;
		};




		//! Constructor
		/*!
			constructor for interface xml parse helper 
		*/
		IXmlParseHelper();
		
		//! Deconstructor
		/*!
			deconstructor for interface xml parse helper 
		*/
		virtual ~IXmlParseHelper();
		
		//! Init
		/*!
			shall initialize this helper.  This will get called just before each file is parsed.
		*/
		virtual void Init();
		
		//! StartElementHandler
		/*!
			shall handle getting a new line or set of xml elements
			\param - the tag of this element
			\param - a slist of pair of string string type 
		*/
		virtual bool StartElementHandler(std::string tagName, SList< pair<std::string, std::string> > attributeValuePairs);
		
		//! EndElementHandler
		/*!
			shall handle ending an element
			\param - the tag of this element
		*/
		virtual bool EndElementHandler(std::string tagName); 
		
		//! CharDataHandler
		/*!
			shall handle the char data of an element
			\param - the buffer of this char data
			\param - the length of the char data
		*/
		virtual void CharDataHandler(std::string buffer);
		
		//! Create
		/*!
			given a reference to an XmlParseMaster, duplicate this helper
			\param - reference to an XmlParseMaster
		*/
		virtual IXmlParseHelper* Create(XmlParseMaster* referencedParseMaster) = 0;
		
		//! CheckIfValidTag
		/*!
			checks if the passed in tag is a valid tag for this helper
			\param - the tag to check
		*/
		bool CheckIfValidTag(std::string tag);

	protected:
		XmlParseMaster* mMaster;
		SList<std::string> mValidTags;
};