#pragma once

#include <string>



class RTTI
{
public:
	virtual const unsigned int& TypeIdInstance() const = 0;
	
	virtual RTTI* QueryInterface(const unsigned id) const
	{
		return NULL;
	}

	virtual bool Is(const unsigned int id) const
	{
		return false;
	}

	virtual bool Is(const std::string& name) const
	{
		return false;
	}

	template <typename T>
	T* As() const
	{
			if (Is(T::TypeIdClass()))
			{
				return (T*)this;
			}

			return NULL;
	}
};

#define RTTI_DECLARATIONS(Type, ParentType)                                                              \
	private:                                                                                             \
		static unsigned int sRunTimeTypeId;                                                              \
	public:                                                                                              \
		typedef ParentType Parent;                                                                       \
		static std::string TypeName() { return std::string(#Type); }                                     \
		virtual const unsigned int& TypeIdInstance() const { return Type::TypeIdClass(); }               \
		static  const unsigned int& TypeIdClass() { return sRunTimeTypeId; }                             \
		virtual RTTI* QueryInterface( const unsigned int id ) const                                      \
		{                                                                                                \
			if (id == sRunTimeTypeId)                                                                    \
				{ return (RTTI*)this; }                                                                  \
			else                                                                                         \
				{ return Parent::QueryInterface(id); }                                                   \
		}                                                                                                \
		virtual bool Is(const unsigned int id) const                                                     \
		{                                                                                                \
			if (id == sRunTimeTypeId)                                                                    \
				{ return true; }                                                                         \
			else                                                                                         \
				{ return Parent::Is(id); }                                                               \
		}                                                                                                \
		virtual bool Is(const std::string& name) const                                                   \
		{                                                                                                \
			if (name == TypeName())                                                                      \
				{ return true; }                                                                         \
			else                                                                                         \
				{ return Parent::Is(name); }                                                             \
		}

#define RTTI_DEFINITIONS(Type) unsigned int Type::sRunTimeTypeId = (unsigned int)& Type::sRunTimeTypeId;


/*! \brief Abstract base class (interface) for objects that have tables and can clone themselves.

	\note We could instead define a class IUser to use multiple inheritance to derive from
			IClonable and RTTI, but single inheritance reduces the number of vtable pointers,
			therefore reduces memory usage.
*/
class IClonable : public RTTI
{
	public:
		virtual ~IClonable() {}

		/*! \brief Clone this object.
		*/
		virtual IClonable * Clone ( void ) = 0;
};
