#pragma once

#include <iostream>
#include <assert.h>

using namespace std;

/*!
		1. What are the ramifications of using a linked list instead of a resizable array for the same public interface?  
		Discuss run-time performance and memory impact, including size, overhead and fragmentation.  Hint: read about realloc.

			Using realloc will be very slow if you happen to increase the size of the list by one element at a time.  
	
			Realloc also has to copy over the contents from the old buffer of memory to the new buffer of memeory.  This will
			take up alot of memory.  
			
			The inital overhead for a linked list is very low and time required to add a new node. 
			A resizeable array can have small overhead but then will require more time when resizing the list.  

			
		2. This implementation allocates memory for link nodes.  
		Discuss strategies for allowing the application to control how to perform that allocation.  
		For example, what if the application wants all link nodes to reside in a static array?  Hint:  Read about STL.

			By increasing the size of the array by a multiplying it's current size by two 
			everytime you need to resize the array, you should have to do less  esizing of the array.  
			You can also easily and quickly access the elements this way since you will be using an 
			array based system.

			If you wanted all link nodes to reside in a static array you could create a static array that would be
			essentally be your memory.  You would then places nodes in that array when they were allocated by using
			placement new.  This would easily speed up allocation, but require more overhead than the SList that we implemented.
		
*/


/*
		1. This class has a Clear method which removes all items from the list. Does Clear eliminate memory leaks?  
		What if the item type is a pointer? Is it possible to write SList so that it has no memory leaks?  
		E.g. what if you call Clear from the list destructor? If so, how and if not, why and what can you do to mitigate the situation?

			No, clear does not eliminate memory leaks.  If the item type is a pointer then SList doesn't own the memory since it was initalized outside of
			SList.  It is possible to write SList with no memory leaks by creating a specialization that uses pointers.
			
		2. What method would be nice to have to deal with a list of pointers?  What problems would arise with such a routine?

			If you had a method to determine if a type was a pointer or not you could then delete the memory and have no memory leaks.
			You could however be deleting memeory that another pointer could be pointing at.  You could then use reference counting to
			solve that issue.

*/
//! A Generic Templated Class
template<typename TemplatedClass>
class SList
{
private:

	//! Node Class
	class MyNode
	{
		public:
			TemplatedClass data; /*! The data this node refers to */
			MyNode*	nextNode; /*! A pointer to next node on the list */

			//! Constructor
			/*!
				A basic constructor that sets up a node.
			*/
			MyNode()
			{
				nextNode = 0; //! A pointer to the front of the list */
			}

			//! Deconstructor
			/*!
				A basic deconstructor that deletes a node and the node that it points too.
			*/
			~MyNode()
			{
			}

			//! Constructor
			/*!
				A specialized constructor that sets up a node with the data passed in.
				\param newData -the data that will be assigned to this node
			*/
			MyNode(const TemplatedClass & newData)
			{
				data = newData;
				nextNode = 0;
			}
	};

	MyNode*	front; /*! A pointer to the front of the list */
	MyNode*	back; /*! A pointer to the end of the list */
	unsigned int size; /*! The size of the list */


public:
	//! Iterator Class
	class SListIterator
	{
		friend class SList;
		template <typename KeyT, typename DataT, typename H > friend class HashMap;

		public:

			//! Constructor
			/*!
				A constructor that creates a new iterator.
			*/
			SListIterator()
			{
			}

			//! Deconstructor
			/*!
				A deconstructor
			*/
			~SListIterator()
			{
			}

			//! = operator overload
			/*!
				Overloads the = operator
				\param - the iterator to set to
				\return - reference to the source iterator
			*/
			SListIterator &operator=(const SListIterator & source)
			{
				myNode = source.myNode;
				return(*this);
			}

			//! Pointer dereference
			/*!
				Returns the reference to data that this iterator's node points at
			*/
			TemplatedClass& operator*()
			{
				return(myNode->data);
			}

			//! Pointer dereference
			/*!
				Returns const data that this iterator's node points at
			*/
			const TemplatedClass& operator*() const
			{
				return(myNode->data);
			}

			//! == operator overload
			/*!
				Overloads the == operator
				/note also long as the iterators point to the same node
				there will be considered equal.  This doesn't check if they
				have the same parent SList
			*/
			bool operator==(const SListIterator & otherIterator) const
			{
				if(myNode == otherIterator.myNode)
					return(true);
				else
					return(false);
			}

			//! != operator overload
			/*!
				Overloads the != operator
				/note also long as the iterators point to the same node
				there will be considered equal.  This doesn't check if they
				have the same parent SList
			*/
			bool operator!=(const SListIterator & otherIterator) const
			{
				if(myNode == otherIterator.myNode)
					return(false);
				else
					return(true);
			}

			//! Postfix operator
			/*!
				Overloads the ++ postfix incrementaor operator
			*/
			SListIterator operator++(int)
			{
				assert(myNode);
				SListIterator tempCopy = (*this); //Make a copy
				myNode = myNode->nextNode; //Move the next node
				return(tempCopy); //Return the copy
			}

			//! Prefix operator
			/*!
				Overloads the ++ prefix incrementaor operator
				\asserts that the iterator has been created with this or end (has a node).
			*/
			SListIterator & operator++()
			{
				assert(myNode);
				myNode = myNode->nextNode; //Move the next node
				return(*this);
			}

		protected:
			//! Constructor
			/*!
				A specialized constructor that sets up a iterator with the desired node
				\param newNode -the node that the constructer will point to
			*/
			SListIterator(MyNode * newNode)
			{
				myNode = newNode;
			}

			MyNode* myNode;

			//! SetMyNode
			/*!
				Sets the myNode data memeber to the passed node
				\param newMyNode -the node that the constructer will point to
			*/
			void SetMyNode(MyNode* & newMyNode)
			{
				myNode = newMyNode;
			}
	};		

	//! Constructor
	/*!
		A basic constructor that sets up a linked list.
	*/
	SList() : size(0), front(0), back(0)
	{
	}

	//! Copy Constructor
	/*!
		A basic copy constructor that performs a deep copy
		\param source -the list to copy from
	*/
	SList(const SList& source) : size(0), front(0), back(0)
	{
		this->operator=(source);
	}

	//! Assignment Operator
	/*!
		A basic assignment operator that performs a deep copy
		\param source -the list to copy from
		\return -returns a poiner to the new deep copied list
	*/
	SList<TemplatedClass> &operator=(const SList& source)
	{
		Clear(); //! We need a empty list to start copying

		SList<TemplatedClass>::SListIterator iIterator = source.Begin();

		if(source.Size() > 0){
			for(SListIterator iIterator = source.Begin(); iIterator != source.End(); iIterator++){
				PushBack(*iIterator);
			}
		}
		return(*this);
	}

	//! Destructor
	/*!
		A basic destructor that clears and deallocs the linked list
	*/
	~SList()
	{
		Clear();
	}

	//! PushFront
	/*!
		Places the data of the front of the SList
		\param newNodeData -the element that will be placed at the top of the list
	*/
	void PushFront(const TemplatedClass & newNodeData)
	{
		MyNode* myNewNode = new MyNode(newNodeData);
		myNewNode->nextNode = front;

		
		front = myNewNode; // Set the new front
		if(size == 0) // Possibly set the new back node
			back = myNewNode;

		size++; // increase the size of the list
	}
	
	//! PopFront
	/*!
		Removed the top of the list and returns it.
		\return -returns the data from the former front of the list or returns 0
	*/
	TemplatedClass PopFront()
	{
		assert (size > 0);
		if(size > 0) // Decrease the size of the sList
			size--;
		
		TemplatedClass myNodeData = front->data; // Get the return data
		
		MyNode* myTempNode = front;

		front = front->nextNode; // Set the new front and pop off the old node.

		myTempNode->nextNode = 0; // Set the next node of this pointer to null so that we don't call it's destructor
		delete(myTempNode);

		return(myNodeData);
	}

	//! PushBack
	/*!
		Places the data of the end of the SList
		\param newNodeData -the element that will be placed at the end of the list
	*/
	void PushBack(const TemplatedClass & newNodeData) 
	{
		MyNode* myNewNode = new MyNode(newNodeData);
		myNewNode->nextNode = 0; // Set the new end of node to null (special symbol)
		
		if(size > 0)
			back->nextNode = myNewNode;
		back = myNewNode; // Set the new back node

		
		if(size == 0)
			front = myNewNode; // Possibly set the new front node

		size++; // increase the size of the list
	}

	//! IsEmpty
	/*!
		Returns if the SList is empty or not
		\return -returns true or false depending on if the list is empty or not
	*/
	bool IsEmpty() const
	{
		if(size <= 0)
			return(true);
		else
			return(false);
	}

	//! Front
	/*!
		Returns a reference to the data that is at the top of the list
		\return -returns a reference the data that is at the top of the list
	*/
	TemplatedClass & Front()
	{
		assert(size > 0);
		return((front->data));
	}

	//! Back
	/*!
		Returns a reference to the data that is at the end of the list
		\return -returns a reference the data that is at the end of the list
	*/
	TemplatedClass & Back()
	{
		assert (size > 0);
		return((back->data));
	}
	
	//! Size
	/*!
		Gets the size of the SList
		\return -returns the size of the SList
	*/
	unsigned int Size() const
	{
		return(size);
	}

	//! Clear
	/*!
		Clears the list.
	*/
	void Clear()
	{
		if(front){
			while(size > 0)
				PopFront();
		}
	}

	//! Begin
	/*!
		Returns an iterator that points to the front of the list.
		\return -returns a iterator that points to the top of the list
	*/
	SListIterator Begin() const
	{
		SListIterator beginIterator(front);
		return(beginIterator);
	}

	//! End
	/*!
		Returns an iterator that points past end of the list.
		\return -returns a iterator that points to the end of the list
	*/
	SListIterator End() const
	{
		SListIterator endIterator(0);//back);
		return(endIterator);
	}

	//! InsertAfter
	/*!
		Inserts the item after the specifed iterator
		\param newNodeData -the data that will be inserted into the list
		\param insertLocationIterator -the location where the data will be inserted into the list
	*/
	void InsertAfter(const TemplatedClass & newNodeData, const SListIterator & insertLocationIterator)
	{
		MyNode* myNewNode = new MyNode(newNodeData); // Create the new node
		SListIterator iIterator = Begin();
		while((iIterator != insertLocationIterator) && (iIterator != End())){
			iIterator++;
		}
		if(iIterator != End()){
			myNewNode->nextNode = iIterator.myNode->nextNode; // Set the new end of node to null (special symbol)
		
			iIterator.myNode->nextNode = myNewNode;
			size++; // Increasae the size of the list
			if(myNewNode->nextNode == 0) // If the new node is at the end of the list then we set it back to it
			{
				back = myNewNode;
			}
		}
	}

	//! Find
	/*!
		Finds the desired item
		\param desiredItem -the data that will be searched for
		\return -returns a iterator that points to the desired data or end if the data wasn't found
	*/
	SListIterator Find(const TemplatedClass & desiredItem)
	{
		SListIterator iIterator = Begin();
		while((iIterator != End()) && (!(desiredItem == *iIterator)))
		{
			iIterator++;
		}
		return(iIterator);
	}

	//! Remove
	/*!
		Removes the item specified by the templated class
		\param deletedItem -the data that will be removed from the list
	*/
	void Remove(const TemplatedClass & deletedItem)
	{
		SListIterator iIterator = Find(deletedItem);

		if(iIterator != End()){ //Only delete if we find the item
			SListIterator jIterator = Begin(); //Find the item before the deleted item
			if(Size() > 1){
				while((jIterator != End()) && (jIterator.myNode->nextNode) && (!(jIterator.myNode->nextNode->data == *iIterator)))
				{
					jIterator++;
				}
			}
			//Special Cases
			if(iIterator.myNode->nextNode == 0) // There is going to be a new end
			{
				if(jIterator == iIterator)
					jIterator.myNode = 0;
				back = jIterator.myNode;
			}
			if(Begin() == iIterator) //There is going to be a new front
			{
				front = iIterator.myNode->nextNode;
			}
			else{
				jIterator.myNode->nextNode = iIterator.myNode->nextNode;
			}
		
			iIterator.myNode->nextNode = 0; //Remove the node from the list so we can delete it
			delete(iIterator.myNode); //Delete the node
	
			size--; //Decrease the size of the list
		}
	}


	//! Remove
	/*!
		Removes the item specified by the iterator
		\param deletedItem -the data that will be removed from the list
	*/
	void Remove(const SListIterator & removeLocationIterator)
	{
		Remove(removeLocationIterator.myNode->data);
	}
};


