#pragma once


#include <string>
#include <sstream>
#include <stdio.h>
#include <utility>
#include <stdlib.h>

#include "RTTI.h"

#include "SList.h"

#include "HashMap.h"

#include "Datum.h"
#include "Scope.h"
#include "Vector.h"
#include "Attributed.h"
#include "IXmlParseHelper.h"

#include "expat.h"



#include "Sector.h"
#include "WorldState.h"

//Event
#include "EventQueue.h"

using namespace std;


class World : public Attributed
{
	public:
		RTTI_DECLARATIONS(World, Attributed)

		//! Constructor
		/*!
			a basic constructor
		*/
		World();
		
		//! Deconstructor
		/*!
			a basic virtual deconstructor
		*/
		virtual ~World();

		//! AppendSector
		/*!
			Creates a new sector and places it in this world
			\param - the name of the new sector in the sector list
			\return - a pointer to the newly created sector
		*/
		Sector * AppendSector(std::string sectorName, std::string fileName = "NO FILENAME PROVIDED", Sector * parentSector = NULL);

		//! GetSectors
		/*!
			Gets the container of sectors that are in this world
			\return - the container of sectors
		*/
		Scope * GetSectors();

		//! Process
		/*!
			Processes this world and all sectors inside of it
			\param - a pointer to the current world state
		*/
		void Process();

		//! GetSignature
		/*!
			Gets the signature of the comment
		*/
		int GetSignature();

		//! GetWorldState
		/*!
			Gets the world state
			\return the world state
		*/
		WorldState & GetWorldState();


		//! GetEventQueue
		/*!
			Gets the world state
			\return the event queue
		*/
		EventQueue & GetEventQueue();

	protected:
		WorldState mWorldState;
		EventQueue mEventQueue;

	private:

};