#pragma once

#include "IXmlParseHelper.h"
#include "XmlParseHelperScope.h"
#include "Stack.h"
#include "Entity.h"
#include "Sector.h"
#include "World.h"

//Actions
#include "Action.h"
#include "ActionCreateAction.h"
#include "ActionDestroyAction.h"
#include "ActionPrint.h"
#include "ActionListWhile.h"

#include "XmlParseMaster.h"

class XmlParseHelperAction : public IXmlParseHelper
{
	public:
		
		//! Constructor
		/*!
			constructor for a xml parse helper entity
		*/
		XmlParseHelperAction(XmlParseMaster* newParseMaster);

		//! Deonstructor
		/*!
			deconstructor for a xml parse helper entity
		*/
		virtual ~XmlParseHelperAction();

		//! Create
		/*!
			given a reference to an XmlParseMaster, duplicate this helper
			\param - reference to an XmlParseMaster
		*/
		virtual XmlParseHelperAction* Create(XmlParseMaster* referencedParseMaster);

		//! StartElementHandler
		/*!
			shall handle getting a new line or set of xml elements
			\param - the tag of this element
			\param - a slist of pair of string string type 
		*/
		virtual bool StartElementHandler(std::string tagName, SList< pair<std::string, std::string> > attributeValuePairs);
		
		//! EndElementHandler
		/*!
			shall handle ending an element
			\param - the tag of this element
		*/
		virtual bool EndElementHandler(std::string tagName); 

		//! CharDataHandler
		/*!
			shall handle the char data of an element
			\param - the buffer of this char data
			\param - the length of the char data
		*/
		virtual void CharDataHandler(std::string buffer);
};