#pragma once

#include "IXmlParseHelper.h"
#include "Stack.h"
#include "Scope.h"
#include "Attributed.h"
#include "Datum.h"

#include "XmlParseMaster.h"

#include "Vec4.h"
#include "Mat4.h"

class XmlParseHelperScope : public IXmlParseHelper
{
	public:
		class SharedDataScope : public IXmlParseHelper::SharedData
		{
			public:
				RTTI_DECLARATIONS(XmlParseHelperScope::SharedDataScope, IXmlParseHelper::SharedData)
				
				friend class XmlParseHelperScope;
				friend class XmlParseHelperEntity;
				friend class XmlParseHelperAction;

				//! Constructor
				/*!
					constructor for a XmlParseHelperScope shared data
				*/
				SharedDataScope();

				//! Specialized Constructor
				/*!
					constructor for a XmlParseHelperScope shared data
					\param - the new gloabl scope to point to
				*/
				SharedDataScope(Scope * newGlobalScope);
				
				//! Create
				/*!
					Returns the global scope for this shared data
					\return - returns a reference ot the global scope 
				*/
				Scope& GetGlobalScope();
			protected:
				Scope* mCurrentScope;
				Scope* mGlobalScope;

				Datum* mCurrentDatum;

		};

		//! Constructor
		/*!
			constructor for a xml parse helper scope
		*/
		XmlParseHelperScope(XmlParseMaster* newParseMaster);

		//! Deonstructor
		/*!
			deconstructor for a xml parse helper scope
		*/
		virtual ~XmlParseHelperScope();

		//! Create
		/*!
			given a reference to an XmlParseMaster, duplicate this helper
			\param - reference to an XmlParseMaster
		*/
		virtual XmlParseHelperScope* Create(XmlParseMaster* referencedParseMaster);

		//! StartElementHandler
		/*!
			shall handle getting a new line or set of xml elements
			\param - the tag of this element
			\param - a slist of pair of string string type 
		*/
		virtual bool StartElementHandler(std::string tagName, SList< pair<std::string, std::string> > attributeValuePairs);
		
		//! EndElementHandler
		/*!
			shall handle ending an element
			\param - the tag of this element
		*/
		virtual bool EndElementHandler(std::string tagName); 

		//! CharDataHandler
		/*!
			shall handle the char data of an element
			\param - the buffer of this char data
			\param - the length of the char data
		*/
		virtual void CharDataHandler(std::string buffer);
};