#pragma once

#include <string>
#include <sstream>
#include <stdio.h>
#include <utility>
#include <stdlib.h>

#include "RTTI.h"

#include "SList.h"

#include "Scope.h"
#include "Vector.h"
#include "Attributed.h"
#include "IXmlParseHelper.h"

#include "expat.h"

class XmlParseMaster
{
	public:
			
		//! Constructor
		/*!
			shall construct a xml parse master
			\param - the shared data to assign to this master
		*/
		XmlParseMaster(IXmlParseHelper::SharedData * newShardedData);
		
		//! Constructor
		/*!
			shall construct an empty xml parse master
		*/
		XmlParseMaster();

		//! Deconstructor
		/*!
			shall construct a xml parse master
		*/
		~XmlParseMaster();
		
		//! AddHelper
		/*!
			shall construct a xml parse master
			\param - the helper to add to this master
		*/
		void AddHelper(IXmlParseHelper* helper); //  given the address of an IXmlParseHelper object, add it to the list.
		
		//! RemoveHelper
		/*!
			shall construct a xml parse master
			\param - the helper to remove from this master
		*/
		void RemoveHelper(IXmlParseHelper* helper); //  given the address of an IXmlParseHelper object, remove it from the list.

		//! Parse
		/*!
			shall parse a buffer 
			\param - the buffer to parse
			\size - the size of the buffer to parse
			\param - a bool indicating if it is the last buffer to parse
		*/
		void Parse(char* buffer, unsigned int size, bool lastChunk);
		
		//! ParseFromFile
		/*!
			shall parse the file passed to it buffer 
			\param - the name of the file to parse
		*/
		void ParseFromFile(std::string fileName);
		
		//! GetSharedData
		/*!
			shall get the shared data associated with this master
			\param - the name of the file to parse
		*/
		IXmlParseHelper::SharedData* GetSharedData();

		//! Create
		/*!
			shall create a new parse master
			\return - the newly created parse master
		*/
		XmlParseMaster* Create();
		
		//! SetFileName
		/*!
			shall set the file name to parse
			\param - the name to set the filename to
		*/
		void SetFileName(std::string newFileName);
		
		//! GetFileName
		/*!
			shall get the file name to parse
			\return - the name of file to parse
		*/
		std::string GetFileName();
		
		//! SetParser
		/*!
			shall set the expat parser for this parse master 
			\param - the expat parser
		*/
		void SetParser(XML_Parser newParser);

		//! Reset
		/*!
			shall reset this parse master so that it is ready for a fresh file
		*/
		void Reset();

		//! GetIsClone
		/*!
			shall get if it is a clone or not
			\return bool indicating if its a clone or not
		*/
		bool GetIsClone();

	protected:
		

	private:

		static void XMLCALL StartElementHandler(void *data, const char *el, const char **attr); // This is the callback that Expat will call to handle XML start elements.  This implements the chain of responsibility design pattern.
		static void XMLCALL EndElementHandler(void *data, const char *el); // This is the callback that Expat will call to handle XML end elements.  This implements the chain of responsibility design pattern.
		static void XMLCALL CharDataHandler(void *data, const char* el, int length); //- This is the callback that Expat will call to handle XML character data.  This implements the chain of responsibility design pattern.


		//Helpers
		SList<IXmlParseHelper*> mHelpers;

		// Data
		IXmlParseHelper::SharedData* mSharedData;

		// Parser
		XML_Parser mParser;

		// File Name
		std::string mFileName;


		bool mIsClone;
};