/*! \file util.h

    \brief utility routines

    \author Copyright 2005-2008 UCF/FIEA/MJG; All rights reserved.
*/
#ifndef UTIL_H
#define UTIL_H

#include <stdio.h>
#include <string>


inline bool IsInf( float x )
{
    return ( ( x - x ) != 0.0f ) ;
}


inline bool IsNan( float x )
{
    // If x is a real number then it must be either less-than 0 or greather-than-or-equal-to 0.
    // If x is neither then it is Not A Number (NAN).
    return ( ( ! ( x < 0.0f ) ) && ( ! ( x >= 0.0f ) ) ) ;
}


//! \brief Return sign bit of floating point value
inline int ISignBit( const float & f )
{
    return ( (int&) f & 0x80000000 ) >> 31 ;
}


//! \brief Return sign (-1 or 1) of floating point value
inline  float FSign( const float & f )
{
    float fReturnValue = 1.0f ;
    // Mask sign bit from f and set it in fReturnValue so that fReturnValue has the same sign
    (int&) fReturnValue |= ( ( (int&) f ) & 0x80000000 ) ;
    return fReturnValue ;
}


//! \brief Multiply floating point value by the sign of another floating point value
inline void FMultSign( float & dest , float const & source )
{
    // NOTE: For transfering sign from source to dest, it seems like this first line could be simpler:
    // int sign_mask = (int&) source & 0x80000000 ;
    int sign_mask = ((int&)dest ^ (int&)source) & 0x80000000; // XOR and mask
    (int&)dest &= 0x7FFFFFFF; // clear sign bit
    (int&)dest |= sign_mask; // set sign bit if necessary
}


//! \brief Assign sign of floating point value from the sign of another floating point value
inline void FSetSign( float & dest , float const & source )
{
    int sign_mask = (int&) source & 0x80000000 ;
    (int&)dest &= 0x7FFFFFFF; // clear sign bit
    (int&)dest |= sign_mask; // set sign bit if necessary
}


//! \brief Return absolute value of floating point value
inline float FAbs( const float & f )
{
    int iReturnValue = (int&) f & 0x7FFFFFFF ; // clear sign bit
    return (float&) iReturnValue ; // clear sign bit
}


//! \brief Assign floating point value in-place
inline void FAbsInPlace( float & f )
{
    (int&) f &= 0x7FFFFFFF ; // clear sign bit
}

extern float RandomSpread( float fSpread ) ;
extern void strTranslate( char * str , int iOriginal , int iFinal ) ;
extern FILE * FileOpen( const char * strFilename , const char * strAccessMode , const char ** pPathChosen = 0 ) ;
extern unsigned int FileLoad( char ** ppBuffer , const char * strFilename , const char ** pPathChosen = 0 ) ;

inline std::string RemoveWhiteSpace(std::string & str){
	for(unsigned int i = 0; i < str.length(); i++){
		if((str[i] == ' ')||(str[i] == '\n')||(str[i] == '\t')||(str[i] == '\r')){
			str.erase(i,1);
			i--;
		}
	}
	return(str);
}



#endif
