#include "ActionEvent.h"

RTTI_DEFINITIONS(ActionEvent)

ActionEvent::ActionEvent() : mRecurring(true), mHasBeenProcessed(false)
{
	Signature signatureSubtype;
	signatureSubtype.mNumElements = 1;
	signatureSubtype.mName = "Subtype";
	signatureSubtype.mInit = NULL;
	signatureSubtype.mStorage.TYPE_STRING = &mSubtype;
	signatureSubtype.mType = Datum::TYPE_STRING;
	
	Signature signatureDelay;
	signatureDelay.mNumElements = 1;
	signatureDelay.mName = "Delay";
	signatureDelay.mInit = NULL;
	signatureDelay.mStorage.TYPE_FLOAT = &mDelay;
	signatureDelay.mType = Datum::TYPE_FLOAT;

	Signature signatureRecurring;
	signatureRecurring.mNumElements = 1;
	signatureRecurring.mName = "Recurring";
	signatureRecurring.mInit = NULL;
	signatureRecurring.mStorage.TYPE_INT = &mRecurring;
	signatureRecurring.mType = Datum::TYPE_INT;
}

ActionEvent::~ActionEvent()
{
}

void ActionEvent::Process(WorldState * currentWorldState)
{
	if((mRecurring > 0) || (mHasBeenProcessed == false)){
		//Set that this has been processed
		mHasBeenProcessed = true;

		//Get the world
		World * world = currentWorldState->mWorld;

		//Set message infomation
		EventMessageAttributed  myMessage;
		myMessage.SetWorld(currentWorldState->mWorld);
		myMessage.SetSubtype(mSubtype);
		
		//Creating a new event
		Event<EventMessageAttributed> * event = new Event<EventMessageAttributed>(myMessage, true);

		//Copy aux parameters
		for(unsigned int i = AuxBegin(); i < Size(); i++){
			Datum & referenceAttribute = myMessage.AppendAuxiliaryAttribute(GetMemberName(i));
			referenceAttribute = GetEntry(i);
		}

		//Enqueing the event
		world->GetEventQueue().Enqueue(event, currentWorldState->mCurrentTime, mDelay);
	}
}

bool ActionEvent::SetupAction(SList< pair<std::string, std::string> > * attributes)
{
	for(SList< pair<std::string, std::string> >::SListIterator mySListIterator = attributes->Begin(); mySListIterator != attributes->End(); mySListIterator++){
		if((*mySListIterator).first == "subtype"){
			mSubtype = (*mySListIterator).second;
		}
		else if((*mySListIterator).first == "delay"){
			Datum tempStringDatum;
			tempStringDatum.SetType(Datum::TYPE_STRING);
			tempStringDatum.Set((*mySListIterator).second);
			mDelay = tempStringDatum.GetAs<float>();
		}
		else if((*mySListIterator).first == "recurring"){
			Datum tempStringDatum;
			tempStringDatum.SetType(Datum::TYPE_STRING);
			tempStringDatum.Set((*mySListIterator).second);
			mRecurring = tempStringDatum.GetAs<int>();
		}
	}
	if(mSubtype.size() > 0)
		return true;
	return false;
}