#include "ActionListWhile.h"

RTTI_DEFINITIONS(ActionListWhile)


ActionListWhile::ActionListWhile() : mLoopNumber(NULL), mPreamble(NULL), mLoopProcess(NULL)
{
	mPreamble = &AppendScope("Preamble");
	mLoopProcess = &AppendScope("LoopProcess");

	Signature signaturePreamble;
	signaturePreamble.mNumElements = 1;
	signaturePreamble.mName = "sPreamble";
	signaturePreamble.mInit = NULL;
	signaturePreamble.mStorage.TYPE_SCOPE = &mPreamble;
	signaturePreamble.mType = Datum::TYPE_SCOPE;
	
	Signature signatureLoopProcess;
	signatureLoopProcess.mNumElements = 1;
	signatureLoopProcess.mName = "sLoopProcess";
	signatureLoopProcess.mInit = NULL;
	signatureLoopProcess.mStorage.TYPE_SCOPE = &mLoopProcess;
	signatureLoopProcess.mType = Datum::TYPE_SCOPE;

	Signature signatureLoopCount;
	signatureLoopCount.mNumElements = 1;
	signatureLoopCount.mName = "iLoopCounter";
	signatureLoopCount.mInit = NULL;
	signatureLoopCount.mStorage.TYPE_REFERENCE = &mLoopNumber;
	signatureLoopCount.mType = Datum::TYPE_INT;
	
	Populate();
	mLoopNumber = NULL;
	mPreamble = NULL;
	mLoopProcess = NULL;
}

ActionListWhile::~ActionListWhile()
{
	mPreamble = NULL;
	mLoopProcess = NULL;
	mLoopNumber = NULL;
}

void ActionListWhile::Process(WorldState * currentWorldState)
{
	if(LinkActionParameters()){
		if(mLoopNumber->Size() > 0){
			// Preamble
			cout<<"While Loop Preamble\n";
			if(mPreamble->Is("Action")){ 
				Action * actionPreamble = mPreamble->As<Action>();
				actionPreamble->Process(currentWorldState);
			}

			//Loop Count
			int currentLoopValue = mLoopNumber->Get<int>(0);

			// Loop Process
			Action * loopProcess = NULL;
			if(mLoopProcess->Is("Action")){ 
				loopProcess = mLoopProcess->As<Action>();
			}

			//Actual Loop
			if(loopProcess != NULL){
				while(currentLoopValue > 0){
					loopProcess->Process(currentWorldState);

					//Getting Next Value
					mLoopNumber->Set((currentLoopValue - 1), 0);
					currentLoopValue = mLoopNumber->Get<int>(0);
				}
			}
		}
	}
}

bool ActionListWhile::LinkActionParameters()
{
	if((mLoopNumber == NULL) || (mPreamble == NULL) || (mLoopProcess == NULL))
	{
		if(mLoopNumber == NULL){
			if(mLoopNumberName.size() > 0){
				Datum * newLoopNumber =  Search(mLoopNumberName);
				if(newLoopNumber->GetType() == Datum::TYPE_INT){
					mLoopNumber = newLoopNumber;
				}
				else
					return false;
			}
			else
				return false;
		}
		if(mPreamble == NULL){
			if(mPreambleName.size() > 0){
				Datum * newPreamble =  Search(mPreambleName);
				if(newPreamble->GetType() == Datum::TYPE_SCOPE){
					mPreamble = newPreamble->Get<Datum::ScopeType>();
				}
				else
					return false;
			}
			else
				return false;
		}
		if(mLoopProcess == NULL){
			if(mLoopProcessName.size() > 0){
				Datum * newLoopProcess =  Search(mLoopProcessName);
				if(newLoopProcess ->GetType() == Datum::TYPE_SCOPE){
					mLoopProcess = newLoopProcess->Get<Datum::ScopeType>();
				}
				else
					return false;
			}
			else
				return false;
		}
		return true;
	}
	return true;
}

bool ActionListWhile::SetupAction(SList< pair<std::string, std::string> > * attributes)
{
	for(SList< pair<std::string, std::string> >::SListIterator mySListIterator = attributes->Begin(); mySListIterator != attributes->End(); mySListIterator++){
		if((*mySListIterator).first == "preambleName")
			mPreambleName = (*mySListIterator).second;
		else if((*mySListIterator).first == "loopProcessName")
			mLoopProcessName = (*mySListIterator).second;
		else if((*mySListIterator).first == "loopNumberName")
			mLoopNumberName = (*mySListIterator).second;
	}
	if((mPreambleName.size() > 0) && (mLoopProcessName.size() > 0) && (mLoopNumberName.size() > 0))
		return true;
	return false;
}