#include "Attributed.h"
#include "RTTI.h"
#include "vec4.h"
#include "mat4.h"


RTTI_DEFINITIONS(Attributed)

Attributed::Attributed()
{
}

Attributed::~Attributed()
{	
}

Attributed::Attributed(const Attributed& source)
{
	operator=(source);
}


Attributed& Attributed::operator=(const Attributed& source)
{
	mSignatures = source.mSignatures;
	return(*this);
}

bool Attributed::IsPrescribedAttribute(const std::string & checkedAttribute)
{
	if(IsAttribute(checkedAttribute)){
		if(GetEntryIndex(checkedAttribute) < mAuxBegin)
			return(true);
	}
	return false;
}

bool Attributed::IsAuxiliaryAttribute(const std::string & checkedAttribute)
{
	if(IsAttribute(checkedAttribute)){
		if(GetEntryIndex(checkedAttribute) >= mAuxBegin)
			return true;
	}
	return false;
}

bool Attributed::IsAttribute(const std::string & checkedAttribute)
{
	if(this->Find(checkedAttribute))
		return(true);
	return(false);
}

Datum& Attributed::AppendAuxiliaryAttribute(const std::string & checkedAttribute)
{
	assert( IsAuxiliaryAttribute(checkedAttribute) || (Find(checkedAttribute) == NULL) );
	Datum & datum = Append(checkedAttribute);	
	return(datum);
}

const unsigned int Attributed::AuxBegin() const
{
	return(mAuxBegin);
}

void Attributed::Populate()
{
	//	Create a special attribute (i.e. append a Datum to the Scope) with the name This, with type RTTI, 
	//whose value is the address of this object.  This must be the first attribute in the table.
	
	Datum& datum = Append("This");
	datum.SetType(Datum::TYPE_RTTI);
	datum.Set( (RTTI*)(QueryInterface(this->TypeIdInstance()) ) );
	mAuxBegin = mSignatures.Size() + Size();
	
	//For each signature provided by Get Signature,
	//Append a Datum of the appropriate type inside this Scope associated with this object.
	//If the signature includes an external storage address, have the created Datum use that, and mark the Datum as using external storage.  This allows the Datum to wrap members of this object without usurping memory management.
	for(unsigned int i = 0; i < mSignatures.Size(); i++){
		Datum& newDatum = Append((mSignatures[i].mName));
		newDatum.SetType(mSignatures[i].mType);
		if(mSignatures[i].mStorage.TYPE_UNKNOWN == NULL){ // internal storage
			newDatum.SetNumValues(mSignatures[i].mNumElements);
			for( unsigned int j = 0; j < mSignatures[i].mNumElements; j++){
				switch ( mSignatures[i].mType )
				{
					case Datum::TYPE_UNKNOWN:
						break;
					case Datum::TYPE_INT:
						newDatum.Set(*(static_cast<int*>(mSignatures[i].mInit)), j);
						break;
					case Datum::TYPE_FLOAT:
						newDatum.Set(*(static_cast<float*>(mSignatures[i].mInit)), j);
						break;
					case Datum::TYPE_VECTOR:
						newDatum.Set(*(static_cast<Vec4*>(mSignatures[i].mInit)), j);
						break;
					case Datum::TYPE_MATRIX:
						newDatum.Set(*(static_cast<Mat4*>(mSignatures[i].mInit)), j);
						break;
					case Datum::TYPE_REFERENCE:
						newDatum.Set(*(static_cast<Datum::ReferenceType*>(mSignatures[i].mInit)), j);
						break;
					case Datum::TYPE_SCOPE:
						{
							//Currently have an issue with having mutliple scopes point towards the same scope. When deleting will try to delete the scope twice.
							Datum::ScopeType scopeSource = static_cast<Datum::ScopeType>(mSignatures[i].mInit);

							Scope * newScope = new Scope();
				
							newDatum.Set(newScope, j);

							Datum::ScopeType tempScope = newDatum.Get<Datum::ScopeType>(j);
							tempScope->CopyValues(*scopeSource, true);
							tempScope = NULL;
						}
						break;
					case Datum::TYPE_STRING:
						newDatum.Set(*(static_cast<std::string*>(mSignatures[i].mInit)), j);
						break;
					case Datum::TYPE_POINTER:
						newDatum = mSignatures[i].mInit;
						break;
					case Datum::TYPE_RTTI:
						newDatum.Set(*(static_cast<Datum::RTTIType*>(mSignatures[i].mInit)), j);
						break;
					default:
						break;
				}
			}
		}
		else { // external storage
			newDatum.SetStorage(mSignatures[i].mStorage.TYPE_POINTER, mSignatures[i].mNumElements, mSignatures[i].mNumElements);
		}
	}
}
