#include "Datum.h"

//! Constructor
/*!
	A constructor.
*/
Datum::Datum() : mSize(0), mCapacity(0), mType(Datum::TYPE_UNKNOWN), mIsExternal(false)
{
	mData.TYPE_UNKNOWN = 0;
	mType = TYPE_UNKNOWN;
}

//! Deconstructor
/*!
	A deconstructor
*/
Datum::~Datum() 
{
	Clear();
}

//! Copy Constructor
/*!
	Copies one datum to another
	\param - the Datum source
*/
Datum::Datum(const Datum& source) : mSize(0), mCapacity(0), mType(Datum::TYPE_UNKNOWN), mIsExternal(false)
{
	this->operator=(source);
}

//! = operator overload
/*!
	Overloads the = operator
	\param - the Datum source
	\return - A reference to the new Datum
*/
Datum& Datum::operator=(const Datum& source)
{	
	// we must clear the memory if we have some
	if(mIsExternal == false)
		this->~Datum();

	mType = source.mType;
	if(source.mIsExternal == false){ // becoming internal
		if(mIsExternal == true){
			mData.TYPE_UNKNOWN = NULL;
			mSize = 0;
			mCapacity = 0;
		}
		mIsExternal = false;
		// allocate a new values array in the destination object that holds the addresses
		this->SetNumValues(source.Size()); // Setting this's amount of memory to the correct amount
		
		// Deep Copy
		for(unsigned int i = 0; i < source.Size(); i++){
			if(source.mData.TYPE_UNKNOWN){
				switch ( mType )
				{
					case TYPE_UNKNOWN:
						break;
					case TYPE_INT:
						mData.TYPE_INT[i] = source.mData.TYPE_INT[i];
						break;
					case TYPE_FLOAT:
						mData.TYPE_FLOAT[i] = source.mData.TYPE_FLOAT[i];
						break;
					case TYPE_VECTOR:
						mData.TYPE_VECTOR[i] = source.mData.TYPE_VECTOR[i];
						break;
					case TYPE_MATRIX:
						mData.TYPE_MATRIX[i] = source.mData.TYPE_MATRIX[i];
						break;
					case TYPE_REFERENCE:
						mData.TYPE_REFERENCE[i] = source.mData.TYPE_REFERENCE[i];
						break;
					case TYPE_SCOPE:
						mData.TYPE_SCOPE[i] = source.mData.TYPE_SCOPE[i]; //Special Case - This a deep copy need a deep deep copy
						break;
					case TYPE_STRING:
						mData.TYPE_STRING[i] = source.mData.TYPE_STRING[i];
						break;
					case TYPE_POINTER:
						mData.TYPE_POINTER = source.mData.TYPE_POINTER;
						break;
					case TYPE_RTTI:
						mData.TYPE_RTTI[i] = source.mData.TYPE_RTTI[i];
						break;
					default:
						break;
				}
			}
		}
	}
	else if(source.mIsExternal){ // becoming external
		mIsExternal = true;
		this->mData.TYPE_UNKNOWN = source.mData.TYPE_UNKNOWN;
	}
	mSize = source.mSize;
	mCapacity = source.mCapacity;
	return(*this);
}

//! SetType
/*!
	Sets the type of data that the Datum has
*/
void Datum::SetType(const TypeE newType)
{
	mType = newType;
}

//! GetType
/*!
	Gets the type of data that the Datum has
	\preturn - the enum of the type of data that we are storing
*/
const Datum::TypeE Datum::GetType() const
{
	return(mType);
}


//! SetNumValues
/*!
	Sets the number of elements of data in this datum
	\param - the new size of the datum
*/
void Datum::SetNumValues(unsigned int numberOfValues)
{
	if(mIsExternal == false){
		mSize = numberOfValues;
		if(mSize > mCapacity){ // Need to reserve memory
			unsigned int oldCapacity = mCapacity;
			mCapacity = mSize;
			switch ( mType )
			{
				case TYPE_INT:
					{
						int* oldArray = mData.TYPE_INT;
						mData.TYPE_INT = new int[mCapacity]();
						memcpy( mData.TYPE_INT, oldArray, (oldCapacity * sizeof(int)) );
						delete oldArray;
					}
					break;
				case TYPE_FLOAT:
					{
						float* oldArray = mData.TYPE_FLOAT;
						mData.TYPE_FLOAT = new float[mCapacity]();
						memcpy( mData.TYPE_FLOAT, oldArray, (oldCapacity * sizeof(float)) );
						delete oldArray;
					}
					break;
				case TYPE_VECTOR:
					{
						Vec4* oldArray = mData.TYPE_VECTOR;
						mData.TYPE_VECTOR = new Vec4[mCapacity]();
						for(unsigned int i = 0; i < oldCapacity; i++){ //datum reference
							mData.TYPE_VECTOR[i] = oldArray[i];
						}
						delete oldArray;
					}
					break;
				case TYPE_MATRIX:
					{
						Mat4* oldArray = mData.TYPE_MATRIX;
						mData.TYPE_MATRIX = new Mat4[mCapacity]();
						for(unsigned int i = 0; i < oldCapacity; i++){ //matrix reference
							mData.TYPE_MATRIX[i] = oldArray[i];
						}
						delete oldArray;
					}
					break;
				case TYPE_REFERENCE:
					{
						ReferenceType* oldArray = mData.TYPE_REFERENCE;
						mData.TYPE_REFERENCE = new ReferenceType[mCapacity]();
						for(unsigned int i = 0; i < oldCapacity; i++){ //datum reference
							mData.TYPE_REFERENCE[i] = oldArray[i];
						}
						delete oldArray;
					}
					break;
				case TYPE_SCOPE:
					{
						ScopeType* oldArray = mData.TYPE_SCOPE;
						mData.TYPE_SCOPE = new ScopeType[mCapacity]();
						for(unsigned int i = 0; i < oldCapacity; i++){ //scope reference
							mData.TYPE_SCOPE[i] = oldArray[i];
						}
						delete oldArray;
					}
					break;
				case TYPE_STRING:
					{
						std::string* oldArray = mData.TYPE_STRING;
						mData.TYPE_STRING = new std::string[mCapacity]();
						for(unsigned int i = 0; i < oldCapacity; i++){ //Strings can't use memcpy
							mData.TYPE_STRING[i] = oldArray[i];
						}
						delete oldArray;
					}
					break;
				case TYPE_POINTER:
					break;
				case TYPE_RTTI:
					{
						RTTI** oldArray = mData.TYPE_RTTI;
						mData.TYPE_RTTI = new RTTI*[mCapacity]();
						for(unsigned int i = 0; i < oldCapacity; i++){ //scope reference
							mData.TYPE_RTTI[i] = oldArray[i];
						}
						delete oldArray;
					}
					break;
				default:
					break;
			}
		}
	}
}

//! Size
/*!
	Returns the number of elements of data in this datum
	\return - the number if elements
*/
const unsigned int Datum::Size() const
{
	return(mSize);
}

//! SetStorage
/*!
	Sets the datum to an external datum.
	\param - the array that we are externally pointing at
	\param - the number if values in the array
	\param - the capacity of the array
*/
void Datum::SetStorage(void* array, unsigned int numberOfValues, unsigned int capacity)
{
	assert(mType != TYPE_UNKNOWN);
	assert(mCapacity == 0); // This should be called before any allocations are made, if not something is wrong and we assert
	assert(numberOfValues <= capacity );

	// Set the storage to external
	mIsExternal = true;
	mSize = numberOfValues;
	mCapacity = capacity;

	mData.TYPE_UNKNOWN = array;
}

//! operator == 
/*!
	Compares two Datums
	\param - the Datum to compare to
	\return - a bool indictating if the arrays are equal or not
*/
bool Datum::operator==(const Datum & otherDatum) const
{
	// Check if there types are the same
	if((GetType() == otherDatum.GetType()) && (Size() == otherDatum.Size())){
		for(unsigned int i = 0; i < mSize; i++){
			if((mData.TYPE_INT) && (otherDatum.mData.TYPE_INT)){
				switch ( mType )
				{
					case TYPE_UNKNOWN:
							return(false);
						break;
					case TYPE_INT:
						if(mData.TYPE_INT[i] != otherDatum.mData.TYPE_INT[i])
							return(false);
						break;
					case TYPE_FLOAT:
						if(mData.TYPE_FLOAT[i] != otherDatum.mData.TYPE_FLOAT[i])
							return(false);
						break;
					case TYPE_VECTOR:
						if(mData.TYPE_VECTOR[i] != otherDatum.mData.TYPE_VECTOR[i])
							return(false);
						break;
					case TYPE_MATRIX:
						if(mData.TYPE_MATRIX[i] != otherDatum.mData.TYPE_MATRIX[i])
							return(false);
						break;
					case TYPE_REFERENCE:
						if(mData.TYPE_REFERENCE[i] != otherDatum.mData.TYPE_REFERENCE[i])
							return(false);
						break;
					case TYPE_SCOPE:
						if((mData.TYPE_SCOPE[i]) != (otherDatum.mData.TYPE_SCOPE[i])) // Special Case
							return(false);
						break;
					case TYPE_STRING:
						if(mData.TYPE_STRING[i] != otherDatum.mData.TYPE_STRING[i])
							return(false);
						break;
					case TYPE_POINTER:
						if(mData.TYPE_POINTER != otherDatum.mData.TYPE_POINTER)
							return(false);
						break;
					default:
						break;
				}
			}
		}
	}
	else
		return(false);
	return(true);
}

//! operator != 
/*!
	Compares to Datums
	\param - the Datum to compare to
	\return - a bool that relates to if the datums are equal or not
*/
bool Datum::operator!=(const Datum & otherDatum) const
{
	return(!(operator==(otherDatum)));
}


/******************************************************************
	Public value accessors
*******************************************************************/


/*************************************
	Set functions
**************************************/
//Provide one signature for each type


//! Set (int overload)
/*!
	Sets the data to the corresponding spot in the datum, int
	\param - the data to insert
	\param - the spot to insert at
*/
void Datum::Set(int &data, unsigned int index)
{
	if(mType == TYPE_UNKNOWN) //Setting the default type if it has not been set yet
		mType = TYPE_INT;
	assert(mType == TYPE_INT); // In debug builds, verify that the given value type matches the type of this object
	if(index >= mSize)
		SetNumValues(index + 1); // Handle possible resizing
	mData.TYPE_INT[index] = data;
}


//! Set (const int overload)
/*!
	Sets the data to the corresponding spot in the datum, int
	\param - the data to insert
	\param - the spot to insert at
*/
void Datum::Set(const int &data, unsigned int index)
{
	if(mType == TYPE_UNKNOWN) //Setting the default type if it has not been set yet
		mType = TYPE_INT;
	assert(mType == TYPE_INT); // In debug builds, verify that the given value type matches the type of this object
	if(index >= mSize)
		SetNumValues(index + 1); // Handle possible resizing
	mData.TYPE_INT[index] = data;
}

//! Set (float overload)
/*!
	Sets the data to the corresponding spot in the datum, float
	\param - the data to insert
	\param - the spot to insert at
*/
void Datum::Set(float &data, unsigned int index)
{
	if(mType == TYPE_UNKNOWN) //Setting the default type if it has not been set yet
		mType = TYPE_FLOAT;
	assert(mType == TYPE_FLOAT); // In debug builds, verify that the given value type matches the type of this object
	if(index >= mSize)
		SetNumValues(index + 1); // Handle possible resizing
	mData.TYPE_FLOAT[index] = data;
}

//! Set (const float overload)
/*!
	Sets the data to the corresponding spot in the datum, float
	\param - the data to insert
	\param - the spot to insert at
*/
void Datum::Set(const float &data, unsigned int index)
{
	if(mType == TYPE_UNKNOWN) //Setting the default type if it has not been set yet
		mType = TYPE_FLOAT;
	assert(mType == TYPE_FLOAT); // In debug builds, verify that the given value type matches the type of this object
	if(index >= mSize)
		SetNumValues(index + 1); // Handle possible resizing
	mData.TYPE_FLOAT[index] = data;
}

//! Set (Vec4 overload)
/*!
	Sets the data to the corresponding spot in the datum, Vec4
	\param - the data to insert
	\param - the spot to insert at
*/
void Datum::Set(Vec4 &data, unsigned int index)
{
	if(mType == TYPE_UNKNOWN) //Setting the default type if it has not been set yet
		mType = TYPE_VECTOR;
	assert(mType == TYPE_VECTOR); // In debug builds, verify that the given value type matches the type of this object
	if(index >= mSize)
		SetNumValues(index + 1); // Handle possible resizing
	// Storage
	mData.TYPE_VECTOR[index] = data;
}

//! Set (const Vec4 overload)
/*!
	Sets the data to the corresponding spot in the datum, Vec4
	\param - the data to insert
	\param - the spot to insert at
*/
void Datum::Set(const Vec4 &data, unsigned int index)
{
	if(mType == TYPE_UNKNOWN) //Setting the default type if it has not been set yet
		mType = TYPE_VECTOR;
	assert(mType == TYPE_VECTOR); // In debug builds, verify that the given value type matches the type of this object
	if(index >= mSize)
		SetNumValues(index + 1); // Handle possible resizing
	// Storage
	mData.TYPE_VECTOR[index] = data;
}


//! Set (Mat4 overload)
/*!
	Sets the data to the corresponding spot in the datum, Mat4
	\param - the data to insert
	\param - the spot to insert at
*/
void Datum::Set(Mat4 &data, unsigned int index)
{
	if(mType == TYPE_UNKNOWN) //Setting the default type if it has not been set yet
		mType = TYPE_MATRIX;
	assert(mType == TYPE_MATRIX); // In debug builds, verify that the given value type matches the type of this object
	if(index >= mSize)
		SetNumValues(index + 1); // Handle possible resizing
	// Storage
	mData.TYPE_MATRIX[index] = data;
}

//! Set (const Mat4 overload)
/*!
	Sets the data to the corresponding spot in the datum, Mat4
	\param - the data to insert
	\param - the spot to insert at
*/
void Datum::Set(const Mat4 &data, unsigned int index)
{
	if(mType == TYPE_UNKNOWN) //Setting the default type if it has not been set yet
		mType = TYPE_MATRIX;
	assert(mType == TYPE_MATRIX); // In debug builds, verify that the given value type matches the type of this object
	if(index >= mSize)
		SetNumValues(index + 1); // Handle possible resizing
	// Storage
	mData.TYPE_MATRIX[index] = data;
}

//! Set (reference overload)
/*!
	Sets the data to the corresponding spot in the datum, reference
	\param - the data to insert
	\param - the spot to insert at
*/
void Datum::Set(ReferenceType &data, unsigned int index )
{
	if(mType == TYPE_UNKNOWN) //Setting the default type if it has not been set yet
		mType = TYPE_REFERENCE;
	assert(mType == TYPE_REFERENCE); // In debug builds, verify that the given value type matches the type of this object
	if(index >= mSize)
		SetNumValues(index + 1); // Handle possible resizing
	// Storage since this is a reference it will always be external
	mData.TYPE_REFERENCE[index] = data;
}

//! Set (const reference overload)
/*!
	Sets the data to the corresponding spot in the datum, reference
	\param - the data to insert
	\param - the spot to insert at
*/
void Datum::Set(const ReferenceType &data, unsigned int index )
{
	if(mType == TYPE_UNKNOWN) //Setting the default type if it has not been set yet
		mType = TYPE_REFERENCE;
	assert(mType == TYPE_REFERENCE); // In debug builds, verify that the given value type matches the type of this object
	if(index >= mSize)
		SetNumValues(index + 1); // Handle possible resizing
	// Storage since this is a reference it will always be external
	mData.TYPE_REFERENCE[index] = data;
}

//! Set (const reference overload)
/*!
	Sets the data to the corresponding spot in the datum, Scope
	\param - the data to insert
	\param - the spot to insert at
*/
void Datum::Set(ScopeType &data, unsigned int index )
{
	if(mType == TYPE_UNKNOWN) //Setting the default type if it has not been set yet
		mType = TYPE_SCOPE;
	assert(mType == TYPE_SCOPE); // In debug builds, verify that the given value type matches the type of this object
	if(index >= mSize)
		SetNumValues(index + 1); // Handle possible resizing
	// Storage since this is a reference it will always be external
	mData.TYPE_SCOPE[index] = data;
}

//! Set (const reference overload)
/*!
	Sets the data to the corresponding spot in the datum, Scope
	\param - the data to insert
	\param - the spot to insert at
*/
void Datum::Set(const ScopeType &data, unsigned int index )
{
	if(mType == TYPE_UNKNOWN) //Setting the default type if it has not been set yet
		mType = TYPE_SCOPE;
	assert(mType == TYPE_SCOPE); // In debug builds, verify that the given value type matches the type of this object
	if(index >= mSize)
		SetNumValues(index + 1); // Handle possible resizing
	// Storage since this is a reference it will always be external
	mData.TYPE_SCOPE[index] = data;
}

//! Set (string overload)
/*!
	Sets the data to the corresponding spot in the datum, string
	\param - the data to insert
	\param - the spot to insert at
*/
void Datum::Set(std::string &data, unsigned int index )
{
	if(mType == TYPE_UNKNOWN) //Setting the default type if it has not been set yet
		mType = TYPE_STRING;
	assert(mType == TYPE_STRING); // In debug builds, verify that the given value type matches the type of this object
	if(index >= mSize)
		SetNumValues(index + 1); // Handle possible resizing
	// Storage
	mData.TYPE_STRING[index] = data;
}

//! Set (const string overload)
/*!
	Sets the data to the corresponding spot in the datum, string
	\param - the data to insert
	\param - the spot to insert at
*/
void Datum::Set(const std::string &data, unsigned int index ) 
{
	if(mType == TYPE_UNKNOWN) //Setting the default type if it has not been set yet
		mType = TYPE_STRING;
	assert(mType == TYPE_STRING); // In debug builds, verify that the given value type matches the type of this object
	if(index >= mSize)
		SetNumValues(index + 1); // Handle possible resizing
	// Storage
	mData.TYPE_STRING[index] = data;
}

//! Set (pointer overload)
/*!
	Sets the data to the corresponding spot in the datum, int
	\param - the data to insert
	\param - the spot to insert at
*/
void Datum::Set(const PointerType data)
{
	if(mType == TYPE_UNKNOWN) //Setting the default type if it has not been set yet
		mType = TYPE_POINTER;
	assert(mType == TYPE_POINTER); // In debug builds, verify that the given value type matches the type of this object
	// Storage - since this is a reference it will always be external
	mData.TYPE_POINTER = data;
}

// rtti
void Datum::Set(RTTIType data, unsigned int index )
{
	if(mType == TYPE_UNKNOWN) //Setting the default type if it has not been set yet
		mType = TYPE_RTTI;
	assert(mType == TYPE_RTTI); // In debug builds, verify that the given value type matches the type of this object
	if(index >= mSize)
		SetNumValues(index + 1); // Handle possible resizing
	// Storage - since this is a reference it will always be external
	mData.TYPE_RTTI[index] = data;
}






/*************************************
	operator = overloads for each primitive type. 
**************************************/
//Provide one signature for each type

//! operator = overload (int overload)
/*!
	Sets the assigns the data to the datum, and sets its size to 1 as if it were a scalar
	\param - the data to set the datum equal to
*/
Datum& Datum::operator=( const int & iVal ) { 
	Set( iVal ) ; SetNumValues(1); return * this ; 
}

//! operator = overload (float overload)
/*!
	Sets the assigns the data to the datum, and sets its size to 1 as if it were a scalar
	\param - the data to set the datum equal to
*/
Datum& Datum::operator=( const float & fVal ) {
	Set( fVal ) ; SetNumValues(1); return * this ; 
}

//! operator = overload (Vec4 overload)
/*!
	Sets the assigns the data to the datum, and sets its size to 1 as if it were a scalar
	\param - the data to set the datum equal to
*/
Datum& Datum::operator=( const Vec4 & vVal ) {
	Set( vVal ) ; SetNumValues(1); return * this ; 
}

//! operator = overload (Mat4 overload)
/*!
	Sets the assigns the data to the datum, and sets its size to 1 as if it were a scalar
	\param - the data to set the datum equal to
*/
Datum& Datum::operator=( const Mat4 & mVal ) {
	Set( mVal ) ; SetNumValues(1); return * this ; 
}

//! operator = overload (Reference overload)
/*!
	Sets the assigns the data to the datum, and sets its size to 1 as if it were a scalar
	\param - the data to set the datum equal to
*/
Datum& Datum::operator=( const ReferenceType & rVal ) { 
	Set( rVal ) ; SetNumValues(1); return * this ; 
}

//! operator = overload (Scope* overload)
/*!
	Sets the assigns the data to the datum, and sets its size to 1 as if it were a scalar
	\param - the data to set the datum equal to
*/
Datum& Datum::operator=( const ScopeType & sVal ) { 
	Set( sVal ) ; SetNumValues(1); return * this ; 
}

//! operator = overload (string overload)
/*!
	Sets the assigns the data to the datum, and sets its size to 1 as if it were a scalar
	\param - the data to set the datum equal to
*/
Datum& Datum::operator=( const std::string & sVal ) { 
	Set( sVal ) ; SetNumValues(1); return * this ; 
}

//! operator = overload (pointer overload)
/*!
	Sets the assigns the data to the datum, and sets its size to 1 as if it were a scalar
	\param - the data to set the datum equal to
*/
Datum& Datum::operator=( const PointerType pVal ) { 
	Set( pVal ) ; SetNumValues(1); return * this ; 
}

/*************************************
	operator == overloads for each primitive type. 
**************************************/
//Provide one signature for each type



//! operator == overload (int overload)
/*!
	Checks if the datum is equal to the scaler
	\param - the data to set the datum equal to
	\return - if datum is equal to the scalar
*/
bool Datum::operator==( const int & that ) const { 
	return (( Size() == 1 ) && ( Get<int>()  == that )); 
}

//! operator != overload (int overload)
/*!
	Checks if the datum is equal to the scaler
	\param - the data to set the datum equal to
	\return - if datum is equal to the scalar
*/
bool Datum::operator!=( const int & that ) const { 
	return ! operator==( that ) ; 
}



//! operator == overload (float overload)
/*!
	Checks if the datum is equal to the scaler
	\param - the data to set the datum equal to
	\return - if datum is equal to the scalar
*/
bool Datum::operator==( const float & that ) const { 
	return ( Size() == 1 ) && ( Get<float>() == that ) ; 
}

//! operator != overload (float overload)
/*!
	Checks if the datum is equal to the scaler
	\param - the data to set the datum equal to
	\return - if datum is equal to the scalar
*/
bool Datum::operator!=( const float & that ) const { 
	return ! operator==( that ) ;
}


//! operator == overload (Vec4 overload)
/*!
	Checks if the datum is equal to the scaler
	\param - the data to set the datum equal to
	\return - if datum is equal to the scalar
*/
bool Datum::operator==( const Vec4 & that ) const {
	return ( Size() == 1 ) && ( Get<Vec4>() == that ) ; 
}

//! operator != overload (Vec4 overload)
/*!
	Checks if the datum is equal to the scaler
	\param - the data to set the datum equal to
	\return - if datum is equal to the scalar
*/
bool Datum::operator!=( const Vec4 & that ) const {
	return ! operator==( that ) ;
}


//! operator == overload (Mat4 overload)
/*!
	Checks if the datum is equal to the scaler
	\param - the data to set the datum equal to
	\return - if datum is equal to the scalar
*/
bool Datum::operator==( const Mat4 & that ) const {
	return ( Size() == 1 ) && ( Get<Mat4>() == that ) ; 
}

//! operator != overload (Mat4 overload)
/*!
	Checks if the datum is equal to the scaler
	\param - the data to set the datum equal to
	\return - if datum is equal to the scalar
*/
bool Datum::operator!=( const Mat4 & that ) const {
	return ! operator==( that ) ;
}


//! operator == overload (reference overload)
/*!
	Checks if the datum is equal to the scaler
	\param - the data to set the datum equal to
	\return - if datum is equal to the scalar
*/
bool Datum::operator==( const ReferenceType& that ) const { 
	return ( Size() == 1 ) && ( Get<ReferenceType>() == that ) ; 
}

//! operator != overload (reference overload)
/*!
	Checks if the datum is equal to the scaler
	\param - the data to set the datum equal to
	\return - if datum is equal to the scalar
*/
bool Datum::operator!=( const ReferenceType & that ) const { 
	return ! operator==( that ) ;
}



//! operator == overload (Scope* overload)
/*!
	Checks if the datum is equal to the scaler
	\param - the data to set the datum equal to
	\return - if datum is equal to the scalar
*/
bool Datum::operator==( const ScopeType & that ) const{
	return ( Size() == 1 ) && ( Get<ScopeType>() == that ) ; 
}

//! operator != overload (Scope* overload)
/*!
	Checks if the datum is equal to the scaler
	\param - the data to set the datum equal to
	\return - if datum is equal to the scalar
*/
bool Datum::operator!=( const ScopeType & that ) const{
	return ! operator==( that ) ;
}


//! operator == overload (string overload)
/*!
	Checks if the datum is equal to the scaler
	\param - the data to set the datum equal to
	\return - if datum is equal to the scalar
*/
bool Datum::operator==( const std::string & that ) const{
	return ( Size() == 1 ) && ( Get<std::string>() == that ) ; 
}

//! operator != overload (string overload)
/*!
	Checks if the datum is equal to the scaler
	\param - the data to set the datum equal to
	\return - if datum is equal to the scalar
*/
bool Datum::operator!=( const std::string & that ) const{
	return ! operator==( that ) ;
}


/*************************************
	Static utility routines
**************************************/

//! StringFromType
/*!
	given type enum, return string name.
	\param - the enum of the type
	\return - a string if the type
*/
std::string Datum::StringFromType(TypeE enumType)
{
	switch(enumType)
	{
		case TYPE_INT: return "TYPE_INT";
		case TYPE_FLOAT: return "TYPE_FLOAT";
		case TYPE_VECTOR: return "TYPE_VECTOR";
		case TYPE_MATRIX: return "TYPE_MATRIX";
		case TYPE_REFERENCE: return "TYPE_REFERENCE";
		case TYPE_SCOPE: return "TYPE_SCOPE";
		case TYPE_STRING: return "TYPE_STRING";
		case TYPE_POINTER: return "TYPE_POINTER";
		case TYPE_RTTI: return "TYPE_RTTI";
		default: return "TYPE_UNKNOWN";
	}
}

//! TypeFromString
/*!
	given type name, return enum.
	\param - a string of the type
	\return - the enum of the type
*/
Datum::TypeE Datum::TypeFromString(std::string typeName)
{
	if(typeName == "TYPE_INT")
		return TYPE_INT;
	else if(typeName == "TYPE_FLOAT")
		return TYPE_FLOAT;
	else if(typeName == "TYPE_VECTOR")
		return TYPE_VECTOR;
	else if(typeName == "TYPE_MATRIX")
		return TYPE_MATRIX;
	else if(typeName == "TYPE_REFERENCE")
		return TYPE_REFERENCE;
	else if(typeName == "TYPE_SCOPE")
		return TYPE_SCOPE;
	else if(typeName == "TYPE_STRING")
		return TYPE_STRING;
	else if(typeName == "TYPE_POINTER")
		return TYPE_POINTER;
	else if(typeName == "TYPE_RTTI")
		return TYPE_RTTI;
	else
		return TYPE_UNKNOWN;
}

//! HasValue
/*!
	Checks if the datum has a value at that location
	\param - a string of the type
	\return - the enum of the type
*/
bool Datum::HasValue(Datum& myDatum, unsigned int index)
{
	if((index < myDatum.Size())&&(myDatum.mData.TYPE_POINTER))
		return(true);
	return(false);
}


//! ReferenceHasValue
/*!
	Checks if the datum is a refernece and it has a value in it
	\param - the datum to check
	\param - the index in the datum to check
	\return - a bool indicating if there is a value at that spot
*/
bool Datum::ReferenceHasValue(Datum& myDatum, unsigned int index)
{
	if(myDatum.GetType() == Datum::TYPE_REFERENCE){
		if((index < myDatum.Size())&&(myDatum.mData.TYPE_REFERENCE[index]->Size() > 0))
			return(true);
	}
	return(false);
}



//! Clear
/*!
	Clears the datum
*/
void Datum::Clear()
{
	if(mIsExternal == false) { // if we own memory then we should delete it
		switch ( mType )
		{
			case TYPE_UNKNOWN:
				break;
			case TYPE_INT:
				delete[] mData.TYPE_INT;
				break;
			case TYPE_FLOAT:
				delete[] mData.TYPE_FLOAT;
				break;
			case TYPE_VECTOR:
				delete[] mData.TYPE_VECTOR;
				break;
			case TYPE_MATRIX:
				delete[] mData.TYPE_MATRIX;
				break;
			case TYPE_REFERENCE:
				delete[] mData.TYPE_REFERENCE;
				break;
			case TYPE_SCOPE:
				delete[] mData.TYPE_SCOPE;
				break;
			case TYPE_STRING:
				delete[] mData.TYPE_STRING;
				break;
			case TYPE_POINTER:
				break;
			case TYPE_RTTI:
				delete[] mData.TYPE_RTTI;
				break;
			default:
				break;
		}
	}
	mSize = 0;
	mCapacity = 0;
	mData.TYPE_UNKNOWN = NULL;
}


void Datum::RemoveIndex(const unsigned int index)
{
	if(index < mSize){
		if(mType != TYPE_UNKNOWN){	
			if(mIsExternal == false){
				switch ( mType )
				{
					case TYPE_UNKNOWN:
						break;
					case TYPE_INT:
						break;
					case TYPE_FLOAT:
						break;
					case TYPE_VECTOR:
						break;
					case TYPE_MATRIX:
						break;
					case TYPE_REFERENCE:
						delete mData.TYPE_REFERENCE[index];
						break;
					case TYPE_SCOPE:
						mData.TYPE_SCOPE[index] = NULL;
						break;
					case TYPE_STRING:
						break;
					case TYPE_POINTER:
						break;
					case TYPE_RTTI:
						break;
					default:
						break;
				}
			} 
			// shifting the list
			for(unsigned int i = index; i < (mSize-1); i++){
				switch ( mType )
				{
					case TYPE_UNKNOWN:
						break;
					case TYPE_INT:
						mData.TYPE_INT[i] = mData.TYPE_INT[i + 1];
						break;
					case TYPE_FLOAT:
						mData.TYPE_FLOAT[i] = mData.TYPE_FLOAT[i + 1];
						break;
					case TYPE_VECTOR:
						mData.TYPE_VECTOR[i] = mData.TYPE_VECTOR[i + 1];
						break;
					case TYPE_MATRIX:
						mData.TYPE_MATRIX[i] = mData.TYPE_MATRIX[i + 1];
						break;
					case TYPE_REFERENCE:
						mData.TYPE_REFERENCE[i] = mData.TYPE_REFERENCE[i + 1];
						break;
					case TYPE_SCOPE:
						mData.TYPE_SCOPE[i] = mData.TYPE_SCOPE[i + 1]; //Special Case - This a deep copy need a deep deep copy
						break;
					case TYPE_STRING:
						mData.TYPE_STRING[i] = mData.TYPE_STRING[i + 1];
						break;
					case TYPE_POINTER:
						mData.TYPE_POINTER = mData.TYPE_POINTER;
						break;
					case TYPE_RTTI:
						mData.TYPE_RTTI[i] = mData.TYPE_RTTI[i + 1]; //Special Case - This a deep copy need a deep deep copy
						break;
					default:
						break;
				}
			}
			if(mSize > 0)
				mSize--;
		}
	}
}

/**int Datum::GetAsInt( std::string value ) { 
	assert(value.empty() == false);
	return (atoi(value.c_str()));
}*/