#include "EventPublisher.h"

RTTI_DEFINITIONS(EventPublisher)

EventPublisher::EventPublisher(SList<IEventSubscriber*> & newSubscribersList, const bool deleteEvent, const double enqueuedTime, const double delay) : mSubscribers(newSubscribersList)
{
	mDeleteEvent = deleteEvent;
	mEnqueuedTime = enqueuedTime;
	mDelay = delay;
}

EventPublisher::EventPublisher(const EventPublisher & source) : mSubscribers(source.mSubscribers)
{
	(*this) = source;
}

EventPublisher & EventPublisher::operator=(const EventPublisher & source)
{
	mSubscribers = source.mSubscribers;
	mDeleteEvent = source.mDeleteEvent;
	mEnqueuedTime = source.mEnqueuedTime;
	mDelay = source.mDelay;
	return(*this);
}

EventPublisher::~EventPublisher()
{
}

void EventPublisher::SetTime(const double newEnqueuedTime, const double newDelay)
{
	mEnqueuedTime = newEnqueuedTime;
	mDelay = newDelay;
}

double EventPublisher::GetTimeEnqueued() const
{
	return(mEnqueuedTime);
}

double EventPublisher::GetDelay() const
{
	return(mDelay);
}

bool EventPublisher::IsExpired(const double currentTime)
{
	if(currentTime > (mEnqueuedTime + mDelay))
		return true;
	return false;
}

void EventPublisher::Deliver()
{
	// notify all subscribers of this event
	for(SList<IEventSubscriber*>::SListIterator subscriberIterator = mSubscribers.Begin(); subscriberIterator != mSubscribers.End(); subscriberIterator++){
		IEventSubscriber * subscriber = (*subscriberIterator);
		subscriber->Notify(this);
	}
}

bool EventPublisher::DeleteAfterPublishing() const
{
	return(mDeleteEvent);
}