#include "EventQueue.h"

void EventQueue::Enqueue(EventPublisher * newEventPublisher, double currentTime, double delay)
{
	mCurrentTime = currentTime;
	if(newEventPublisher){
		newEventPublisher->SetTime(currentTime, delay);
		if(delay == 0.0f) // Synchronous
		{
			newEventPublisher->Deliver();
		}
		else // Asynchronous
		{
			mEvents.PushBack(newEventPublisher);
		}
	}
}

void EventQueue::Publish(const double currentTime)
{
	//SList of events to remove
	SList<EventPublisher*> eventsToRemove;
	
	//publish events 
	for(SList<EventPublisher*>::SListIterator eventIterator = mEvents.Begin(); eventIterator != mEvents.End(); eventIterator++){
		EventPublisher * event = (*eventIterator);
		if(event->IsExpired(currentTime)){
			event->Deliver();
			eventsToRemove.PushBack(event);
		}
	}

	//Remove the events that we need to delete
	for(SList<EventPublisher*>::SListIterator removeIterator = eventsToRemove.Begin(); removeIterator != eventsToRemove.End(); removeIterator++){
		EventPublisher * removedEvent = (*removeIterator);

		//Remove from the event list
		mEvents.Remove(removedEvent);

		//Deleting the event
		if(removedEvent->DeleteAfterPublishing()){
			delete removedEvent;
			removedEvent = 0;
		}
	}
}

void EventQueue::Clear()
{
	mEvents.Clear();
}

