#include "ReactionAttributed.h"

RTTI_DEFINITIONS(ReactionAttributed)

ReactionAttributed::ReactionAttributed()
{
	Signature signatureSubtype;
	signatureSubtype.mNumElements = 1;
	signatureSubtype.mName = "Subtype";
	signatureSubtype.mInit = NULL;
	signatureSubtype.mStorage.TYPE_STRING = &mSubtype;
	signatureSubtype.mType = Datum::TYPE_STRING;

	//Add to the subscriber list
	Event<EventMessageAttributed>::Subscribe(this);
}

ReactionAttributed::~ReactionAttributed()
{
}

void ReactionAttributed::Notify(EventPublisher * eventPublisher)
{
	//Getting the event
	Event<EventMessageAttributed>* event = eventPublisher->As< Event<EventMessageAttributed> >();
	if(event){
		//Getting the message
		EventMessageAttributed eventMessage = event->GetMessage();
		if(mSubtype == eventMessage.GetSubtype())
		{	
			//Getting the world information
			World * pWorld = eventMessage.GetWorld();
			WorldState & worldState = pWorld->GetWorldState();

			//Pusing into the activation record stack
			worldState.mActivationStack.Push(eventMessage.As<Scope>());

			//Call Process
			Process(&worldState);

			//Remove this level of the activation stack
			worldState.mActivationStack.Pop();
		}
	}
}

bool ReactionAttributed::SetupAction(SList< pair<std::string, std::string> > * attributes)
{
	for(SList< pair<std::string, std::string> >::SListIterator mySListIterator = attributes->Begin(); mySListIterator != attributes->End(); mySListIterator++){
		if((*mySListIterator).first == "subtype"){
			mSubtype = (*mySListIterator).second;
		}
	}
	if(mSubtype.size() > 0)
		return true;
	return false;
}
