#include "XmlParseHelperAction.h"

/**********************************
	XmlParseHelperAction
***********************************/

XmlParseHelperAction::XmlParseHelperAction(XmlParseMaster* newParseMaster)
{
	mMaster = newParseMaster;
	if(mMaster)
		mMaster->AddHelper(this);

	//Add Valid Tags
	mValidTags.PushBack("action");
	mValidTags.PushBack("createAction");
	mValidTags.PushBack("destroyAction");
	mValidTags.PushBack("printAction");
	mValidTags.PushBack("loopAction");
}

XmlParseHelperAction::~XmlParseHelperAction()
{
	if(mMaster){
		if(mMaster->GetIsClone() == false){
			mMaster->RemoveHelper(this);
			mMaster = NULL;
		}
	}
}

XmlParseHelperAction* XmlParseHelperAction::Create(XmlParseMaster* referencedParseMaster)
{
	XmlParseHelperAction* newHelper = new XmlParseHelperAction(referencedParseMaster);
	return(newHelper);
}

bool XmlParseHelperAction::StartElementHandler(std::string tagName, SList< pair<std::string, std::string> > attributeValuePairs)
{
	if(CheckIfValidTag(tagName) == false)
		return false;

	//Getting the shared data
	XmlParseHelperScope::SharedDataScope* sharedData = reinterpret_cast<XmlParseHelperScope::SharedDataScope*>(mMaster->GetSharedData());
	
	if(sharedData->Is("XmlParseHelperScope::SharedDataScope"))
	{		
		SList< pair<std::string, std::string> >::SListIterator mySListIterator = attributeValuePairs.Begin();
		
		// Get this entity or action
		if(sharedData->mCurrentScope->Is("ActionList") || sharedData->mCurrentScope->Is("Entity"))
		{		
			if(sharedData->mCurrentScope->Is("Entity")){
				sharedData->mCurrentScope = &sharedData->mCurrentScope->AppendScope("ActionList");
			}

			// General Action Settings
			std::string instanceName;
			unsigned int index = sharedData->mCurrentScope->Size();
			
			//Action Settings
			std::string className;

			if(tagName == "createAction")
				className = std::string("ActionCreateAction");
			else if(tagName == "destroyAction")
				className = std::string("ActionDestroyAction");
			else if(tagName == "printAction")
				className = std::string("ActionPrint");
			else if(tagName == "loopAction")
				className = std::string("ActionListWhile");

			// First Pass for general settings
			for(;mySListIterator != attributeValuePairs.End(); mySListIterator++){
				if((*mySListIterator).first == "instanceName")
					instanceName = (*mySListIterator).second;
				else if((*mySListIterator).first == "class")
						className = (*mySListIterator).second;
				else if((*mySListIterator).first == "index"){
					Datum tempStringDatum;
					tempStringDatum.Set((*mySListIterator).second);
					index = tempStringDatum.GetAs<int>();
				}
			}
				

			// Updating the current scope and create the new action in this entity
			sharedData->mCurrentScope = Action::NewAction(sharedData->mCurrentScope, className, instanceName, index, &attributeValuePairs);
		}
	}
	return false;	
}

bool XmlParseHelperAction::EndElementHandler(std::string tagName)
{
	if(CheckIfValidTag(tagName) == false)
		return false;

	XmlParseHelperScope::SharedDataScope* sharedData = reinterpret_cast<XmlParseHelperScope::SharedDataScope*>(mMaster->GetSharedData());
	if(sharedData->Is("XmlParseHelperScope::SharedDataScope")){
		if(sharedData->mCurrentScope->GetParent() != NULL)
			sharedData->mCurrentScope = sharedData->mCurrentScope->GetParent();
		if(sharedData->mCurrentScope->GetParent() != NULL){
			if(sharedData->mCurrentScope->GetParent()->Is("Entity")){
				if(sharedData->mCurrentScope->GetOwnName() == "ActionList")
					sharedData->mCurrentScope = sharedData->mCurrentScope->GetParent();
			}
		}
		return true;
	}
	return false;
}


void XmlParseHelperAction::CharDataHandler(std::string buffer)
{
}

