#include "XmlParseHelperScope.h"

/**********************************
	SharedData
***********************************/
RTTI_DEFINITIONS(XmlParseHelperScope::SharedDataScope)

XmlParseHelperScope::SharedDataScope::SharedDataScope()
{
	mGlobalScope = new Scope();
	mCurrentScope = mGlobalScope;
}

XmlParseHelperScope::SharedDataScope::SharedDataScope(Scope * newGlobalScope)
{
	mGlobalScope = newGlobalScope;
	mCurrentScope = mGlobalScope;
}

Scope& XmlParseHelperScope::SharedDataScope::GetGlobalScope()
{
	return(*mGlobalScope);
}

/**********************************
	XmlParseHelperScope
***********************************/

XmlParseHelperScope::XmlParseHelperScope(XmlParseMaster* newParseMaster)
{
	mMaster = newParseMaster;
	if(mMaster)
		mMaster->AddHelper(this);

	//Add Valid Tags
	mValidTags.PushBack("TYPE_INT");
	mValidTags.PushBack("TYPE_FLOAT");
	mValidTags.PushBack("TYPE_STRING");
	mValidTags.PushBack("TYPE_VECTOR");
	mValidTags.PushBack("TYPE_MATRIX");
	mValidTags.PushBack("TYPE_SCOPE");
}

XmlParseHelperScope::~XmlParseHelperScope()
{
	if(mMaster){
		if(mMaster->GetIsClone() == false){
			mMaster->RemoveHelper(this);
			mMaster = NULL;
		}
	}
}

XmlParseHelperScope* XmlParseHelperScope::Create(XmlParseMaster* referencedParseMaster)
{
	XmlParseHelperScope* newHelper = new XmlParseHelperScope(referencedParseMaster);
	return(newHelper);
}

bool XmlParseHelperScope::StartElementHandler(std::string tagName, SList< pair<std::string, std::string> > attributeValuePairs)
{
	if(CheckIfValidTag(tagName) == false)
		return false;

	//Is either a new Datum or a new Scope
	XmlParseHelperScope::SharedDataScope* sharedData = reinterpret_cast<XmlParseHelperScope::SharedDataScope*>(mMaster->GetSharedData());
	if(sharedData->Is("XmlParseHelperScope::SharedDataScope")){
		if(tagName != "root"){
			unsigned int insertPosition = 0;

			//Get the type of entry
			Datum::TypeE datumType = Datum::TypeFromString(tagName);

			//Get the name of the entry
			Datum tempStringDatum;
			tempStringDatum.SetType(Datum::TYPE_STRING);
			SList< pair<std::string, std::string> >::SListIterator mySListIterator = attributeValuePairs.Begin();

			std::string newEntryName;
			if((*mySListIterator).first == "name")
				newEntryName = (*mySListIterator).second;
			if(newEntryName.size() == 0)
				return false;

			if(datumType == Datum::TYPE_SCOPE){ // Handling Scope
				sharedData->mCurrentScope = &(sharedData->mCurrentScope->AppendScope(newEntryName));
			}
			else{
				sharedData->mCurrentDatum = &(sharedData->mCurrentScope->Append(newEntryName));
				//Getting the spot to insert the new data at in the datum
				insertPosition = sharedData->mCurrentDatum->Size();
			}

			mySListIterator++;
			for(;mySListIterator != attributeValuePairs.End(); mySListIterator++)
			{
				tempStringDatum.Set((*mySListIterator).second);
				if((*mySListIterator).first == "index"){
					//Getting the spot to insert the new data at in the datum
					insertPosition = tempStringDatum.GetAs<int>();
				}
				else if((*mySListIterator).first == "value")
				{
					switch ( datumType )
					{
						case Datum::TYPE_INT:
							{
								sharedData->mCurrentDatum->SetType(Datum::TYPE_INT);
								int newInt = tempStringDatum.GetAs<int>();
								sharedData->mCurrentDatum->Set(newInt, insertPosition);
							}
							break;
						case Datum::TYPE_FLOAT:
							{
								sharedData->mCurrentDatum->SetType(Datum::TYPE_FLOAT);
								float newFloat = tempStringDatum.GetAs<float>();
								sharedData->mCurrentDatum->Set(newFloat, insertPosition);
							}
							break;
						case Datum::TYPE_VECTOR:
							{
								sharedData->mCurrentDatum->SetType(Datum::TYPE_VECTOR);
								Vec4 newVector4 = tempStringDatum.GetAs<Vec4>();
								sharedData->mCurrentDatum->Set(newVector4, insertPosition);
							}
							break;
						case Datum::TYPE_MATRIX:
							{
								sharedData->mCurrentDatum->SetType(Datum::TYPE_MATRIX);
								Mat4 newMatrix = tempStringDatum.GetAs<Mat4>();
								sharedData->mCurrentDatum->Set(newMatrix, insertPosition);
							}
							break;
						case Datum::TYPE_SCOPE:
							{
							}
							break;
						case Datum::TYPE_STRING:
							{
								sharedData->mCurrentDatum->SetType(Datum::TYPE_STRING);
								sharedData->mCurrentDatum->Set((*mySListIterator).second, insertPosition);
							}
							break;
						default:
							break;
					}
				}
			}
			return true;
		}
	}
	return false;
}

bool XmlParseHelperScope::EndElementHandler(std::string tagName)
{
	if(CheckIfValidTag(tagName) == false)
		return false;

	XmlParseHelperScope::SharedDataScope* sharedData = reinterpret_cast<XmlParseHelperScope::SharedDataScope*>(mMaster->GetSharedData());
	if(sharedData->Is("XmlParseHelperScope::SharedDataScope")){
		if(tagName == Datum::StringFromType(Datum::TYPE_SCOPE)){
			if(sharedData->mCurrentScope->GetParent() != NULL)
				sharedData->mCurrentScope = sharedData->mCurrentScope->GetParent();
		}
		return true;
	}
	return false;
}


void XmlParseHelperScope::CharDataHandler(std::string buffer)
{
	XmlParseHelperScope::SharedDataScope* sharedData = reinterpret_cast<XmlParseHelperScope::SharedDataScope*>(mMaster->GetSharedData());
	if(sharedData->Is("XmlParseHelperScope::SharedDataScope")){
	}
}

