#include "XmlParseMaster.h"

/**********************************
	XmlParseMaster
***********************************/

XmlParseMaster::XmlParseMaster(IXmlParseHelper::SharedData * newShardedData) : mIsClone(false)
{
	mParser = XML_ParserCreate(NULL);
	mSharedData = newShardedData;
	mSharedData->SetXmlParseMaster(this);
	XML_SetElementHandler(mParser, StartElementHandler, EndElementHandler);
	XML_SetCharacterDataHandler(mParser, CharDataHandler);
	XML_SetUserData(mParser, mSharedData);

	mHelpers.Clear();
	//Reset();
}

XmlParseMaster::XmlParseMaster() : mIsClone(false)
{
}

XmlParseMaster::~XmlParseMaster()
{
	if(mIsClone){
		while(mHelpers.Size() > 0)
		{
			if(*(mHelpers.Begin())){
				delete(*(mHelpers.Begin()));
				mHelpers.PopFront();
			}
		}
	}
	mSharedData = NULL;
	mHelpers.Clear();
	XML_ParserFree(mParser);
}


void XmlParseMaster::AddHelper(IXmlParseHelper* helper)
{
	mHelpers.PushFront(helper);	
}

void XmlParseMaster::RemoveHelper(IXmlParseHelper* helper)
{
	mHelpers.Remove(helper);
}

void XmlParseMaster::Parse(char* buffer, unsigned int size, bool lastChunk)
{
	Reset();
	//assert(XML_Parse(mParser,  buffer, size, lastChunk) != XML_STATUS_ERROR); 
	XML_Parse(mParser, buffer, size, lastChunk);
}

void XmlParseMaster::ParseFromFile(std::string fileName)
{
	mFileName = fileName;
	char** buffer = new char*();
	unsigned int fileSize = FileLoad(buffer, mFileName.c_str());
	
	assert(fileSize);
	assert(*buffer);

	Reset();
	Parse(*buffer, fileSize, true);

	delete buffer;
}

IXmlParseHelper::SharedData* XmlParseMaster::GetSharedData()
{
	return(mSharedData);
}

void XMLCALL XmlParseMaster::StartElementHandler(void *data, const char *el, const char **attr)
{
	IXmlParseHelper::SharedData* sharedData = reinterpret_cast<IXmlParseHelper::SharedData*>(data);

	//Get Master
	XmlParseMaster* master = sharedData->GetXmlParseMaster();
	
	//Create an SList if pairs
	SList< pair<std::string, std::string> > attributeValuePairs;
	for (int i = 0; attr[i]; i += 2) {
		pair<std::string, std::string> attributeEntry(attr[i], attr[i + 1]);
		attributeValuePairs.PushBack(attributeEntry);

	}
	//Pass to every helper
	for(SList<IXmlParseHelper*>::SListIterator myHelperIterator = master->mHelpers.Begin(); myHelperIterator != master->mHelpers.End(); myHelperIterator++)
	{
		(*myHelperIterator)->StartElementHandler(std::string(el), attributeValuePairs);
	}

	// Increase Depth
	sharedData->IncDepth(); 
}

void XMLCALL XmlParseMaster::EndElementHandler(void *data, const char *el)
{
	IXmlParseHelper::SharedData* sharedData = reinterpret_cast<IXmlParseHelper::SharedData*>(data);
	
	//Get Master
	XmlParseMaster* master = sharedData->GetXmlParseMaster();

	//Decrease Depth
	sharedData->DecDepth();


	for(SList<IXmlParseHelper*>::SListIterator myHelperIterator = master->mHelpers.Begin(); myHelperIterator != master->mHelpers.End(); myHelperIterator++)
	{
		(*myHelperIterator)->EndElementHandler(std::string(el));
	}
}

void XMLCALL XmlParseMaster::CharDataHandler(void *data, const char* el, int length)
{
	IXmlParseHelper::SharedData* sharedData = reinterpret_cast<IXmlParseHelper::SharedData*>(data);
	
	//Get Master
	XmlParseMaster* master = sharedData->GetXmlParseMaster();



	for(SList<IXmlParseHelper*>::SListIterator myHelperIterator = master->mHelpers.Begin(); myHelperIterator != master->mHelpers.End(); myHelperIterator++)
	{
		(*myHelperIterator)->CharDataHandler(std::string(el));
	}
}

XmlParseMaster* XmlParseMaster::Create()
{
	//Nerw Shared Data
	IXmlParseHelper::SharedData* newSharedData = this->GetSharedData()->Create();
	//New Master
	XmlParseMaster* newParseMaster = new XmlParseMaster(newSharedData);

	newParseMaster->mIsClone = true;

	//Clone helpers
	SList<IXmlParseHelper*>::SListIterator myHelperIterator;
	for(myHelperIterator = mHelpers.Begin(); myHelperIterator != mHelpers.End(); myHelperIterator++)
	{
		(*myHelperIterator)->Create(newParseMaster);
	}
	return(newParseMaster);
}

void XmlParseMaster::SetFileName(std::string newFileName)
{
	mFileName = newFileName;
}

std::string XmlParseMaster::GetFileName()
{
	return(mFileName);
}

void XmlParseMaster::SetParser(XML_Parser newParser)
{
	mParser = newParser;
}

void XmlParseMaster::Reset()
{
	//if(mParser != NULL)
	XML_ParserReset(mParser, NULL);
	XML_SetUserData(mParser, mSharedData);
	XML_SetElementHandler(mParser, StartElementHandler, EndElementHandler);
	XML_SetCharacterDataHandler(mParser, CharDataHandler);
}

bool XmlParseMaster::GetIsClone()
{
	return(mIsClone);
}
