/*! \file util.cpp

    \brief utility routines

    \author Copyright 2005-2008 UCF/FIEA/MJG; All rights reserved.
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "macros.h"

// Private variables --------------------------------------------------------------

static const char * sPaths[] =
{
    #if defined( _XBOX )
        // In the Real World, _XBOX media path would depend on whether the build
        // was for optical media (i.e., release) or hard drive (i.e. internal builds).
        "" ,
        "d:\\Media\\Configuration\\"
    #elif defined( _WIN32 ) || defined(__CYGWIN32__) 
        "" ,
        "../../Media/Configuration/"
   #else
        #error "unknown platform"
    #endif
} ;

static const int sNumPaths = sizeof( sPaths ) / sizeof( sPaths[0] ) ;

// Public functions --------------------------------------------------------------

/*! \brief  Generate a pseudo-random floating-point number
*/
float RandomSpread( float fSpread )
{
    const float randomFloat = fSpread * ( float( rand() ) / float( RAND_MAX ) - 0.5f ) ;
    return randomFloat ;
}

void strTranslate( char * str , int iOriginal , int iFinal )
{
    const int iStrLen = (const int)strlen( str ) ;
    for( int i = 0 ; i < iStrLen ; i ++ )
    {
        str[ i ] = ( iOriginal == str[ i ] ) ? iFinal : str[i] ;
    }
}

FILE * FileOpen( const char * strFilename , const char * strAccessMode , const char ** pPathChosen )
{
    char strFullyQualifiedFilename[ 256 ] ;
    for( int iPath = 0 ; iPath < sNumPaths ; ++ iPath )
    {
        ASSERT( strlen( sPaths[ iPath ] ) + strlen( strFilename ) + 1 < sizeof( strFullyQualifiedFilename ) ) ;
        sprintf( strFullyQualifiedFilename , "%s%s" , sPaths[ iPath ] , strFilename ) ;

        #if defined( _XBOX )
            // Xbox requires backward slashes as a delimiter
            strTranslate( strFullyQualifiedFilename , '/' , '\\' ) ;
        #endif

        // Open file
        FILE * pFile = fopen( strFullyQualifiedFilename , strAccessMode ) ;
        if( pFile != NULL )
        {
            if( pPathChosen != 0 )
            {   // Caller requested path this routine chose to open the requested file
                * pPathChosen = sPaths[ iPath ] ;
            }
            return pFile ;
        }
    }

    // Could not open file
    ASSERT( 0 ) ;
    return 0 ;
}

unsigned int FileLoad( char ** ppBuffer , const char * strFilename , const char ** pPathChosen )
{
    // Open file for reading (in "binary" mode, i.e. do not translate cr+lf)
    FILE * pFile = FileOpen( strFilename , "rb" , pPathChosen ) ;
    if( NULL == pFile )
    {   // Could not open file
        ASSERT( 0 ) ;
        return 0 ;
    }

    // Get file size
    fseek( pFile , 0, SEEK_END ) ;
    unsigned long dwFileSize = ftell( pFile ) ;
    fseek( pFile, 0, SEEK_SET ) ;

    // Read file
    * ppBuffer = (char *) new char[ dwFileSize + 1 ] ;
    (*ppBuffer) [ dwFileSize ] = '\0' ; // file apparently does not end in nul character, so force it.
    fread( * ppBuffer , 1 , dwFileSize , pFile ) ;
    fclose( pFile ) ;

    return dwFileSize ;
}