#ifndef __ACTIONTEST_H
#define __ACTIONTEST_H

#include <cxxtest/TestSuite.h>
#include "Vector.h"
#include "string.h"

#include "World.h"
#include "Action.h"
#include "ActionCreateAction.h"

//Parse
#include "XmlParseMaster.h"
#include "XmlParseHelperScope.h"
#include "XmlParseHelperEntity.h"
#include "XmlParseHelperAction.h"

#include "Universe.h"

class ActionEntityTest : public Entity
{
	RTTI_DECLARATIONS(ActionEntityTest, Entity)


	ActionEntityTest()
	{
		mySignatureInt.mName = ("My Int");
		mySignatureInt.mNumElements = 3;
		mySignatureInt.mType = Datum::TYPE_INT;
		mySignatureInt.mInit = new int(0);
		mSignatures.PushBack(mySignatureInt);

		mySignatureFloat.mName = ("My Float");
		mySignatureFloat.mNumElements = 3;
		mySignatureFloat.mType = Datum::TYPE_FLOAT;
		mySignatureFloat.mInit = new float(3.0f);
		mSignatures.PushBack(mySignatureFloat);

		mySignatureScope.mName = ("My Scope");
		mySignatureScope.mNumElements = 2;
		mySignatureScope.mType = Datum::TYPE_SCOPE;
		mySignatureScope.mInit = new Scope();
		mSignatures.PushBack(mySignatureScope);

		Populate();
	}

	virtual ~ActionEntityTest()
	{
		delete mySignatureInt.mInit;
		delete mySignatureFloat.mInit;
		delete mySignatureScope.mInit;
	}

	private:
		Signature mySignatureInt;
		Signature mySignatureFloat;
		Signature mySignatureScope;

};
RTTI_DEFINITIONS(ActionEntityTest)

EntityFactory( ActionEntityTest )

/*!
*	A simple test suite: Just inherit CxxTest::TestSuite and write tests!
*/
class ActionUnitTest : public CxxTest::TestSuite
{
public:

	void testAction()
	{
		
	}
	
	void testActionXMLParser()
	{
		//Factories
		ActionEntityTestFactory myPlayerFactory;

		Universe myUniverse;

		//New World
		World * myWorld = new World();

		//Shared Data
		XmlParseHelperScope::SharedDataScope* mySharedData = new XmlParseHelperScope::SharedDataScope(myWorld);

		//Master
		XmlParseMaster* myXMLParseMaster = new XmlParseMaster(mySharedData);

		//Helpers
		XmlParseHelperScope* myNewScopeHelper = new XmlParseHelperScope(myXMLParseMaster);
		XmlParseHelperEntity* myNewEntityHelper = new XmlParseHelperEntity(myXMLParseMaster);
		XmlParseHelperAction* myNewActionHelper = new XmlParseHelperAction(myXMLParseMaster);

		//Parse
		myXMLParseMaster->ParseFromFile("XML\\ActionTest.xml");

		//Depth Test
		TS_ASSERT_EQUALS(myXMLParseMaster->GetSharedData()->GetDepth(), 0);
		//Path Test
		TS_ASSERT_EQUALS(myXMLParseMaster->GetFileName(), "XML\\ActionTest.xml");

		//1st Process
		cout<<"Process 1\n";
		myWorld->Process();

		//2nd Process - MyPrintEntry should be deleted
		cout<<"Process 2\n";
		myWorld->Process();

		delete myNewActionHelper;
		delete myNewEntityHelper;
		delete myNewScopeHelper;
		
		delete mySharedData;
		delete myXMLParseMaster;
	}
};

#endif // __ACTIONTEST_H