#ifndef __ATTRIBUTEDTEST_H
#define __ATTRIBUTEDTEST_H

#include <cxxtest/TestSuite.h>
#include "Attributed.h"

/*!
*	A simple test suite: Just inherit CxxTest::TestSuite and write tests!
*/
class AttributedUnitTest : public CxxTest::TestSuite
{
public:

	class Player : public Attributed
	{
		public:
			RTTI_DECLARATIONS(Player, Attributed)

			Player() : mySignatureInt(), mySignatureFloat(), mySignatureExternal(), mySignatureScope()
			{	
				mySignatureInt.mName = ("My Int");
				mySignatureInt.mNumElements = 3;
				mySignatureInt.mType = Datum::TYPE_INT;
				mySignatureInt.mInit = new int(0);
				mSignatures.PushBack(mySignatureInt);

				mySignatureFloat.mName = ("My Float");
				mySignatureFloat.mNumElements = 3;
				mySignatureFloat.mType = Datum::TYPE_FLOAT;
				mySignatureFloat.mInit = new float(3.0f);
				mSignatures.PushBack(mySignatureFloat);

				arrayInt = new int[10];
				arrayInt[0] = 10;
				arrayInt[2] = 12;
				mySignatureExternal.mName = ("My External");
				mySignatureExternal.mNumElements = 10;
				mySignatureExternal.mType = Datum::TYPE_INT;
				mySignatureExternal.mStorage.TYPE_INT = arrayInt;
				mSignatures.PushBack(mySignatureExternal);

				//Scope				
				mySignatureScope.mName = ("My Scope");
				mySignatureScope.mNumElements = 2;
				mySignatureScope.mType = Datum::TYPE_SCOPE;
				mySignatureScope.mInit = new Scope();
				mSignatures.PushBack(mySignatureScope);

				
				mySignatureIntCopy = mySignatureInt;
				mySignatureIntCopy.mName = ("My Int Copy");
				mSignatures.PushBack(mySignatureIntCopy);

				mySignatureExternalCopy = mySignatureExternal;
				mySignatureExternalCopy.mName = ("My External Copy");
				mSignatures.PushBack(mySignatureExternalCopy);
 
				
				Populate();
			}

			virtual ~Player()
			{
				delete mySignatureInt.mInit;
				delete mySignatureFloat.mInit;
				delete mySignatureScope.mInit;
				delete mySignatureExternal.mInit;
			}

			int GetSignature(const unsigned int index, Attributed::Signature* mySignature)
			{
				return 0;
			}

		private:
			Signature mySignatureInt;
			Signature mySignatureFloat;
			Signature mySignatureExternal;

			//External
			Signature mySignatureIntCopy;
			Signature mySignatureExternalCopy;

			//Scope
			Signature mySignatureScope;

			int* arrayInt;
	};

	/*!
		A simple class used in the unit tests
	*/
	void testAttributed()
	{
		TS_ASSERT_EQUALS(1, 1);
		Player myPlayer;

		TS_ASSERT_EQUALS(myPlayer.IsPrescribedAttribute("My Int"), true);
		TS_ASSERT_EQUALS(myPlayer.IsAttribute("My Int"), true);
		TS_ASSERT_EQUALS(myPlayer.IsAuxiliaryAttribute("My Int"), false);
		Datum* intDatum = myPlayer.Find("My Int");
		TS_ASSERT_EQUALS(intDatum->Get<int>(0), 0);
		intDatum->Set(1,0);
		TS_ASSERT_EQUALS(intDatum->Get<int>(1), 0);
		TS_ASSERT_EQUALS(intDatum->Size(), 3);


		//External
		TS_ASSERT_EQUALS(myPlayer.IsAuxiliaryAttribute("My External"), false);
		Datum* externalDatum = myPlayer.Find("My External");

		TS_ASSERT_EQUALS(externalDatum->Get<int>(0), 10);
		TS_ASSERT_EQUALS(externalDatum->Get<int>(2), 12);

		
		//Copy
		TS_ASSERT_EQUALS(myPlayer.IsAuxiliaryAttribute("My Int Copy"), false);
		Datum* intDatumCopy = myPlayer.Find("My Int Copy");
		TS_ASSERT_EQUALS(intDatumCopy->Get<int>(0), 0);
		TS_ASSERT_EQUALS(intDatumCopy->Get<int>(1), 0);
		TS_ASSERT_EQUALS(intDatumCopy->Size(), 3);

		TS_ASSERT_EQUALS(myPlayer.IsAuxiliaryAttribute("My External"), false);
		Datum* externalDatumCopy = myPlayer.Find("My External");


		TS_ASSERT_EQUALS(externalDatumCopy->Get<int>(0), 10);
		TS_ASSERT_EQUALS(externalDatumCopy->Get<int>(2), 12);

		//Scope
		Datum* scopeDatum = myPlayer.Find("My Scope");
		scopeDatum->Get<Datum::ScopeType>(0)->Append("LOL");
		
		Datum* foundScope  = scopeDatum->Get<Datum::ScopeType>(0)->Find("LOL");
		foundScope->SetType(Datum::TYPE_INT);


		//Aux
		Datum newDatum = myPlayer.AppendAuxiliaryAttribute("New Attribute");
		newDatum.SetType(Datum::TYPE_INT);
		TS_ASSERT_EQUALS(7, myPlayer.AuxBegin());
		
		TS_ASSERT_EQUALS(myPlayer.IsPrescribedAttribute("New Attribute"), false);
		TS_ASSERT_EQUALS(myPlayer.IsAttribute("New Attribute"), true);
		TS_ASSERT_EQUALS(myPlayer.IsAuxiliaryAttribute("New Attribute"), true);
	}
};


RTTI_DEFINITIONS(AttributedUnitTest::Player)

#endif