#ifndef __DATUMTEST_H
#define __DATUMTEST_H

#include <cxxtest/TestSuite.h>
#include "Datum.h"

/*!
*	A simple test suite: Just inherit CxxTest::TestSuite and write tests!
*/
class DatumUnitTest : public CxxTest::TestSuite
{
public:
	/*!
		A simple class used in the unit tests
	*/
	void testSetAndGet()
	{
		//int 
		int myInt = 5;
		int myInt2 = 2;
		Datum myIntDatum;
		myIntDatum.Set(myInt, 5);
		myIntDatum.Set(myInt2);
		TS_ASSERT_EQUALS(myInt2, myIntDatum.Get<int>(0));
		TS_ASSERT_EQUALS(myInt, myIntDatum.Get<int>(5));

		//float 
		float myFloat = 5.0f;
		float myFloat2 = 2.0f;
		Datum myFloatDatum;
		myFloatDatum.Set(myFloat, 5);
		myFloatDatum.Set(myFloat2);
		TS_ASSERT_EQUALS(myFloat2, myFloatDatum.Get<float>(0));
		TS_ASSERT_EQUALS(myFloat, myFloatDatum.Get<float>(5));

		//string
		std::string myString = "LOL";
		std::string myString2 = "COOL";
		Datum myStringDatum;
		myStringDatum.Set(myString, 5);
		myStringDatum.Set(myString2);
		TS_ASSERT_EQUALS(myString2, myStringDatum.Get<std::string>(0));
		TS_ASSERT_EQUALS(myString, myStringDatum.Get<std::string>(5));

		//reference
		Datum myReferenceDatum;
		myReferenceDatum.Set(&myStringDatum);
		TS_ASSERT_EQUALS(myString2, myReferenceDatum.Get<Datum::ReferenceType>(0)->Get<std::string>());

		//pointer
		Datum myPointerDatum;
		void* myPointer = (void*)&myReferenceDatum;
		void* myPointer2 = (void*)&myFloatDatum;
		myPointerDatum.Set(myPointer);
		TS_ASSERT_EQUALS(myString2, ((Datum*)myPointerDatum.Get<Datum::PointerType>())->Get<Datum::ReferenceType>(0)->Get<std::string>());
		myPointerDatum.Set(myPointer2);
		TS_ASSERT_EQUALS(myFloat2, ((Datum*)myPointerDatum.Get<Datum::PointerType>(1))->Get<float>());

		void* myPointer3 = (void*)&myInt;
		TS_ASSERT_EQUALS(myInt, *((int*)myPointer3));
	}

	void testAssignmentAndSetStorage()
	{
		//int 
		int myInt = 5;
		int myInt2 = 2;
		Datum myIntDatum;
		myIntDatum.Set(myInt, 5);
		myIntDatum.Set(myInt2);
		Datum myIntDatum2;
		myIntDatum2 = myIntDatum;
		TS_ASSERT_EQUALS((myIntDatum == myIntDatum2), true);
		myIntDatum.Set(8);
		TS_ASSERT_EQUALS((myIntDatum == myIntDatum2), false);
		TS_ASSERT_EQUALS(8, myIntDatum.Get<int>(0));
		TS_ASSERT_EQUALS(2, myIntDatum2.Get<int>(0));

		//float 
		float myFloat = 5.0f;
		float myFloat2 = 2.0f;
		Datum myFloatDatum;
		myFloatDatum.Set(myFloat, 5);
		myFloatDatum.Set(myFloat2);
		Datum myFloatDatum2;
		myFloatDatum2 = myFloatDatum;
		TS_ASSERT_EQUALS((myFloatDatum == myFloatDatum2), true);
		myFloatDatum.Set(8.0f);
		TS_ASSERT_EQUALS(8.0f, myFloatDatum.Get<float>(0));
		TS_ASSERT_EQUALS(2.0f, myFloatDatum2.Get<float>(0));

		//string
		std::string myString = "LOL";
		std::string myString2 = "COOL";
		Datum myStringDatum;
		myStringDatum.Set(myString, 5);
		myStringDatum.Set(myString2);
		Datum myStringDatum2;
		myStringDatum2 = myStringDatum;
		TS_ASSERT_EQUALS(myString2, myStringDatum2.Get<std::string>(0));
		TS_ASSERT_EQUALS((myStringDatum == myStringDatum2), true);
		myStringDatum.Set(myString);
		TS_ASSERT_EQUALS((myStringDatum == myStringDatum2), false);

		//reference
		Datum myReferenceDatum;
		myReferenceDatum.Set(&myStringDatum);
		Datum myReferenceDatum2;
		myReferenceDatum2 = myReferenceDatum;
		TS_ASSERT_EQUALS(myString, myReferenceDatum.Get<Datum::ReferenceType>(0)->Get<std::string>());
		TS_ASSERT_EQUALS(myString, myReferenceDatum2.Get<Datum::ReferenceType>(0)->Get<std::string>());
		TS_ASSERT_EQUALS((myReferenceDatum == myReferenceDatum2), true);
		//myReferenceDatum.Clear();
		//TS_ASSERT_EQUALS((myReferenceDatum == myReferenceDatum2), true);

		//Set Storage - External
		Datum myStringDatumExternal;
		std::string myStringArray[3];
		myStringArray[0] = "LOL";
		myStringArray[1] = "DATUM";
		myStringArray[2] = "MASS EFFECT 3";
		myStringDatumExternal.SetType(Datum::TYPE_STRING);
		myStringDatumExternal.SetStorage((void*)(myStringArray),3,3);
		TS_ASSERT_EQUALS("LOL", myStringDatumExternal.Get<std::string>(0));
		TS_ASSERT_EQUALS("DATUM", myStringDatumExternal.Get<std::string>(1));
		TS_ASSERT_EQUALS("MASS EFFECT 3", myStringDatumExternal.Get<std::string>(2));
	}

	void testInternalExternal()
	{
		//string
		std::string myString = "LOL";
		std::string myString2 = "COOL";
		Datum myStringDatum;
		myStringDatum.Set(myString, 5);
		myStringDatum.Set(myString2);

		//Set Storage - External
		Datum myStringDatumExternal;
		std::string myStringArray[3];
		myStringArray[0] = "LOL";
		myStringArray[1] = "DATUM";
		myStringArray[2] = "MASS EFFECT 3";
		myStringDatumExternal.SetType(Datum::TYPE_STRING);
		myStringDatumExternal.SetStorage((void*)(myStringArray),3,3);
		TS_ASSERT_EQUALS("LOL", myStringDatumExternal.Get<std::string>(0));
		TS_ASSERT_EQUALS("DATUM", myStringDatumExternal.Get<std::string>(1));
		TS_ASSERT_EQUALS("MASS EFFECT 3", myStringDatumExternal.Get<std::string>(2));

		//Making an Internal into a External
		TS_ASSERT_EQUALS((myStringDatumExternal == myStringDatum), false);
		myStringDatum = myStringDatumExternal;
		TS_ASSERT_EQUALS((myStringDatumExternal == myStringDatum), true);
		myStringDatum.Set(myString,2);
		TS_ASSERT_EQUALS((myStringDatumExternal == myStringDatum), true);
	}

	void testExternalInternal()
	{
		//string
		std::string myString = "LOL";
		std::string myString2 = "COOL";
		Datum myStringDatum;
		myStringDatum.Set(myString, 5);
		myStringDatum.Set(myString2);

		//Set Storage - External
		Datum myStringDatumExternal;
		std::string myStringArray[3];
		myStringArray[0] = "LOL";
		myStringArray[1] = "DATUM";
		myStringArray[2] = "MASS EFFECT 3";
		myStringDatumExternal.SetType(Datum::TYPE_STRING);
		myStringDatumExternal.SetStorage((void*)(myStringArray),3,3);
		TS_ASSERT_EQUALS("LOL", myStringDatumExternal.Get<std::string>(0));
		TS_ASSERT_EQUALS("DATUM", myStringDatumExternal.Get<std::string>(1));
		TS_ASSERT_EQUALS("MASS EFFECT 3", myStringDatumExternal.Get<std::string>(2));

		//External into a Internal
		myStringDatumExternal = myStringDatum;
		TS_ASSERT_EQUALS((myStringDatumExternal == myStringDatum), true);
		myStringDatum.Set(myString,2);
		TS_ASSERT_EQUALS((myStringDatumExternal == myStringDatum), false);
	}

	void testExternalExternal()
	{
		std::string myString = "LOL";
		std::string myString2 = "COOL";
		Datum myStringDatum;
		myStringDatum.Set(myString, 5);
		myStringDatum.Set(myString2);
		
		//Set Storage - External
		Datum myStringDatumExternal;
		std::string myStringArray[3];
		myStringArray[0] = "LOL";
		myStringArray[1] = "DATUM";
		myStringArray[2] = "MASS EFFECT 3";
		myStringDatumExternal.SetType(Datum::TYPE_STRING);
		myStringDatumExternal.SetStorage((void*)(myStringArray),3,3);
		TS_ASSERT_EQUALS("LOL", myStringDatumExternal.Get<std::string>(0));
		TS_ASSERT_EQUALS("DATUM", myStringDatumExternal.Get<std::string>(1));
		TS_ASSERT_EQUALS("MASS EFFECT 3", myStringDatumExternal.Get<std::string>(2));
		
		Datum myStringDatumExternal2;
		myStringDatumExternal2.SetType(Datum::TYPE_STRING);
		myStringDatumExternal2.SetStorage((void*)(myStringArray),3,3);

		//External - External
		myStringDatumExternal2 = myStringDatum;
		TS_ASSERT_EQUALS((myStringDatumExternal2 == myStringDatumExternal2), true);
		myStringDatumExternal.Set(myString,2);
		TS_ASSERT_EQUALS((myStringDatumExternal2 == myStringDatumExternal2), true);
	}

	void testStaticFunctions()
	{
		Datum myIntDatum;
		myIntDatum.SetType(Datum::TYPE_INT);
		myIntDatum.SetNumValues(10);
		myIntDatum.Set(5, 5);
		myIntDatum.Set(2);
		TS_ASSERT_EQUALS(Datum::HasValue(myIntDatum, 5), true);
		TS_ASSERT_EQUALS(Datum::HasValue(myIntDatum, 11), false);
		TS_ASSERT_EQUALS(Datum::TypeFromString("TYPE_INT"), Datum::TYPE_INT);
		TS_ASSERT_EQUALS(Datum::StringFromType(Datum::TYPE_INT),"TYPE_INT");
	}

	void testScalarAssignments()
	{
		//int 
		Datum myIntDatum;
		myIntDatum = 5;
		TS_ASSERT_EQUALS(true, myIntDatum.operator==(5));
		TS_ASSERT_EQUALS(5, myIntDatum.Get<int>(0));

		//float 
		Datum myFloatDatum;
		myFloatDatum = 5.0f;
		TS_ASSERT_EQUALS(true, myFloatDatum.operator==(5.0f));
		TS_ASSERT_EQUALS(5.0f, myFloatDatum.Get<float>(0));

		//string
		std::string myString = "LOL";
		Datum myStringDatum;
		myStringDatum = myString;
		TS_ASSERT_EQUALS(true, myStringDatum.operator==(myString));
		TS_ASSERT_EQUALS(myString, myStringDatum.Get<std::string>(0));

		//reference
		Datum myReferenceDatum;
		myReferenceDatum = &myStringDatum;
		TS_ASSERT_EQUALS(true, myReferenceDatum.operator==(&myStringDatum));
		TS_ASSERT_EQUALS(myString, myReferenceDatum.Get<Datum::ReferenceType>(0)->Get<std::string>());
	}

	void testGetAs()
	{
		std::string myString = "5";
		//int 
		Datum myIntDatum;
		myIntDatum = 5;
		TS_ASSERT_EQUALS(true, myIntDatum.operator==(5));
		TS_ASSERT_EQUALS(5, myIntDatum.Get<int>(0));
		TS_ASSERT_EQUALS(myIntDatum.GetAs<float>(), 5.0f);
		TS_ASSERT_EQUALS(myIntDatum.GetAs<std::string>(), myString);

		//float 
		Datum myFloatDatum;
		myFloatDatum = 5.0f;
		TS_ASSERT_EQUALS(true, myFloatDatum.operator==(5.0f));
		TS_ASSERT_EQUALS(5, myFloatDatum.GetAs<int>());
		std::string stringResult = myFloatDatum.GetAs<std::string>();
		
		//TS_ASSERT_EQUALS(
		//TS_ASSERT_EQUALS(stringResult.compare(myString), true)
		//TS_ASSERT_EQUALS(stringResult, myString);

		//string
		Datum myStringDatum;
		myStringDatum = myString;
		TS_ASSERT_EQUALS(true, myStringDatum.operator==(myString));
		TS_ASSERT_EQUALS(myString, myStringDatum.Get<std::string>(0));
		TS_ASSERT_EQUALS(5, myStringDatum.GetAs<int>(0));
		TS_ASSERT_EQUALS(5.0f, myStringDatum.GetAs<float>(0));
	}
};

#endif // __STACKTEST_H