#ifndef __EVENTTEST_H
#define __EVENTTEST_H

#include <cxxtest/TestSuite.h>

#include "string.h"

#include "IEventSubscriber.h"
#include "EventQueue.h"
#include "Event.h"


/*!
*	A simple test suite: Just inherit CxxTest::TestSuite and write tests!
*/
class EventUnitTest : public CxxTest::TestSuite
{
public:

	class ConcreteSubscriber : public IEventSubscriber
	{
		public:
			ConcreteSubscriber() : mHasGottenMessage(false)
			{
			}

			bool HasGottenMessage()	{ return mHasGottenMessage; }
			void SetHasGottenMessage(bool newHasGottenMessage) { mHasGottenMessage = newHasGottenMessage; }


		private:
			
			virtual void Notify(EventPublisher * eventPublisher)
			{
				Event<std::string> * event = eventPublisher->As< Event< std::string> >();
				cout<<event->GetMessage()<<"\n";
				mHasGottenMessage = true;
			}

			bool mHasGottenMessage;
	};

	/*!
	*	Tests the event system
	*/
	void testEvent()
	{
		EventQueue myEventQueue;
		myEventQueue.Clear();

		ConcreteSubscriber myConcreteSubscriber;


		std::string myMessage("Helloooo!");
		Event<std::string> * myEvent = new  Event<std::string>(myMessage, true);

		Event<std::string>::Subscribe(&myConcreteSubscriber);

		myEventQueue.Enqueue(myEvent, 0, 0.5);
		
		myEventQueue.Publish(0.0f);
		TS_ASSERT_EQUALS(myConcreteSubscriber.HasGottenMessage(), false);

		myEventQueue.Publish(0.5f);
		TS_ASSERT_EQUALS(myConcreteSubscriber.HasGottenMessage(), false);

		myEventQueue.Publish(1.0f);
		TS_ASSERT_EQUALS(myConcreteSubscriber.HasGottenMessage(), true);

		myConcreteSubscriber.SetHasGottenMessage(false);

		myEventQueue.Publish(1.5f);
		TS_ASSERT_EQUALS(myConcreteSubscriber.HasGottenMessage(), false);



	}
};

#endif // __EVENTTEST_H