#ifndef __HASHMAPTEST_H
#define __HASHMAPTEST_H

#include <cxxtest/TestSuite.h>
#include "HashMap.h"
#include "string.h"

/*!
*	A simple test suite: Just inherit CxxTest::TestSuite and write tests!
*/
class HashMapUnitTest : public CxxTest::TestSuite
{
public:
	/*!
		A simple class used in the unit tests
	*/
	class Foo
	{
		public:
			Foo() : mData(0) { }

			Foo(int* data) : mData(data) { }
			int* GetData() const { return(mData); }

			bool operator==(const Foo & otherFoo) const
			{
				if((GetData()) && (otherFoo.GetData())){

					if((GetData()) == (otherFoo.GetData()))
						return(true);
					else
						return(false);
				}
				return(false);
			}
		private:
			int* mData;
	};

	// Foo hash func
	class HashFoo{
		public:
			unsigned int operator()(const Foo myKey) const {
				unsigned int returnValue = (*(myKey.GetData()));
				return (returnValue);
			}
	};

	//int 
	HashMap< int, int, HashInt >* myIntHashMap;

	// char*
	HashMap< char*, int , HashCharString >* myCharStringHashMap;

	// Foo
	HashMap< Foo, int , HashFoo >* myFooHashMap;


	int* myInt1;
	int* myInt2;
	Foo* myFoo1;
	Foo* myFoo2;

	void setUp() 
	{
		// int
		myIntHashMap = new HashMap<int, int, HashInt>(10);

		// Foo
		myFooHashMap = new HashMap<Foo, int, HashFoo>(10);

		myInt1 = new int(1);
		myInt2 = new int(2);

		myFoo1 = new Foo(myInt1);
		myFoo2 = new Foo(myInt2);
	}

    void tearDown() 
	{
		// int 
		delete myIntHashMap;

		// Foo
		delete myFooHashMap;
		delete myInt1;
		delete myInt2;
		delete myFoo1;
		delete myFoo2;
	}

	void testFind()
	{
		// int 
		for(int i = 0; i < 10; i++){
			pair < int, int > entryInt ((i),(i * 2));
			myIntHashMap->Insert(entryInt);
		}
		HashMap< int, int, HashInt >::HashMapIterator myIntIterator = myIntHashMap->Find(7);
		TS_ASSERT_EQUALS((*myIntIterator).first, 7);
		myIntIterator = myIntHashMap->Find(21);
		TS_ASSERT_EQUALS(myIntIterator, myIntHashMap->End());

		//Test reinstert
		myIntIterator = myIntHashMap->Find(0);
		TS_ASSERT_EQUALS((*myIntIterator).second, 0);
		pair < int, int > entryInt ((0),(3));
		myIntHashMap->Insert(entryInt);
		TS_ASSERT_EQUALS((*(myIntHashMap->Begin())).second, 0);


		// char *
		myCharStringHashMap = new HashMap< char*, int , HashCharString >(4);
		string inventory[10] = {"Sword", "Armor", "Shield", "Potion", "Boots", "Helm", "Axe", "Dagger", "Food", "Junk"};
		for(int i = 0; i < 10; i++){
			pair < char*, int> entryCharString1((char*)inventory[i].c_str(), i);
			myCharStringHashMap->Insert(entryCharString1);
		}
		HashMap< char* , int, HashCharString >::HashMapIterator myCharPtrIterator = myCharStringHashMap->Begin();

		pair < char*, int> entryCharString1((char*)inventory[0].c_str() , i);
		myCharPtrIterator = myCharStringHashMap->Find((char*)inventory[0].c_str());
		TS_ASSERT_EQUALS((*myCharPtrIterator).second, 0);

		HashMap< char* , int, HashCharString >::HashMapIterator myCharPtrIterator2 = myCharStringHashMap->Find( (char*)inventory[0].c_str() );
		TS_ASSERT_EQUALS((char*)inventory[0].c_str() , (*myCharPtrIterator2).first);
		
		myCharPtrIterator2 = myCharStringHashMap->Find( (char*)inventory[3].c_str() );
		TS_ASSERT_EQUALS((char*)inventory[3].c_str(), (*myCharPtrIterator2).first);

		delete myCharStringHashMap;
		myCharStringHashMap = new HashMap< char*, int , HashCharString >(4);
		delete myCharStringHashMap;

		// Foo
		pair < Foo, int > entryFoo1(*myFoo1, 10);
		pair < Foo, int > entryFoo2(*myFoo2, 20);
		myFooHashMap->Insert(entryFoo1);
		myFooHashMap->Insert(entryFoo2);
		HashMap< Foo , int, HashFoo >::HashMapIterator myFooIterator = myFooHashMap->Find(*myFoo1);
		TS_ASSERT_EQUALS((*myFooIterator).second, 10);
		myFooIterator = myFooHashMap->Find(*myFoo2);
		TS_ASSERT_EQUALS((*myFooIterator).second, 20);
		myFooHashMap->Clear();
		myFooIterator = myFooHashMap->Find(*myFoo2);
		TS_ASSERT_EQUALS(myFooIterator, myFooHashMap->End());
	}


	void testArrow()
	{
		// int 
		for(int i = 0; i < 10; i++){
			pair < int, int > entryInt ((i),(i * 2));
			myIntHashMap->Insert(entryInt);
		}
		HashMap< int, int, HashInt >::HashMapIterator myIntIterator = myIntHashMap->Find(7);
		TS_ASSERT_EQUALS(myIntIterator->first, 7);
		
		// char *
		myCharStringHashMap = new HashMap< char*, int , HashCharString >(4);
		string inventory[10] = {"Sword", "Armor", "Shield", "Potion", "Boots", "Helm", "Axe", "Dagger", "Food", "Junk"};
		for(int i = 0; i < 10; i++){
			pair < char*, int> entryCharString1((char*)inventory[i].c_str(), i);
			myCharStringHashMap->Insert(entryCharString1);
		}
		HashMap< char* , int, HashCharString >::HashMapIterator myCharPtrIterator = myCharStringHashMap->Begin();

		pair < char*, int> entryCharString1((char*)inventory[0].c_str() , i);
		myCharPtrIterator = myCharStringHashMap->Find((char*)inventory[0].c_str());
		TS_ASSERT_EQUALS(myCharPtrIterator->second, 0);

		HashMap< char* , int, HashCharString >::HashMapIterator myCharPtrIterator2 = myCharStringHashMap->Find( (char*)inventory[0].c_str() );
		TS_ASSERT_EQUALS((char*)inventory[0].c_str() , myCharPtrIterator2->first);
		
		myCharPtrIterator2 = myCharStringHashMap->Find( (char*)inventory[3].c_str() );
		TS_ASSERT_EQUALS((char*)inventory[3].c_str(), myCharPtrIterator2->first);

		delete myCharStringHashMap;
		myCharStringHashMap = new HashMap< char*, int , HashCharString >(4);
		delete myCharStringHashMap;

		// Foo
		pair < Foo, int > entryFoo1(*myFoo1, 10);
		pair < Foo, int > entryFoo2(*myFoo2, 20);
		myFooHashMap->Insert(entryFoo1);
		myFooHashMap->Insert(entryFoo2);
		HashMap< Foo , int, HashFoo >::HashMapIterator myFooIterator = myFooHashMap->Find(*myFoo1);
		TS_ASSERT_EQUALS(myFooIterator->second, 10);
		myFooIterator = myFooHashMap->Find(*myFoo2);
		TS_ASSERT_EQUALS((*myFooIterator).second, 20);
	}


	void testBracket()
	{
		// int 
		for(int i = 0; i < 10; i++){
			pair < int, int > entryInt ((i),(i * 2));
			myIntHashMap->Insert(entryInt);
		}
		TS_ASSERT_EQUALS(myIntHashMap->operator[](7), 14);

		
		// char *
		myCharStringHashMap = new HashMap< char*, int , HashCharString >(4);
		string inventory[10] = {"Sword", "Armor", "Shield", "Potion", "Boots", "Helm", "Axe", "Dagger", "Food", "Junk"};
		for(int i = 0; i < 10; i++){
			pair < char*, int> entryCharString1((char*)inventory[i].c_str(), i);
			myCharStringHashMap->Insert(entryCharString1);
		}
		int tempKey = myCharStringHashMap->operator[]( (char*)(inventory[0].c_str()) );
		TS_ASSERT_EQUALS( tempKey , 0 );


		// Foo
		pair < Foo, int > entryFoo1(*myFoo1, 10);
		pair < Foo, int > entryFoo2(*myFoo2, 20);
		myFooHashMap->Insert(entryFoo1);
		myFooHashMap->Insert(entryFoo2);
		TS_ASSERT_EQUALS(myFooHashMap->operator[](*myFoo1), 10);
	}

	void testIterator()
	{
		// Test skipping buckets
		for(int i = 0; i < 2; i++){
			pair < int, int > entryInt ((i + 2),(i * 2));
			myIntHashMap->Insert(entryInt);
		}
		HashMap< int, int, HashInt >::HashMapIterator myIntIterator = myIntHashMap->Begin();//Find(7);
		myIntIterator++;
		myIntIterator++;
		TS_ASSERT_EQUALS(myIntHashMap->End(), myIntIterator);


		// Test begin and end
		myCharStringHashMap = new HashMap< char* , int , HashCharString >(4);
		string inventory[10] = {"Sword", "Armor", "Shield", "Potion", "Boots", "Helm", "Axe", "Dagger", "Food", "Junk"};
		for(int i = 0; i < 10; i++){
			pair < char*, int> entryCharString1((char*)inventory[i].c_str(), i);
			myCharStringHashMap->Insert(entryCharString1);
		}
		HashMap< char* , int, HashCharString >::HashMapIterator myCharPtrIterator = myCharStringHashMap->Begin();
		while(myCharPtrIterator != myCharStringHashMap->End())
		{
			myCharPtrIterator++;
		}
	}

	void testClear()
	{
		// int 
		for(int i = 0; i < 2; i++){
			pair < int, int > entryInt ((i + 2),(i * 2));
			myIntHashMap->Insert(entryInt);
		}
		TS_ASSERT_EQUALS(myIntHashMap->Size(), 2);
		myIntHashMap->Clear();
		TS_ASSERT_EQUALS(myIntHashMap->Begin(), myIntHashMap->End());


		// char*
		myCharStringHashMap = new HashMap< char* , int , HashCharString >(4);
		string inventory[10] = {"Sword", "Armor", "Shield", "Potion", "Boots", "Helm", "Axe", "Dagger", "Food", "Junk"};
		for(int i = 0; i < 10; i++){
			pair < char*, int> entryCharString1((char*)inventory[i].c_str(), i);
			myCharStringHashMap->Insert(entryCharString1);
		}
		myCharStringHashMap->Clear();
		TS_ASSERT_EQUALS(myCharStringHashMap->Begin(), myCharStringHashMap->End());

		// Foo
		pair < Foo, int > entryFoo1(*myFoo1, 10);
		pair < Foo, int > entryFoo2(*myFoo2, 20);
		myFooHashMap->Insert(entryFoo1);
		myFooHashMap->Insert(entryFoo2);
		myFooHashMap->Clear();
		TS_ASSERT_EQUALS(myFooHashMap->Begin(), myFooHashMap->End());
	}


	void testRemove()
	{
		// int 
		for(int i = 0; i < 10; i++){
			pair < int, int > entryInt ((i),(i * 2));
			myIntHashMap->Insert(entryInt);
		}
		myIntHashMap->Remove(0);
		TS_ASSERT_EQUALS((*(myIntHashMap->Begin())).first, 1);

		
		// char *
		myCharStringHashMap = new HashMap< char*, int , HashCharString >(4);
		string inventory[10] = {"Sword", "Armor", "Shield", "Potion", "Boots", "Helm", "Axe", "Dagger", "Food", "Junk"};
		for(int i = 0; i < 10; i++){
			pair < char*, int> entryCharString1((char*)inventory[i].c_str(), i);
			myCharStringHashMap->Insert(entryCharString1);
		}
		TS_ASSERT_EQUALS((*(myCharStringHashMap->Begin())).second, 7);
		myCharStringHashMap->Remove( (*(myCharStringHashMap->Begin())).first );
		TS_ASSERT_EQUALS((*(myCharStringHashMap->Begin())).second, 5);
		delete myCharStringHashMap;

		// Foo
		pair < Foo, int > entryFoo1(*myFoo1, 10);
		pair < Foo, int > entryFoo2(*myFoo2, 20);
		myFooHashMap->Insert(entryFoo1);
		myFooHashMap->Insert(entryFoo2);
		myFooHashMap->Remove((*myFoo1));

		HashMap< Foo , int, HashFoo >::HashMapIterator myFooIterator = myFooHashMap->Begin();
		TS_ASSERT_EQUALS((*myFooIterator).second, 20);
	}

	void testDefaultHash()
	{
		//Test Default hash function
		HashMap<int, int> myIntHashMap2(30);

		for(int i = 0; i < 10; i++){
			pair < int, int > entryInt ((i),(i * 2));
			myIntHashMap2.Insert(entryInt);
		}
		
		HashMap< int, int>::HashMapIterator myIntIterator = myIntHashMap2.Find(7);
		TS_ASSERT_EQUALS((*myIntIterator).first, 7);
		myIntIterator = myIntHashMap2.Find(21);
		TS_ASSERT_EQUALS(myIntIterator, myIntHashMap2.End());

		//Test reinstert
		myIntIterator = myIntHashMap2.Find(0);
		TS_ASSERT_EQUALS((*myIntIterator).second, 0);
		pair < int, int > entryInt ((0),(3));
		myIntHashMap2.Insert(entryInt);
		TS_ASSERT_EQUALS((*(myIntHashMap2.Begin())).second, 0);
	}

};

#endif 