#ifndef __ACTIONTEST_H
#define __ACTIONTEST_H

#include <cxxtest/TestSuite.h>
#include "Vector.h"
#include "string.h"

#include "World.h"
#include "Action.h"
#include "ActionCreateAction.h"

//Parse
#include "XmlParseMaster.h"
#include "XmlParseHelperScope.h"
#include "XmlParseHelperEntity.h"
#include "XmlParseHelperAction.h"

//Reaction
#include "Reaction.h"
#include "ReactionAttributed.h"

//Event
#include "EventPublisher.h"
#include "IEventSubscriber.h"

//Universe
#include "Universe.h"

class ReactionEntityTest : public Entity
{
	RTTI_DECLARATIONS(ReactionEntityTest, Entity)


	ReactionEntityTest()
	{
		mySignatureInt.mName = ("My Int");
		mySignatureInt.mNumElements = 3;
		mySignatureInt.mType = Datum::TYPE_INT;
		mySignatureInt.mInit = new int(0);
		mSignatures.PushBack(mySignatureInt);

		mySignatureFloat.mName = ("My Float");
		mySignatureFloat.mNumElements = 3;
		mySignatureFloat.mType = Datum::TYPE_FLOAT;
		mySignatureFloat.mInit = new float(3.0f);
		mSignatures.PushBack(mySignatureFloat);

		mySignatureScope.mName = ("My Scope");
		mySignatureScope.mNumElements = 2;
		mySignatureScope.mType = Datum::TYPE_SCOPE;
		mySignatureScope.mInit = new Scope();
		mSignatures.PushBack(mySignatureScope);

		Populate();
	}

	virtual ~ReactionEntityTest()
	{
		delete mySignatureInt.mInit;
		delete mySignatureFloat.mInit;
		delete mySignatureScope.mInit;
	}

	private:
		Signature mySignatureInt;
		Signature mySignatureFloat;
		Signature mySignatureScope;

};
RTTI_DEFINITIONS(ReactionEntityTest)

EntityFactory( ReactionEntityTest )

/*!
*	A simple test suite: Just inherit CxxTest::TestSuite and write tests!
*/
class ReactionUnitTest : public CxxTest::TestSuite
{
public:

	void testReaction()
	{
		//Universe
		/**Universe myUniverse;

		//New World
		World * myWorld = new World();

		//Create a subscriber
		ReactionAttributed myReactionAttributedSubscriber;
		myReactionAttributedSubscriber

		//Enqueue the event
		myWorld->GetEventQueue().Enqueue(myReactionAttributedSubscriber.As<EventPublisher>(),0);
	
		delete myWorld;*/
	}

	void testReactionParse()
	{
		//Universe
		Universe myUniverse;

		//New World
		World * myWorld = new World();

		//Factories
		ReactionEntityTestFactory myPlayerFactory;

		//Shared Data
		XmlParseHelperScope::SharedDataScope* mySharedData = new XmlParseHelperScope::SharedDataScope(myWorld);

		//Master
		XmlParseMaster* myXMLParseMaster = new XmlParseMaster(mySharedData);

		//Helpers
		XmlParseHelperScope* myNewScopeHelper = new XmlParseHelperScope(myXMLParseMaster);
		XmlParseHelperEntity* myNewEntityHelper = new XmlParseHelperEntity(myXMLParseMaster);
		XmlParseHelperAction* myNewActionHelper = new XmlParseHelperAction(myXMLParseMaster);

		//Parse
		myXMLParseMaster->ParseFromFile("XML\\ReactionTest.xml");

		//Depth Test
		TS_ASSERT_EQUALS(myXMLParseMaster->GetSharedData()->GetDepth(), 0);
		//Path Test
		TS_ASSERT_EQUALS(myXMLParseMaster->GetFileName(), "XML\\ReactionTest.xml");

		//Depth Test
		TS_ASSERT_EQUALS(myXMLParseMaster->GetSharedData()->GetDepth(), 0);
		//Path Test
		TS_ASSERT_EQUALS(myXMLParseMaster->GetFileName(), "XML\\ReactionTest.xml");

		myWorld->GetWorldState().SetTime(1,1);
		myWorld->Process();

		myWorld->GetWorldState().SetTime(2,1);
		myWorld->Process();

		myWorld->GetWorldState().SetTime(3,1);
		myWorld->Process();

		TS_ASSERT_EQUALS(myWorld->GetWorldState().mCurrentTime, 100);
		TS_ASSERT_EQUALS(myWorld->GetWorldState().mElapsedTime, 100);
		
				
	
		delete myNewActionHelper;
		delete myNewEntityHelper;
		delete myNewScopeHelper;
		
		delete mySharedData;
		delete myXMLParseMaster;


		delete myWorld;
	}

};

#endif // __ACTIONTEST_H