#ifndef __SIMPLETEST_H
#define __SIMPLETEST_H

#include <cxxtest/TestSuite.h>
#include "SList.h"
#include "string.h"

/*!
*	A simple test suite: Just inherit CxxTest::TestSuite and write tests!
*/
class SListUnitTest : public CxxTest::TestSuite
{
public:
	/*!
		A simple class used in the unit tests
	*/
	class Foo
	{
		public:
			Foo() : mData(0) { }
			Foo(int data) : mData(data) { }
			int GetData() const { return(mData); }

			bool operator==(const Foo & otherFoo) const
			{
				if(GetData() == otherFoo.GetData())
					return(true);
				else
					return(false);
			}
		private:
			int mData;
	};

	void setUp() {
		
	}
    void tearDown() {
		
	}


	/*!
		Tests the PushFront function
	*/
    void testPushFront()
    {
		// int
		SList<int> mySListInt;
		TS_ASSERT_EQUALS(mySListInt.Size() , 0);
		mySListInt.PushFront(1);
		mySListInt.PushFront(1);
		mySListInt.PushFront(2);
		TS_ASSERT_EQUALS(mySListInt.Front() , 2);
		TS_ASSERT_EQUALS(mySListInt.Back() , 1);
		TS_ASSERT_EQUALS(mySListInt.Size() , 3);
		
		// int*
		SList<int*> mySListIntPtr;
		int* myIntPtr = new int(1);
		TS_ASSERT_EQUALS(mySListIntPtr.Size() , 0);
		mySListIntPtr.PushFront(myIntPtr);
		mySListIntPtr.PushFront(myIntPtr);
		mySListIntPtr.PushFront(myIntPtr);
		TS_ASSERT_EQUALS(mySListIntPtr.Size() , 3);
		TS_ASSERT_EQUALS( *mySListIntPtr.Front() , 1);
		TS_ASSERT_EQUALS( *mySListIntPtr.Back() , 1);
		delete myIntPtr;
		

		// Foo
		SList<Foo> mySListFoo;
		Foo myFoo;
		TS_ASSERT_EQUALS(mySListFoo.Size() , 0);
		mySListFoo.PushFront(myFoo);
		mySListFoo.PushFront(myFoo);
		mySListFoo.PushFront(myFoo);
		TS_ASSERT_EQUALS(mySListFoo.Size() , 3);
    }

	/*!
		Tests the PushBack function
	*/
    void testPushBack()
    {
		// int
		SList<int> mySListInt;
		TS_ASSERT_EQUALS(mySListInt.Size() , 0);
		mySListInt.PushBack(1);
		mySListInt.PushBack(1);
		mySListInt.PushBack(2);
		TS_ASSERT_EQUALS(mySListInt.Front() , 1);
		TS_ASSERT_EQUALS(mySListInt.Back() , 2);
		TS_ASSERT_EQUALS(mySListInt.Size() , 3);
		
		// int*
		SList<int*> mySListIntPtr;
		int* myIntPtr = new int(1);
		TS_ASSERT_EQUALS(mySListIntPtr.Size() , 0);
		mySListIntPtr.PushBack(myIntPtr);
		mySListIntPtr.PushBack(myIntPtr);
		mySListIntPtr.PushBack(myIntPtr);
		TS_ASSERT_EQUALS(mySListIntPtr.Size() , 3);
		TS_ASSERT_EQUALS( *mySListIntPtr.Front() , 1);
		TS_ASSERT_EQUALS( *mySListIntPtr.Back() , 1);
		delete myIntPtr;
		

		// Foo
		SList<Foo> mySListFoo;
		Foo myFoo;
		TS_ASSERT_EQUALS(mySListFoo.Size() , 0);
		mySListFoo.PushBack(myFoo);
		mySListFoo.PushBack(myFoo);
		mySListFoo.PushBack(myFoo);
		TS_ASSERT_EQUALS(mySListFoo.Size() , 3);
    }


	/*!
	*	Tests the size function
	*/
	void testSize()
	{
		//int
		SList<int> mySListInt;
		mySListInt.PushFront(1);
		mySListInt.PushFront(1);
		TS_ASSERT_EQUALS(mySListInt.Size() , 2);

		// int*
		SList<int*> mySListIntPtr;
		int* myIntPtr = new int(1);
		mySListIntPtr.PushFront(myIntPtr);
		mySListIntPtr.PushFront(myIntPtr);
		TS_ASSERT_EQUALS(mySListIntPtr.Size() , 2);
		delete myIntPtr;

		// Foo
		SList<Foo> mySListFoo;
		Foo myFoo;
		mySListFoo.PushFront(myFoo);
		mySListFoo.PushFront(myFoo);
		TS_ASSERT_EQUALS(mySListFoo.Size() , 2);
	}

	/*!
	*	Tests the assignment operator and the copy constructor
	*/
	void testCopy()
	{
		// int
		SList<int> mySListInt;
		SList<int> mySListInt3;
		mySListInt.PushFront(1);
		mySListInt.PushFront(2);
		SList<int>* mySListInt2 = new SList<int>(mySListInt);
		mySListInt3 = mySListInt;
		TS_ASSERT_EQUALS(mySListInt.Size() , mySListInt2->Size());
		TS_ASSERT_EQUALS(mySListInt.Size() , mySListInt3.Size());
		mySListInt.PopFront();
		TS_ASSERT_EQUALS(mySListInt.Size() , 1);
		TS_ASSERT_EQUALS(mySListInt3.Size() , 2);
		TS_ASSERT_EQUALS(2, mySListInt2->Size());
		delete mySListInt2;

		// int*
		int* myIntPtr = new int(1);
		SList<int*> mySListIntPtr;
		SList<int*> mySListIntPtr3;
		mySListIntPtr.PushFront(myIntPtr);
		mySListIntPtr.PushFront(myIntPtr);
		SList<int*>* mySListIntPtr2 = new SList<int*>(mySListIntPtr);
		mySListIntPtr3 = mySListIntPtr;
		TS_ASSERT_EQUALS(mySListIntPtr.Size() , mySListIntPtr2->Size());
		TS_ASSERT_EQUALS(mySListIntPtr.Size() , mySListIntPtr3.Size());
		mySListIntPtr.PopFront();
		TS_ASSERT_EQUALS(mySListIntPtr.Size() , 1);
		TS_ASSERT_EQUALS(mySListIntPtr3.Size() , 2);
		TS_ASSERT_EQUALS(2, mySListIntPtr2->Size());
		delete mySListIntPtr2;
		delete myIntPtr;

		// Foo
		Foo myFoo;
		SList<Foo> mySListFoo;
		SList<Foo> mySListFoo3;
		mySListFoo.PushFront(myFoo);
		mySListFoo.PushFront(myFoo);
		SList<Foo>* mySListFoo2 = new SList<Foo>(mySListFoo);
		mySListFoo3 = mySListFoo;
		TS_ASSERT_EQUALS(mySListFoo.Size() , mySListFoo2->Size());
		TS_ASSERT_EQUALS(mySListFoo.Size() , mySListFoo3.Size());
		mySListFoo.PopFront();
		TS_ASSERT_EQUALS(mySListFoo.Size() , 1);
		TS_ASSERT_EQUALS(mySListFoo3.Size() , 2);
		TS_ASSERT_EQUALS(2, mySListFoo2->Size());
		delete mySListFoo2;
	}

	/*!
		Tests the IsEmpty function
	*/
	void testIsEmpty()
	{
		// int*
		SList<int> mySListInt;
		TS_ASSERT_EQUALS(mySListInt.IsEmpty() , true);
		mySListInt.PushFront(1);
		TS_ASSERT_EQUALS(mySListInt.IsEmpty() , false);

		// int*
		int* myIntPtr = new int(1);
		SList<int*> mySListIntPtr;
		TS_ASSERT_EQUALS(mySListIntPtr.IsEmpty() , true);
		mySListIntPtr.PushFront(myIntPtr);
		TS_ASSERT_EQUALS(mySListIntPtr.IsEmpty() , false);
		delete myIntPtr;

		// Foo
		Foo myFoo;
		SList<Foo> mySListFoo;
		TS_ASSERT_EQUALS(mySListFoo.IsEmpty() , true);
		mySListFoo.PushFront(myFoo);
		TS_ASSERT_EQUALS(mySListFoo.IsEmpty() , false);
	}

	/*!
		Tests the Clear function
	*/
	void testClear()
	{
		// int
		SList<int> mySListInt;
		mySListInt.PushFront(1);
		mySListInt.PushFront(1);
		mySListInt.PushFront(1);
		mySListInt.PushFront(1);
		mySListInt.PushFront(1);
		mySListInt.PushFront(1);
		mySListInt.PushFront(1);
		mySListInt.Clear();
		TS_ASSERT_EQUALS(mySListInt.IsEmpty() , true);

		// int*
		SList<int*> mySListIntPtr;
		int* myIntPtr = new int(1);
		mySListIntPtr.PushFront(myIntPtr);
		mySListIntPtr.PushFront(myIntPtr);
		mySListIntPtr.PushFront(myIntPtr);
		mySListIntPtr.PushFront(myIntPtr);
		mySListIntPtr.PushFront(myIntPtr);
		mySListIntPtr.PushFront(myIntPtr);
		mySListIntPtr.PushFront(myIntPtr);
		mySListIntPtr.Clear();
		TS_ASSERT_EQUALS(mySListIntPtr.IsEmpty() , true);
		delete myIntPtr;

		// Foo
		SList<Foo> mySListFoo;
		Foo myFoo;
		mySListFoo.PushFront(myFoo);
		mySListFoo.PushFront(myFoo);
		mySListFoo.PushFront(myFoo);
		mySListFoo.PushFront(myFoo);
		mySListFoo.PushFront(myFoo);
		mySListFoo.PushFront(myFoo);
		mySListFoo.PushFront(myFoo);
		mySListFoo.Clear();
		TS_ASSERT_EQUALS(mySListFoo.IsEmpty() , true);
	}

	/*!
		Tests the PopFront function
	*/
	void testPopFront()
	{
		// int
		SList<int> mySListInt;
		mySListInt.PushFront(1);
		mySListInt.PushFront(2);
		mySListInt.PushFront(3);
		mySListInt.PushFront(1);
		TS_ASSERT_EQUALS(mySListInt.PopFront() , 1);
		TS_ASSERT_EQUALS(mySListInt.PopFront() , 3);
		TS_ASSERT_EQUALS(mySListInt.PopFront() , 2);
		TS_ASSERT_EQUALS(mySListInt.PopFront() , 1);
		//TS_ASSERT( mySListInt.PopFront() );
		
		TS_ASSERT_EQUALS(mySListInt.Size() , 0);

		// int*
		SList<int*> mySListIntPtr;
		int* myIntPtr = new int(1);
		mySListIntPtr.PushFront(myIntPtr);
		mySListIntPtr.PushFront(myIntPtr);
		mySListIntPtr.PushFront(myIntPtr);
		mySListIntPtr.PushFront(myIntPtr);
		TS_ASSERT_EQUALS(*mySListIntPtr.PopFront() , 1);
		TS_ASSERT_EQUALS(*mySListIntPtr.PopFront() , 1);
		TS_ASSERT_EQUALS(*mySListIntPtr.PopFront() , 1);
		TS_ASSERT_EQUALS(*mySListIntPtr.PopFront() , 1);
		TS_ASSERT_EQUALS(mySListIntPtr.Size() , 0);
		delete myIntPtr;

		// Foo
		SList<Foo> mySListFoo;
		Foo myFoo;
		mySListFoo.PushFront(myFoo);
		mySListFoo.PushFront(myFoo);
		mySListFoo.PushFront(myFoo);
		mySListFoo.PushFront(myFoo);
		mySListFoo.PopFront();
		TS_ASSERT_EQUALS(mySListFoo.Size() , 3);
	}

	void testBegin()
	{
		// int
		SList<int> mySListInt;
		mySListInt.PushFront(1);
		mySListInt.PushFront(2);
		mySListInt.PushFront(3);
		mySListInt.PushFront(4);
		mySListInt.PushFront(1);
		//mySListInt::SListIterator mI;
		SList<int>::SListIterator myIntIterator;
		myIntIterator = mySListInt.Begin();
		TS_ASSERT_EQUALS(*myIntIterator , mySListInt.Front());

		// int*
		SList<int*> mySListIntPtr;
		int* myIntPtr = new int(1);
		int* myIntPtr2 = new int(2);
		mySListIntPtr.PushFront(myIntPtr);
		mySListIntPtr.PushFront(myIntPtr);
		mySListIntPtr.PushFront(myIntPtr2);
		mySListIntPtr.PushFront(myIntPtr2);
		mySListIntPtr.PushFront(myIntPtr);
		SList<int*>::SListIterator myIntPtrIterator;
		myIntPtrIterator = mySListIntPtr.Begin();
		TS_ASSERT_EQUALS(*myIntPtrIterator , mySListIntPtr.Front());
		delete myIntPtr;

		// Foo
		SList<Foo> mySListFoo;
		Foo myFoo(1);
		Foo myFoo2(2);
		mySListFoo.PushFront(myFoo);
		mySListFoo.PushFront(myFoo2);
		SList<Foo>::SListIterator myFooIterator;
		myFooIterator = mySListFoo.Begin();
		TS_ASSERT_EQUALS(*myFooIterator , mySListFoo.Front());
	}


	void testInsertAfter()
	{
		// int
		SList<int> mySListInt;
		mySListInt.PushFront(1);

		SList<int>::SListIterator myIntIterator;
		myIntIterator = mySListInt.Begin();
		mySListInt.InsertAfter(2, myIntIterator);
		mySListInt.InsertAfter(3, myIntIterator);
		TS_ASSERT_EQUALS(*myIntIterator , mySListInt.Front());
		myIntIterator++;
		TS_ASSERT_EQUALS(*myIntIterator , 3);
		myIntIterator++;
		TS_ASSERT_EQUALS(*myIntIterator , 2);

		// int*
		SList<int*> mySListIntPtr;
		int* myIntPtr = new int(1);
		int* myIntPtr2 = new int(2);
		int* myIntPtr3 = new int(3);
		mySListIntPtr.PushFront(myIntPtr);
	
		SList<int*>::SListIterator myIntPtrIterator;
		myIntPtrIterator = mySListIntPtr.Begin();
		mySListIntPtr.InsertAfter( myIntPtr2, myIntPtrIterator);
		mySListIntPtr.InsertAfter( myIntPtr3, myIntPtrIterator);
		TS_ASSERT_EQUALS(*myIntPtrIterator , mySListIntPtr.Front());
		myIntPtrIterator++;
		TS_ASSERT_EQUALS(*myIntPtrIterator , myIntPtr3 );
		myIntPtrIterator++;
		TS_ASSERT_EQUALS(*myIntPtrIterator , myIntPtr2 );
		delete myIntPtr;
		delete myIntPtr2;
		delete myIntPtr3;

		// Foo
		SList<Foo> mySListFoo;
		Foo myFoo(1);
		Foo myFoo2(2);
		mySListFoo.PushFront(myFoo);

		SList<Foo>::SListIterator myFooIterator;
		myFooIterator = mySListFoo.Begin();
		mySListFoo.InsertAfter( myFoo2, myFooIterator);
		TS_ASSERT_EQUALS(*myFooIterator , mySListFoo.Front());
		myFooIterator++;
		TS_ASSERT_EQUALS(*myFooIterator , mySListFoo.Back());
	}

	/*!
	*	Tests the Find function
	*/
	void testFind()
	{
		// int
		SList<int> mySListInt;
		mySListInt.PushFront(1);
		mySListInt.PushFront(2);
		mySListInt.PushFront(3);
		mySListInt.PushFront(4);
		mySListInt.PushFront(1);
		SList<int>::SListIterator myIntIterator;
		myIntIterator = mySListInt.Find(2);
		TS_ASSERT_EQUALS(*myIntIterator , 2);

		// int*
		SList<int*> mySListIntPtr;
		int* myIntPtr = new int(1);
		int* myIntPtr2 = new int(2);
		mySListIntPtr.PushFront(myIntPtr);
		mySListIntPtr.PushFront(myIntPtr);
		mySListIntPtr.PushFront(myIntPtr2);
		mySListIntPtr.PushFront(myIntPtr2);
		mySListIntPtr.PushFront(myIntPtr);
		SList<int*>::SListIterator myIntPtrIterator;
		myIntPtrIterator = mySListIntPtr.Find(myIntPtr2);
		TS_ASSERT_EQUALS(*(*myIntPtrIterator) , *myIntPtr2);
		delete myIntPtr;

		// Foo
		SList<Foo> mySListFoo;
		Foo myFoo(1);
		Foo myFoo2(2);
		mySListFoo.PushFront(myFoo);
		mySListFoo.PushFront(myFoo2);
		SList<Foo>::SListIterator myFooIterator;
		myFooIterator = mySListFoo.Find(myFoo);
		TS_ASSERT_EQUALS(*myFooIterator , myFoo);
	}

	/*!
	*	Tests the Find function
	*/
	void testRemove()
	{
		// int
		SList<int> mySListInt;
		mySListInt.PushFront(1);
		mySListInt.PushFront(2);
		mySListInt.PushFront(3);
		mySListInt.PushFront(4);
		mySListInt.PushFront(1);
		SList<int>::SListIterator myIntIterator;
		mySListInt.Remove(2);
		SList<int>::SListIterator myIntIterator2;
		myIntIterator2 = mySListInt.Begin();
		TS_ASSERT_EQUALS(*myIntIterator2 , 1);
		myIntIterator2++;
		TS_ASSERT_EQUALS(*myIntIterator2 , 4);
		myIntIterator2++;
		TS_ASSERT_EQUALS(*myIntIterator2 , 3);
		myIntIterator2++;
		TS_ASSERT_EQUALS(*myIntIterator2 , 1);
		mySListInt.Remove(1);
		mySListInt.Remove(1);
		TS_ASSERT_EQUALS( mySListInt.Front(), 4);
		TS_ASSERT_EQUALS( mySListInt.Back(), 3);
		mySListInt.Remove(3);
		mySListInt.Remove(1);
		mySListInt.Remove(4);
		mySListInt.Remove(1);
		TS_ASSERT_EQUALS( mySListInt.Size(), 0);
		TS_ASSERT_EQUALS( mySListInt.Begin(), mySListInt.End());

		// int*
		SList<int*> mySListIntPtr;
		int* myIntPtr = new int(1);
		int* myIntPtr2 = new int(2);
		int* myIntPtr3;
		myIntPtr3 = myIntPtr2;
		mySListIntPtr.PushFront(myIntPtr);
		mySListIntPtr.PushFront(myIntPtr);
		mySListIntPtr.PushFront(myIntPtr2);
		mySListIntPtr.PushFront(myIntPtr2);
		mySListIntPtr.PushFront(myIntPtr);
		mySListIntPtr.Remove(myIntPtr);
		mySListIntPtr.Remove(myIntPtr);
		mySListIntPtr.Remove(myIntPtr);
		TS_ASSERT_EQUALS(mySListIntPtr.Front() , myIntPtr2);
		TS_ASSERT_EQUALS(mySListIntPtr.Back() , myIntPtr2);
		TS_ASSERT_EQUALS(mySListIntPtr.Size(), 2);
		delete myIntPtr;
		delete myIntPtr2;

		// Foo
		SList<Foo> mySListFoo;
		Foo myFoo(1);
		Foo myFoo2(2);
		mySListFoo.PushFront(myFoo);
		mySListFoo.PushFront(myFoo2);
		mySListFoo.Remove(myFoo);
		TS_ASSERT_EQUALS(mySListFoo.Front() , myFoo2);
		TS_ASSERT_EQUALS(mySListFoo.Back() , myFoo2);
		TS_ASSERT_EQUALS(mySListFoo.Size() , 1);
		mySListFoo.Remove(myFoo2);
		TS_ASSERT_EQUALS(mySListFoo.Size() , 0);
	}

};

#endif // __SIMPLETEST_H