#ifndef __SCOPETEST_H
#define __SCOPETEST_H

#include <cxxtest/TestSuite.h>
#include "Scope.h"

/*!
*	A simple test suite: Just inherit CxxTest::TestSuite and write tests!
*/
class ScopeUnitTest : public CxxTest::TestSuite
{
public:

	/*!
		A simple class used in the unit tests
	*/
	void testAppend()
	{
		TS_ASSERT_EQUALS(1, 1);
		Scope myScope;
		myScope.Append("FIRST!").SetType(Datum::TYPE_INT);
		myScope["Second"].SetType(Datum::TYPE_INT);
	}

	void testAssignment()
	{
		Scope myScope;
		myScope.Append("FIRST!").SetType(Datum::TYPE_INT);
		myScope["Second"].SetType(Datum::TYPE_INT);
		Datum* datumRef = myScope.Find("FIRST!");
		datumRef->Set(100);
		datumRef->Set(101,1);
		myScope.AppendScope("Scope1");
		myScope.AppendScope("Scope1",1);
		myScope.AppendScope("Scope1",2);

		TS_ASSERT_EQUALS(myScope.Find("FIRST!")->Size(), 2); 
		TS_ASSERT_EQUALS(((myScope.Find("Scope1"))->Size()),3);

		Scope myScope2;
		myScope2 = myScope;


		TS_ASSERT_EQUALS(myScope2.Find("FIRST!")->Size(), 2);
		TS_ASSERT_EQUALS(((myScope2.Find("Scope1"))->Size()),3);


		TS_ASSERT_EQUALS(true, (myScope2 == myScope));
		TS_ASSERT_EQUALS(myScope2.Find("FIRST!")->Size(), 2);

		myScope2.Find("FIRST!")->Set(100, 1);
		TS_ASSERT_EQUALS(false, (myScope2 == myScope));

		myScope2 = myScope;
		myScope2.AppendScope("My Scope");
		TS_ASSERT_EQUALS(false, (myScope2 == myScope));

		myScope2 = myScope;
		myScope2.Clear();
		TS_ASSERT_EQUALS(false, (myScope2 == myScope));

		Scope myScope3;
		myScope3.CopyValues(myScope, true);
		TS_ASSERT_EQUALS(true, (myScope3 == myScope));


		Scope myScope4 = myScope.AppendScope("Scope1").AppendScope("LOL");
		Scope myScope5;
		myScope5 = myScope4;
		TS_ASSERT_EQUALS(true, (myScope4 == myScope5));
	}

	void testReverseFind()
	{
		Scope myScope;
		myScope.AppendScope("Child"); // Create an entry of type scope
		std::string foundString;
		myScope.ReverseFind(myScope["Child"].Get<Datum::ScopeType>(), foundString);
		TS_ASSERT_EQUALS("Child", foundString);
	}


	void testAdopt()
	{
		Scope myScope;
		myScope.AppendScope("FIRST!"); // create an entry of type scope 
		myScope.AppendScope("FIRST!", 0); // get the scope that was just created
		myScope.AppendScope("FIRST!", 1); // get the scope that was just created

		myScope.Append("SECOND!"); // create a new entry

		myScope["Second"].SetType(Datum::TYPE_SCOPE);

		TS_ASSERT_EQUALS((NULL != myScope.Search("FIRST!")), true);
		

		TS_ASSERT_EQUALS(Datum::HasValue(*myScope.Find("FIRST!"), 1), true);
		Scope myScope2;
		myScope2.Adopt((myScope.Find("FIRST!")->Get<Datum::ScopeType>(0)), "NEW FIRST!", 0);
		TS_ASSERT_EQUALS(Datum::HasValue(*myScope.Find("FIRST!"), 1), false);
		TS_ASSERT_EQUALS(Datum::HasValue(*myScope2.Find("NEW FIRST!"), 0), true);
		TS_ASSERT_EQUALS(myScope2.Find("NEW FIRST!")->Size(), 1); // This still has 1 thing in it
		TS_ASSERT_EQUALS(myScope.Find("FIRST!")->Size(), 1); // this should have lost it's first child
	}
	
	void testCopyValues()
	{
		Scope myScope;
		myScope.AppendScope("FIRST!"); // create an entry of type scope 
		myScope.AppendScope("FIRST!", 0); // get the scope that was just created
		myScope.AppendScope("FIRST!", 1); // get the scope that was just created
		myScope.Append("SECOND!"); // create a new entry

		Scope myScope2;
		myScope2.AppendScope("THE REAL FIRST");
		myScope2.AppendScope("THE REAL FIRST", 1);
		myScope2.AppendScope("THE REAL FIRST", 2);
		myScope2.CopyValues(myScope);

		TS_ASSERT_EQUALS(myScope2.Size(), 3);
		TS_ASSERT_EQUALS(myScope2.Find("FIRST!")->Size(), 2);

		//Scope3 myScope3;
		//myScope3.CopyValues(myScope2);
	}
};

#endif // __SCOPETEST_H