#ifndef __STACKTEST_H
#define __STACKTEST_H

#include <cxxtest/TestSuite.h>
#include "Stack.h"
#include "string.h"

/*!
*	A simple test suite: Just inherit CxxTest::TestSuite and write tests!
*/
class StackUnitTest : public CxxTest::TestSuite
{
public:
	/*!
		A simple class used in the unit tests
	*/
	class Foo
	{
		public:
			Foo() : mData(0) { }

			Foo(int data) : mData(data) { }
			int GetData() const { return(mData); }

			bool operator==(const Foo & otherFoo) const
			{
				if(GetData() == otherFoo.GetData())
					return(true);
				else
					return(false);
			}

			void SetData(int newInt)
			{
				mData = newInt;
			}
		private:
			int mData;
	};

	Stack<int> myIntStack1;
	Stack<int> myIntStack2;

	Stack<int*> myPtrStack1;
	Stack<int*> myPtrStack2;
	int* myIntPtr1;
	int* myIntPtr2;
	int* myIntPtr3;
	
	Stack<Foo> myFooStack1;
	Stack<Foo> myFooStack2;
	Foo myFoo1;
	Foo myFoo2;
	Foo myFoo3;

	void setUp() {
		//int
		myIntStack1.Push(1);
		myIntStack1.Push(2);
		myIntStack1.Push(3);
		myIntStack1.Push(4);
		myIntStack1.Push(5);

		//int*
		myIntPtr1 = new int(1);
		myIntPtr2 = new int(2);
		myIntPtr3 = new int(3);
		myPtrStack1.Push(myIntPtr1);
		myPtrStack1.Push(myIntPtr2);

		//Foo
		myFoo1.SetData(1);
		myFoo2.SetData(2);
		myFoo3.SetData(3);
		myFooStack1.Push(myFoo1);
		myFooStack1.Push(myFoo2);
	}
    void tearDown() {
		//int
		myIntStack1.Clear();
		myIntStack2.Clear();

		//int*
		myPtrStack1.Clear();
		myPtrStack2.Clear();
		delete myIntPtr1;
		delete myIntPtr2;
		delete myIntPtr3;

		//Foo
		myFooStack1.Clear();
		myFooStack2.Clear();

		
	}

	/*!
	*	Tests the Push function
	*/
	void testPush()
	{
		//int
		myIntStack1.Push(6);
		TS_ASSERT_EQUALS(6, myIntStack1.Top());
		TS_ASSERT_EQUALS(6, myIntStack1.Size());
		myIntStack1.Push(7);
		TS_ASSERT_EQUALS(7, myIntStack1.Top());
		TS_ASSERT_EQUALS(7, myIntStack1.Size());
		myIntStack1.Push(8);
		TS_ASSERT_EQUALS(8, myIntStack1.Top());
		TS_ASSERT_EQUALS(8, myIntStack1.Size());
		
		//int*
		TS_ASSERT_EQUALS(2, myPtrStack1.Size());
		myPtrStack1.Push(myIntPtr3);
		TS_ASSERT_EQUALS(3, *(myPtrStack1.Top()));
		TS_ASSERT_EQUALS(3, myPtrStack1.Size());
		myPtrStack1.Push(myIntPtr2);
		TS_ASSERT_EQUALS(2, *(myPtrStack1.Top()));
		TS_ASSERT_EQUALS(4, myPtrStack1.Size());

		//Foo
		TS_ASSERT_EQUALS(2, myFooStack1.Size());
		myFooStack1.Push(myFoo3);
		TS_ASSERT_EQUALS(3, myFooStack1.Top().GetData());
		TS_ASSERT_EQUALS(3, myFooStack1.Size());
		myFooStack1.Push(myFoo2);
		TS_ASSERT_EQUALS(2, myFooStack1.Top().GetData());
		TS_ASSERT_EQUALS(4, myFooStack1.Size());
	}


	/*!
	*	Tests the Pop function
	*/
	void testPop()
	{
		//int
		myIntStack1.Pop();
		TS_ASSERT_EQUALS(4, myIntStack1.Top());
		TS_ASSERT_EQUALS(4, myIntStack1.Size());
		myIntStack1.Pop();
		TS_ASSERT_EQUALS(3, myIntStack1.Top());
		TS_ASSERT_EQUALS(3, myIntStack1.Size());
		myIntStack1.Pop();
		TS_ASSERT_EQUALS(2, myIntStack1.Top());
		TS_ASSERT_EQUALS(2, myIntStack1.Size());

		//int*
		TS_ASSERT_EQUALS(2, myPtrStack1.Size());
		myPtrStack1.Pop();
		TS_ASSERT_EQUALS(1, *(myPtrStack1.Top()));
		TS_ASSERT_EQUALS(1, myPtrStack1.Size());
		myPtrStack1.Pop();
		TS_ASSERT_EQUALS(0, myPtrStack1.Size());
		myPtrStack1.Pop();
		TS_ASSERT_EQUALS(0, myPtrStack1.Size());

		//Foo
		TS_ASSERT_EQUALS(2, myFooStack1.Size());
		myFooStack1.Pop();
		TS_ASSERT_EQUALS(1, myFooStack1.Top().GetData());
		TS_ASSERT_EQUALS(1, myFooStack1.Size());
		myFooStack1.Pop();
		TS_ASSERT_EQUALS(0, myFooStack1.Size());
		myFooStack1.Pop();
		TS_ASSERT_EQUALS(0, myFooStack1.Size());
	}

	/*!
	*	Tests the IsEmpty
	*/
	void testsIsEmpty()
	{
		//int
		TS_ASSERT_EQUALS(false, myIntStack1.IsEmpty());
		myIntStack1.Clear();
		myIntStack1.Pop();
		TS_ASSERT_EQUALS(0, myIntStack1.Size());
		TS_ASSERT_EQUALS(true, myIntStack1.IsEmpty());

		//int*
		TS_ASSERT_EQUALS(false, myPtrStack1.IsEmpty());
		myPtrStack1.Clear();
		myPtrStack1.Pop();
		TS_ASSERT_EQUALS(0, myPtrStack1.Size());
		TS_ASSERT_EQUALS(true, myPtrStack1.IsEmpty());

		//Foo
		TS_ASSERT_EQUALS(false, myFooStack1.IsEmpty());
		TS_ASSERT_EQUALS(2, myFooStack1.Size());
		myFooStack1.Clear();
		myFooStack1.Pop();
		TS_ASSERT_EQUALS(0, myFooStack1.Size());
		TS_ASSERT_EQUALS(true,  myFooStack1.IsEmpty());
	}

	/*!
	*	Tests the Clear
	*/
	void testClear()
	{
		//int
		TS_ASSERT_EQUALS(5, myIntStack1.Size());
		myIntStack1.Clear();
		myIntStack1.Pop();
		TS_ASSERT_EQUALS(0, myIntStack1.Size());

		//int*
		TS_ASSERT_EQUALS(2, myPtrStack1.Size());
		myPtrStack1.Clear();
		myPtrStack1.Pop();
		TS_ASSERT_EQUALS(0, myPtrStack1.Size());

		//Foo
		TS_ASSERT_EQUALS(2, myFooStack1.Size());
		myFooStack1.Clear();
		myFooStack1.Pop();
		TS_ASSERT_EQUALS(0, myFooStack1.Size());
	}

	/*!
	*	Tests the Copy Constructor and the assignment operator
	*/
	void testCopy()
	{
		//int
		myIntStack2 = myIntStack1;
		TS_ASSERT_EQUALS(myIntStack2.Size(), myIntStack1.Size());
		while(myIntStack2.IsEmpty() == false)
		{
			TS_ASSERT_EQUALS(myIntStack2.Top(), myIntStack1.Top());
			myIntStack1.Pop();
			TS_ASSERT_DIFFERS(myIntStack1.Size(), myIntStack2.Size());
			myIntStack2.Pop();

		}

		//int*
		myPtrStack2 = myPtrStack1;
		TS_ASSERT_EQUALS(myPtrStack2.Size(), myPtrStack1.Size());
		while(myPtrStack2.IsEmpty() == false)
		{
			TS_ASSERT_EQUALS(*myPtrStack2.Top(), *myPtrStack1.Top());
			myPtrStack1.Pop();
			TS_ASSERT_DIFFERS(myPtrStack1.Size(), myPtrStack2.Size());
			myPtrStack2.Pop();

		}

		//Foo
		myFooStack2 = myFooStack1;
		TS_ASSERT_EQUALS(myFooStack2.Size(), myFooStack1.Size());
		while(myFooStack2.IsEmpty() == false)
		{
			TS_ASSERT_EQUALS(myFooStack2.Top(), myFooStack1.Top());
			myFooStack1.Pop();
			TS_ASSERT_DIFFERS(myFooStack1.Size(), myFooStack2.Size());
			myFooStack2.Pop();

		}
	}
};

#endif // __STACKTEST_H