#ifndef __VECTORTEST_H
#define __VECTORTEST_H

#include <cxxtest/TestSuite.h>
#include "Vector.h"
#include "string.h"

/*!
*	A simple test suite: Just inherit CxxTest::TestSuite and write tests!
*/
class VectorUnitTest : public CxxTest::TestSuite
{
public:
	/*!
		A simple class used in the unit tests
	*/
	class Foo
	{
		public:
			Foo() : mData(0) { }

			Foo(int* data) : mData(data) { }
			int* GetData() const { return(mData); }

			bool operator==(const Foo & otherFoo) const
			{
				if((GetData()) && (otherFoo.GetData())){

					if((GetData()) == (otherFoo.GetData()))
						return(true);
					else
						return(false);
				}
				return(false);
			}
		private:
			int* mData;
	};

	void setUp() {
		
	}
    void tearDown() {
		
	}

	/*!
	*	Tests the PushBack function
	*/
	void testPushBack()
	{
		//int
		Vector<int> myVector;
		Vector<int> myVector2;
		TS_ASSERT_EQUALS(4 , 4);
		myVector.PushBack(1);
		myVector.PushBack(2);
	    myVector.PushBack(3);
		myVector.PushBack(4);
		myVector.PushBack(5);
		myVector.PushBack(6);
		myVector.PushBack(7);
		myVector.PopBack();
		TS_ASSERT_EQUALS(myVector[0], 1);
		TS_ASSERT_EQUALS(myVector[1], 2);
		TS_ASSERT_EQUALS(myVector[2], 3);
		TS_ASSERT_EQUALS(myVector.Size(), 6);
		TS_ASSERT_EQUALS(myVector.At(2), 3);
		myVector2 = myVector;
		for(unsigned int i = 0; i < myVector2.Size(); i++){
			TS_ASSERT_EQUALS(myVector.At(i), myVector2[i]);
		}
		myVector.PopBack();
		myVector.PopBack();
		myVector.PopBack();
		myVector.PopBack();
		myVector.PopBack();
		myVector.PopBack();
		myVector.PushBack(3);
		myVector.PushBack(4);
		myVector.PushBack(5);
		myVector.PushBack(6);
		myVector.PushBack(7);

		//int*
		int* myInt1 = new int(1);
		int* myInt2 = new int(2);
		int* myInt3 = new int(3);
		int* myInt4 = new int(4);
		int* myInt5 = new int(5);
		int* myInt6 = new int(6);
		Vector<int*> myVectorPtr;
		myVectorPtr.PushBack(myInt1);
		myVectorPtr.PushBack(myInt2);
		myVectorPtr.PushBack(myInt3);
		myVectorPtr.PushBack(myInt4);
		myVectorPtr.PushBack(myInt5);
		myVectorPtr.PushBack(myInt6);
		myVectorPtr.PopBack();
		TS_ASSERT_EQUALS(myVectorPtr.Size(), 5);

		TS_ASSERT_EQUALS((*myVectorPtr.Front()), 1);
		TS_ASSERT_EQUALS((*myVectorPtr.Back()), 5);
		Vector<int*>::VectorIterator myIntPtrIterator = myVectorPtr.Begin();
		Vector<int*>::VectorIterator myIntPtrIterator2 = myIntPtrIterator;
		Vector<int*>::VectorIterator myIntPtrIterator3(myIntPtrIterator);
		myIntPtrIterator3++;
		TS_ASSERT_EQUALS(*(*myIntPtrIterator), 1);
		TS_ASSERT_EQUALS(*(*myIntPtrIterator3), 2);

		//Foo
		Vector<Foo> myFooVector;
		Foo myFoo1(myInt1);
		Foo myFoo2(myInt2);
		myFooVector.PushBack(myFoo1);
		myFooVector.PushBack(myFoo2);
	}

	/*!
	*	Tests the PopBack function
	*/
	void testPopBack()
	{
		//int
		Vector<int> myVector;
		Vector<int> myVector2;
		TS_ASSERT_EQUALS(4 , 4);
		myVector.PushBack(1);
		myVector.PushBack(2);
	    myVector.PushBack(3);
		myVector.PushBack(4);
		myVector.PushBack(5);
		myVector.PushBack(6);
		myVector.PushBack(7);
		myVector.PopBack();
		myVector.PopBack();
		myVector.PopBack();
		myVector.PopBack();
		myVector.PopBack();
		TS_ASSERT_EQUALS(myVector[0], 1);
		TS_ASSERT_EQUALS(myVector[1], 2);
		TS_ASSERT_EQUALS(myVector.Size(), 2);
		TS_ASSERT_EQUALS(myVector.At(1), 2);
		myVector.PushBack(3);
		myVector.PushBack(4);
		myVector.PushBack(5);
		myVector.PushBack(6);
		myVector.PushBack(7);
		myVector.PushBack(8);
		myVector.Find(9);

		//int*
		int* myInt1 = new int(1);
		int* myInt2 = new int(2);
		int* myInt3 = new int(3);
		int* myInt4 = new int(4);
		int* myInt5 = new int(5);
		int* myInt6 = new int(6);
		Vector<int*> myVectorPtr;
		myVectorPtr.PushBack(myInt1);
		myVectorPtr.PushBack(myInt2);
		myVectorPtr.PushBack(myInt3);
		myVectorPtr.PushBack(myInt4);
		myVectorPtr.PushBack(myInt5);
		myVectorPtr.PushBack(myInt6);
		myVectorPtr.PopBack();
		myVectorPtr.PopBack();
		myVectorPtr.PopBack();
		TS_ASSERT_EQUALS(myVectorPtr.Size(), 3);
		TS_ASSERT_EQUALS((*myVectorPtr.Front()), 1);
		TS_ASSERT_EQUALS((*myVectorPtr.Back()), 3);
		Vector<int*>::VectorIterator myIntPtrIterator = myVectorPtr.Begin();
		Vector<int*>::VectorIterator myIntPtrIterator2 = myIntPtrIterator;
		Vector<int*>::VectorIterator myIntPtrIterator3(myIntPtrIterator);
		myIntPtrIterator3++;
		TS_ASSERT_EQUALS(*(*myIntPtrIterator), 1);
		TS_ASSERT_EQUALS(*(*myIntPtrIterator3), 2);

		//Foo
		Vector<Foo> myFooVector;
		Foo myFoo1(myInt1);
		Foo myFoo2(myInt2);
		myFooVector.PushBack(myFoo1);
		myFooVector.PushBack(myFoo2);
		TS_ASSERT_EQUALS(myFooVector.Size(), 2);
		myFooVector.PopBack();
		TS_ASSERT_EQUALS(myFooVector.Size(), 1);
		myFooVector.PopBack();
		TS_ASSERT_EQUALS(myFooVector.Size(), 0);
	}

	/*!
	*	Tests the Copy function
	*/
	void testCopy()
	{
		//int
		Vector<int> myVector;
		Vector<int> myVector2;
		TS_ASSERT_EQUALS(4 , 4);
		myVector.PushBack(1);
		myVector.PushBack(2);
	    myVector.PushBack(3);
		myVector.PushBack(4);
		myVector.PushBack(5);
		myVector.PushBack(6);
		myVector.PushBack(7);
		myVector.PopBack();
		TS_ASSERT_EQUALS(myVector[0], 1);
		TS_ASSERT_EQUALS(myVector[1], 2);
		TS_ASSERT_EQUALS(myVector[2], 3);
		TS_ASSERT_EQUALS(myVector.Size(), 6);
		TS_ASSERT_EQUALS(myVector.At(2), 3);
		myVector2 = myVector;
		for(unsigned int i = 0; i < myVector2.Size(); i++){
			TS_ASSERT_EQUALS(myVector.At(i), myVector2[i]);
		}

		//int*
		int* myInt1 = new int(1);
		int* myInt2 = new int(2);
		int* myInt3 = new int(3);
		int* myInt4 = new int(4);
		int* myInt5 = new int(5);
		int* myInt6 = new int(6);
		Vector<int*> myVectorPtr;
		myVectorPtr.PushBack(myInt1);
		myVectorPtr.PushBack(myInt2);
		myVectorPtr.PushBack(myInt3);
		myVectorPtr.PushBack(myInt4);
		myVectorPtr.PushBack(myInt5);
		myVectorPtr.PushBack(myInt6);
		myVectorPtr.PopBack();
		Vector<int*>* myVectorPtr2 = new Vector<int*>(myVectorPtr);
		for(unsigned int i = 0; i < (*myVectorPtr2).Size(); i++){
			TS_ASSERT_EQUALS((myVectorPtr.At(i)), (*myVectorPtr2)[i]);
		}
		TS_ASSERT_EQUALS(myVectorPtr.Size(), 5);
		TS_ASSERT_EQUALS((*myVectorPtr.Front()), 1);
		TS_ASSERT_EQUALS((*myVectorPtr.Back()), 5);
		Vector<int*>::VectorIterator myIntPtrIterator = myVectorPtr.Begin();
		Vector<int*>::VectorIterator myIntPtrIterator2 = myIntPtrIterator;
		Vector<int*>::VectorIterator myIntPtrIterator3(myIntPtrIterator);
		myIntPtrIterator3++;
		TS_ASSERT_EQUALS(*(*myIntPtrIterator), 1);
		TS_ASSERT_EQUALS(*(*myIntPtrIterator3), 2);

		//Foo
		Vector<Foo> myFooVector;
		Foo myFoo1(myInt1);
		Foo myFoo2(myInt1);
		myFooVector.PushBack(myFoo1);
		myFooVector.PushBack(myFoo2);
		Vector<Foo> myFooVector2 = myFooVector;

		Foo myFoo3;
		Vector<Foo>* myFooVector3 = new Vector<Foo>();
		myFooVector3->PushBack(myFoo3);
		delete myFooVector3;
	}

	/*!
	*	Tests the Find function
	*/
	void testFind()
	{
		//int
		Vector<int> myVector;
		Vector<int> myVector2;
		TS_ASSERT_EQUALS(4 , 4);
		myVector.PushBack(1);
		myVector.PushBack(2);
	    myVector.PushBack(3);
		myVector.PushBack(4);
		myVector.PushBack(5);
		myVector.PushBack(6);
		myVector.PushBack(7);
		myVector.PopBack();
		Vector<int>::VectorIterator myIntIterator = myVector.Find(4);
		TS_ASSERT_EQUALS(*myIntIterator, 4);
		myIntIterator = myVector.Find(9);
		TS_ASSERT_EQUALS(myIntIterator, myVector.End());
		TS_ASSERT_EQUALS(myVector[0], 1);
		TS_ASSERT_EQUALS(myVector[1], 2);
		TS_ASSERT_EQUALS(myVector[2], 3);
		TS_ASSERT_EQUALS(myVector.Size(), 6);
		TS_ASSERT_EQUALS(myVector.At(2), 3);
		myVector2 = myVector;
		for(unsigned int i = 0; i < myVector2.Size(); i++){
			TS_ASSERT_EQUALS(myVector.At(i), myVector2[i]);
		}

		//int*
		int* myInt1 = new int(1);
		int* myInt2 = new int(2);
		int* myInt3 = new int(3);
		int* myInt4 = new int(4);
		int* myInt5 = new int(5);
		int* myInt6 = new int(6);
		Vector<int*> myVectorPtr;
		myVectorPtr.PushBack(myInt1);
		myVectorPtr.PushBack(myInt2);
		myVectorPtr.PushBack(myInt3);
		myVectorPtr.PushBack(myInt4);
		myVectorPtr.PushBack(myInt5);
		myVectorPtr.PushBack(myInt6);
		myVectorPtr.PopBack();
		Vector<int*>* myVectorPtr2 = new Vector<int*>(myVectorPtr);
		for(unsigned int i = 0; i < (*myVectorPtr2).Size(); i++){
			TS_ASSERT_EQUALS((myVectorPtr.At(i)), (*myVectorPtr2)[i]);
		}
		Vector<int*>::VectorIterator myIntPtrIterator4 = myVectorPtr.Find(myInt4);
		TS_ASSERT_EQUALS(*myIntPtrIterator4, myInt4);
		myIntPtrIterator4 = myVectorPtr.Find(myInt6);
		TS_ASSERT_EQUALS(myIntPtrIterator4, myVectorPtr.End());
		TS_ASSERT_EQUALS(myVectorPtr.Size(), 5);
		TS_ASSERT_EQUALS((*myVectorPtr.Front()), 1);
		TS_ASSERT_EQUALS((*myVectorPtr.Back()), 5);
		Vector<int*>::VectorIterator myIntPtrIterator = myVectorPtr.Begin();
		Vector<int*>::VectorIterator myIntPtrIterator2 = myIntPtrIterator;
		Vector<int*>::VectorIterator myIntPtrIterator3(myIntPtrIterator);
		myIntPtrIterator3++;
		TS_ASSERT_EQUALS(*(*myIntPtrIterator), 1);
		TS_ASSERT_EQUALS(*(*myIntPtrIterator3), 2);

		//Foo
		Vector<Foo> myFooVector;
		Foo myFoo1(myInt1);
		Foo myFoo2(myInt2);
		myFooVector.PushBack(myFoo1);
		myFooVector.PushBack(myFoo2);
		Vector<Foo> myFooVector2 = myFooVector;
		Vector<Foo>::VectorIterator myFooIterator = myFooVector.Find(myFoo1);
		TS_ASSERT_EQUALS(myFoo1, *myFooIterator);
		TS_ASSERT_EQUALS(myFooVector2[0], myFooVector[0]);
	}

	/*!
	*	Tests the Size function
	*/
	void testSize()
	{
		//int
		Vector<int> myVector;
		Vector<int> myVector2;
		TS_ASSERT_EQUALS(4 , 4);
		TS_ASSERT_EQUALS(myVector.Size(), 0);
		myVector.PushBack(1);
		myVector.PushBack(2);
	    myVector.PushBack(3);
		myVector.PushBack(4);
		myVector.PushBack(5);
		myVector.PushBack(6);
		myVector.PushBack(7);
		myVector.PopBack();
		TS_ASSERT_EQUALS(myVector.Size(), 6);
		myVector2 = myVector;
		
		//int*
		int* myInt1 = new int(1);
		int* myInt2 = new int(2);
		int* myInt3 = new int(3);
		int* myInt4 = new int(4);
		int* myInt5 = new int(5);
		int* myInt6 = new int(6);
		Vector<int*> myVectorPtr;
		myVectorPtr.PushBack(myInt1);
		TS_ASSERT_EQUALS(myVectorPtr.Size(), 1);
		myVectorPtr.PushBack(myInt2);
		myVectorPtr.PushBack(myInt3);
		myVectorPtr.PushBack(myInt4);
		myVectorPtr.PushBack(myInt5);
		myVectorPtr.PushBack(myInt6);
		TS_ASSERT_EQUALS(myVectorPtr.Size(), 6);
		myVectorPtr.PopBack();
		TS_ASSERT_EQUALS(myVectorPtr.Size(), 5);
		delete myInt1;
		delete myInt2;
		delete myInt3;
		delete myInt4;
		delete myInt5;
		delete myInt6;

		//Foo
		Vector<Foo> myFooVector;
		Foo myFoo1(myInt1);
		Foo myFoo2(myInt2);
		myFooVector.PushBack(myFoo1);
		myFooVector.PushBack(myFoo2);
		Vector<Foo>::VectorIterator myFooIterator = myFooVector.Find(myFoo1);
		TS_ASSERT_EQUALS(myFoo1, *myFooIterator);
		TS_ASSERT_EQUALS(myFooVector.Size(), 2);
	}


	void testResize()
	{
		//int
		Vector<int> myVector;
		Vector<int> myVector2;
		TS_ASSERT_EQUALS(myVector.Size(), 0);
		myVector.PushBack(1);
		myVector.PushBack(2);
	    myVector.PushBack(3);
		myVector.PushBack(4);
		myVector.PushBack(5);
		myVector.PushBack(6);
		myVector.PushBack(7);
		myVector.PopBack();
		TS_ASSERT_EQUALS(myVector.Size(), 6);
		myVector.Resize(1);
		TS_ASSERT_EQUALS(myVector.Size(), 1);


		//int*
		int* myInt1 = new int(1);
		int* myInt2 = new int(2);
		int* myInt3 = new int(3);
		int* myInt4 = new int(4);
		int* myInt5 = new int(5);
		int* myInt6 = new int(6);
		Vector<int*> myVectorPtr;
		myVectorPtr.PushBack(myInt1);
		TS_ASSERT_EQUALS(myVectorPtr.Size(), 1);
		myVectorPtr.PushBack(myInt2);
		myVectorPtr.PushBack(myInt3);
		myVectorPtr.PushBack(myInt4);
		myVectorPtr.PushBack(myInt5);
		myVectorPtr.PushBack(myInt6);
		TS_ASSERT_EQUALS(myVectorPtr.Size(), 6);
		myVectorPtr.Resize(1);
		TS_ASSERT_EQUALS(myVectorPtr.Size(), 1);
		TS_ASSERT_EQUALS(*myInt6, 6);
		TS_ASSERT_EQUALS(*myInt5, 5);
		TS_ASSERT_EQUALS(*myInt4, 4);
		TS_ASSERT_EQUALS(*myInt3, 3);
		TS_ASSERT_EQUALS(*myInt2, 2);
		delete myInt1;
		delete myInt2;
		delete myInt3;
		delete myInt4;
		delete myInt5;
		delete myInt6;


		//Foo
		Vector<Foo> myFooVector;
		Foo myFoo1(myInt1);
		Foo myFoo2(myInt2);
		myFooVector.PushBack(myFoo1);
		myFooVector.PushBack(myFoo2);
		Vector<Foo>::VectorIterator myFooIterator = myFooVector.Find(myFoo1);
		TS_ASSERT_EQUALS(myFoo1, *myFooIterator);
		TS_ASSERT_EQUALS(myFooVector.Size(), 2);
		myFooVector.Resize(0);
		TS_ASSERT_EQUALS(myFooVector.Size(), 0);
		TS_ASSERT_EQUALS(myFoo1, myFoo1);
	}
};

#endif // __VECTORTEST_H