#ifndef __XMLPARSEMASTERTEST_H
#define __XMLPARSEMASTERTEST_H

#include <cxxtest/TestSuite.h>
#include "Vector.h"
#include "string.h"
#include "util.h"

#include <iostream>


#include "XmlParseMaster.h"
#include "IXmlParseHelper.h"
#include "XmlParseHelperConcrete.h"
#include "XmlParseHelperScope.h"




using namespace std;

/*!
*	A simple test suite: Just inherit CxxTest::TestSuite and write tests!
*/
class XMLParseMasterUnitTest : public CxxTest::TestSuite
{
public:
	/*!
		A simple class used in the unit tests
	*/

	void setUp()
	{
		
	}
    void tearDown()
	{
		
	}

	void testXML()
	{
		XmlParseHelperConcrete::SharedDataConcrete* mySharedData = new XmlParseHelperConcrete::SharedDataConcrete();
		XmlParseMaster* myXMLParseMaster = new XmlParseMaster(mySharedData);

		XmlParseHelperConcrete* myNewHelper = new XmlParseHelperConcrete(myXMLParseMaster);

		myXMLParseMaster->ParseFromFile("XML\\PathCameraTest.xml");

		//Depth Test
		TS_ASSERT_EQUALS(myXMLParseMaster->GetSharedData()->GetDepth(), 0);
		//Path Test
		TS_ASSERT_EQUALS(myXMLParseMaster->GetFileName(), "XML\\PathCameraTest.xml");
		// Clone Test
		XmlParseMaster* myClonedXMLParseMaster = myXMLParseMaster->Create();
		TS_ASSERT_EQUALS(myClonedXMLParseMaster->GetSharedData() != myXMLParseMaster->GetSharedData(), false);

		//Content Test
		RemoveWhiteSpace(mySharedData->xmlResults);
		
		TS_ASSERT_EQUALS(mySharedData->xmlResults.size() > 0, true);
		char** buffer = new char*();
		FileLoad(buffer, 	myXMLParseMaster->GetFileName().c_str());
		std::string myFileContents(*buffer);
		RemoveWhiteSpace(myFileContents);
		TS_ASSERT_EQUALS(mySharedData->xmlResults, myFileContents);

		delete myNewHelper;
		delete mySharedData;
		delete myXMLParseMaster;
		delete myClonedXMLParseMaster;
	}

	void testScopeHelper()
	{
		XmlParseHelperScope::SharedDataScope* mySharedData = new XmlParseHelperScope::SharedDataScope();
		XmlParseMaster* myXMLParseMaster = new XmlParseMaster(mySharedData);

		XmlParseHelperScope* myNewHelper = new XmlParseHelperScope(myXMLParseMaster);

		myXMLParseMaster->ParseFromFile("XML\\ScopeTest.xml");

		//Depth Test
		TS_ASSERT_EQUALS(myXMLParseMaster->GetSharedData()->GetDepth(), 0);
		//Path Test
		TS_ASSERT_EQUALS(myXMLParseMaster->GetFileName(), "XML\\ScopeTest.xml");
		
		// Scope Test
		Scope globalScope = mySharedData->GetGlobalScope();

		//First Set
		Scope firstScope = globalScope.AppendScope("FirstScope"); 
		Datum firstInt = firstScope.Append("FirstInt");
		Datum firstVector = firstScope.Append("FirstVector");
		Datum firstMatrix = firstScope.Append("FirstMatrix");
		//Inner Set
		Scope innerScope = firstScope.AppendScope("InnerScope");
		Datum innerInt = innerScope.Append("InnerInt");
		Datum innerFloat = innerScope.Append("InnerFloat");
		Datum innerString = innerScope.Append("InnerString");


		TS_ASSERT_EQUALS(firstInt.Get<int>(), 1);
		TS_ASSERT_EQUALS(firstInt.Get<int>(1), 2);
		TS_ASSERT_EQUALS(firstInt.Get<int>(2), 3);


		TS_ASSERT_EQUALS(innerInt.Get<int>(), 1);
		TS_ASSERT_EQUALS(innerInt.Get<int>(1), 2);
		TS_ASSERT_EQUALS(innerFloat.Get<float>(), 1.5f);
		TS_ASSERT_EQUALS(innerString.Get<std::string>(), "Stuff");


		Vec4 myVector(3.9f, 1.5f, 2.1f, 1.0f);
		TS_ASSERT_EQUALS(myVector,  firstVector.Get<Vec4>(0));

		Mat4 myMatrix(1.0f,2.0f,3.0f,4.0f,5.0f,6.0f,7.0f,8.0f,9.0f,10.0f,11.0f,12.0f,13.0f,14.0f,15.0f,16.0f);
		TS_ASSERT_EQUALS(myMatrix,  firstMatrix.Get<Mat4>());

		delete myNewHelper;
		delete mySharedData;
		delete myXMLParseMaster;
	}
};

#endif // __XMLPARSEMASTERTEST_H